/*

args_4: Command line arguments parsing library
Yauheni A. Akhotnikau (C) 2002-2003
eao197@yahoo.com
-------------------------------------------------

Permission is granted to anyone to use this software for any purpose on any
computer system, and to redistribute it freely, subject to the following
restrictions:

1. This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

2. The origin of this software must not be misrepresented, either by
   explicit claim or by omission.

3. Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

-------------------------------------------------

*/
/*!
	Args-4
*/

/*!
	\file
	\brief	   args-4.
*/

#ifndef _ARGS_4__ARGS_HPP_
#define _ARGS_4__ARGS_HPP_

#include <cstdio>
#include <cstddef>


//  Visual C++ 6.0   cstdio  cstddef 
//    std.   Borland C++
// ,      size_t  
//   std.
namespace std {}
using namespace std;

namespace args_4
{

/*
**    .
*/

//!  .
const int c_ok = 0;

//!    .
const int c_eof = 501;

//!   .
const int c_unknown_cmd = 502;

//!    .
const int c_mandatory_cmd_missed = 503;

//!      .
/*!
	:
\verbatim
"some string starts...
end finished here"
\endverbatim
	 .  :
\verbatim
"some string starts...\
end finished here"
\endverbatim
*/
const int c_new_line_in_string = 504;

//!  .
const int c_empty_string = 505;

//!     .
/*!
	 <tt>wor"d"</tt>  .
	 <tt>wor "d"</tt>.
*/
const int c_quote_in_word = 506;

//!     .
const int c_invalid_back_slash_seq = 507;

//!   .
const int c_cmd_already_defined = 508;

//!    .
const int c_value_missed = 509;

//!  .
const int c_invalid_value = 510;

//!  .
const int c_no_cmd = 511;

//!  -.
const int c_io_error = 512;

/*
** error_stream_t
*/

//!     .
/*!
	 args-4  , 
	    
	  (stdsn, ostream  ..).  ,
	     args-4
	   - 
	 (,   ,   
	   multiline-edit) 
	     
	  error_stream_t.

	      null-,
	.. ,    -
	.
*/
class	error_stream_t {
	public :
		error_stream_t();
		virtual ~error_stream_t();

		//!     .
		/*!
			     .
		*/
		virtual error_stream_t &
		out( const char * str );

	private :
		error_stream_t(
			const error_stream_t & );
		error_stream_t &
		operator=( const error_stream_t & );
};

/*
** word_container_t
*/

/*!
	\brief     .

	  args-4  
	    (
	 ).

	     
	  (stdin, istream, FILE *, pipe,
	,     ..).   
	       ,
	  word_container_t.

	 ,  word_container_t 
	    
	     . 
	    
	word_container_t::getch().
*/
class	word_container_t {
	public :
		word_container_t();
		virtual	~word_container_t( void );

		//!     .
		/*!
			\return   C-
			   ,
			   . 
			   
			  next().   
			 0.
		*/
		virtual	const char *
		next(
			//!   ,  
			//!  .
			int & error_code,
			//!      
			//! .    0 --  
			//!      
			//! .
			error_stream_t * error_stream );

		//!    
		//!   .
		/*!
			  next()  
			.

			   .. lock-ahead
			 .
		*/
		void
		back();

	protected :
		//!      .
		char *	m_word;
		//!   %m_word.
		size_t	m_word_capacity;
		//!     %m_word.
		size_t	m_word_len;

		//!   .
		/*!       next(). */
		error_stream_t *	m_error_stream;

		//!  ,   
		//!  %next()    %back().
		bool	m_prev_word_returned;

		/*!  ,  m_word  
			ch  m_word. */
		void
		expand( char ch );

		//!   .
		/*!     ,
			    c_eof.

			\attention   MS-DOS
			  '\r'  
			.
		*/
		virtual int
		getch(
			//!    
			//! .
			char & ch );

		/*! \return      
			c_new_line_in_string. */
		const char *
		err_new_line_in_string(
			int & error_flag );

		/*! \return      
			c_empty_string. */
		const char *
		err_empty_string(
			int & error_flag );

		/*! \return      
			c_quote_in_word. */
		const char *
		err_quote_in_word(
			int & error_flag );

		/*! \return      
			c_invalid_back_slash_seq. */
		const char *
		err_invalid_back_slash_seq(
			int & error_flag );

	private :
		word_container_t(
			const word_container_t & );
		word_container_t &
		operator=(
			const word_container_t & );
};

/*
** words_from_str_t
*/
/*!
	\brief  C-.

	 word_container_t   C-.
*/
class	words_from_str_t :
	public word_container_t {
	public :
		//!  .
		words_from_str_t(
			//!   .
			const char * str );
		virtual ~words_from_str_t( void );

	protected :
		//!     
		//!  .
		const char * m_cur;

		/*!  *m_cur != 0,   *m_cur 
			 m_cur.   
			,    . */
		virtual int
		getch( char & ch );
};

/*
** words_from_file_t
*/
/*!
	\brief  .

	 word_container_t   
	.      
	 stdio.
*/
class	words_from_file_t :
	public word_container_t {
	public :
		//!  .
		/*!     
			 read-only. */
		words_from_file_t(
			//!   .
			const char * file_name );
		virtual ~words_from_file_t( void );

		//!    
		//!  .
		/*!   errno.  
			  ,     
			. */
		int
		query_last_io_result() const;

		//!    .
		const char *
		query_file_name() const;

		//!      .
		unsigned int
		query_line() const;

	protected :
		//!    .
		/*!     
			 file_name  . */
		char *	m_file_name;
		//! .
		FILE *	m_file;
		//!  .
		char *	m_buf;
		//!  ,    %m_buf.
		size_t	m_size;
		//!    %m_buf.
		size_t	m_pos;
		//!    .
		unsigned int	m_line;

		//!    -.
		int	m_errno;

		//!     .
		virtual int
		getch( char & ch );

		//!      .
		int
		lo_get_ch( char & ch );

		//!    
		//!      .
		/*!    lock-ahead
			. */
		void
		lo_back_ch();
};

/*
** words_from_file_with_simple_comment_t
*/
/*!
	\brief       .

	 words_from_file_t   
	    , ..
	   .. response-.
	  ,  args-4   
	   .   
	    
	 .

	   
	%words_from_file_with_simple_comment_t. 
	    ,
	  , :
\verbatim
#
# Some server socket config
#
ip-addr www.some-site.com
port 5000

#
# Client authentification config
#
client-auth-method	password
\endverbatim

	\attention   , 
	 
	    . 
	  '#'   ,
	..  '#'    :
\verbatim
mxx-cmd-line	mxxc -f somefile.#not-comment
\endverbatim
*/
class	words_from_file_with_simple_comment_t:
	public words_from_file_t
{
	public :
		//!  .
		/*!     
			 read-only. */
		words_from_file_with_simple_comment_t(
			//!    .
			const char * file_name,
			//!  , 
			//!  .
			char comment_char );
		//!  .
		/*!     
			 read-only. */
		words_from_file_with_simple_comment_t(
			//!    .
			const char * file_name,
			//!  ,   
			//!   .
			const char * comment_char_set );
		virtual ~words_from_file_with_simple_comment_t();

	protected :
		//!  .
		enum parse_state_t {
			//!   .
			e_new_line,
			//!  .
			e_comment,
			//!  .
			e_normal
		};

		//!   .
		parse_state_t	m_state;
		//!    , 
		//!   .
		char *	m_comment_char_set;

		//!   .
		/*!   */
		virtual int
		getch( char & ch );
};

/*
** words_from_argv_t
*/

/*!
	\brief   argv.

	 word_container_t  
	 ,   argc, argv.

	\attention     argv .
*/
class	words_from_argv_t :
	public word_container_t {
	public :
		//!  .
		words_from_argv_t(
			//!    argv.
			int argc,
			//!  .
			char ** argv );
		virtual ~words_from_argv_t();

		//!     argv.
		virtual
		const char *	next(
			int & error_code,
			error_stream_t * error_stream );

		//!   argc.
		int
		query_argc() const;

	protected :
		//!  argc.
		int	m_argc;
		//!  argv.
		char **	m_argv;
		//!     argv.
		int	m_cur;
};

/*
** cmd_t
*/

/*!
	\brief  ,   ''.

	    ,   
	    ().
*/
class	cmd_t {
	public :
		cmd_t();
		//!  .
		cmd_t(
			//!  .    
			//!   %cmd_t.
			const char * name );
		virtual ~cmd_t();

		//!      .
		/*!    ,  
			     
			 .

			    m_name 
			 \c name.

			\return true,  \c name  
			.
		*/
		virtual	bool
		compare_name(
			//!   .
			const char * name ) const;

		//!   .
		/*!       %cmd_t,
			   
			 ,    
			     
			.     %cmd_t
			  .

			 %cmd_t    
			      :
			-    , 
			-    , 
			-      .

			  ,  
			     
			  (  ),
			    
			   .   
			 ,     ,
			      .
			      
			     
			.

			     
			   ,  
			  c_ok.

			     
			  .

			\return c_ok,    
			.     .
		*/
		virtual int
		process_params(
			//!  .
			word_container_t & words,
			//!   .
			//!    0.
			error_stream_t * error_stream );

		//!   .
		/*! \return  m_name. */
		const char *
		query_name() const;

		//!   ?
		/*!    
			   ,  ,
			   . 
			 ,   , 
			   .

			 ,   . */
		virtual bool
		is_mandatory() const;

		//!   .
		/*!      
			   ,  
			  .
		*/
		virtual bool
		is_defined() const;

	protected :
		//! ,     .
		char *	m_name;

		//!    
		//!   .
		/*!     
			   
			   process_params().

			\return    
			 .  
			    
			word_container_t::next() 
			cmd_t::next_mandatory_word().
			 0,   
			   .
		*/
		const char *
		next_mandatory_word(
			//!  .
			word_container_t & words,
			//!   .
			//!    0.
			error_stream_t * error_stream,
			//!   , 
			//!    
			//!  .
			int & error_code );
};

/*
** cmd_parser_t
*/

/*!
	\brief     .

	      ,
	    , 
	   , 
	 .   
	   
	.
*/
class	cmd_parser_t {
	public :
		//!  .
		cmd_parser_t(
			//!    
			//!     .
			bool is_word_backing_enabled = false );
		~cmd_parser_t();

		//!     
		//!  .
		void
		add_cmd( cmd_t * pcmd );

		//!     
		//!  .
		/*!     
			. */
		void
		add_cmds( cmd_t * p1, ... );

		//!  .
		int
		parse(
			//!  .
			word_container_t & words,
			//!   .
			//!    0.
			error_stream_t * error_stream );

	protected :
		//!   .
		cmd_t **	m_list_cmd;
		//!   
		//!  %m_list_cmd.
		size_t	m_list_len;
		//!  %m_list_cmd.
		size_t	m_list_capacity;
		//!    
		//!     .
		bool	m_is_word_backing_enabled;

		//!  .
		/*!       parse.

			   0. */
		error_stream_t *	m_error_stream;

		//!     .
		cmd_t *
		find_cmd_by_name(
			//!   .
			const char * name );

		//!     
		//!   .
		int
		process_cmd(
			//!   .
			const char * name,
			//!  .
			word_container_t & words );

		//!  ,   
		//!  .
		int
		check_mandatory_args();
};

/*
** cmd_str_value_t
*/
/*!
	\brief    
	 - .

	    C-.
*/
class	cmd_str_value_t :
	public cmd_t
{
	public :
		//!  .
		cmd_str_value_t(
			//!  .
			const char * name,
			//! true,   .
			bool mandatory );
		virtual ~cmd_str_value_t();

		//!     
		//!  .
		/*!    , 
			   .

			    
			   
			 .
		*/
		virtual int
		process_params(
			word_container_t & words,
			error_stream_t * error_stream );

		/*!   m_is_mandatory. */
		virtual bool
		is_mandatory() const;

		/*!   m_is_defined. */
		virtual bool
		is_defined() const;

		/*!   . */
		const char *
		query_value() const;

	protected :
		/*!   . */
		char *	m_value;
		/*! true,   . */
		bool	m_is_mandatory;
		/*! true,   . */
		bool	m_is_defined;
};

/*
** cmd_uint_value_t
*/
/*!
	\brief    
	 -   .

	    
	  C-  
	( ,   
	).
*/
class	cmd_uint_value_t :
	public cmd_t
{
	public :
		//!  .
		cmd_uint_value_t(
			//!  .
			const char * name,
			//! true,   .
			bool mandatory );
		virtual ~cmd_uint_value_t();

		//!     
		//!  .
		/*!    , 
			   .

			    
			   
			 .
		*/
		virtual int
		process_params(
			word_container_t & words,
			error_stream_t * error_stream );

		/*!   m_is_mandatory. */
		virtual bool
		is_mandatory() const;

		/*!   m_is_defined. */
		virtual bool
		is_defined() const;

		/*!   . */
		unsigned int
		query_value() const;

	protected :
		/*!   . */
		unsigned int	m_value;
		/*! true,   . */
		bool	m_is_mandatory;
		/*! true,   . */
		bool	m_is_defined;
};

/*
** cmd_on_off_switch_t
*/
/*!
	\brief    - .
*/
class	cmd_on_off_switch_t :
	public cmd_t
{
	public :
		//!  .
		cmd_on_off_switch_t(
			//!  .
			const char * name,
			//! true,   .
			bool mandatory );
		virtual ~cmd_on_off_switch_t( void );

		//!    ,
		//!   .
		/*!
			    
			   
			 .
		*/
		virtual int
		process_params(
			word_container_t & words,
			error_stream_t * error_stream );

		/*!   m_is_mandatory. */
		virtual bool
		is_mandatory() const;

		/*!   m_is_defined. */
		virtual bool
		is_defined() const;

	protected :
		/*! true,   . */
		bool	m_is_mandatory;
		/*! true,   . */
		bool	m_is_defined;
};

/*
** cmd_strvector_value_t
*/
/*!
	\brief      - ,
	     .

	, ,  :
	<tt>-file p1 -file p2 ... -file p3</tt>.  
	    .   
	   
	query_value_count(), query_value().
*/
class	cmd_strvector_value_t :
	public cmd_t
{
	public :
		//!  .
		cmd_strvector_value_t(
			//!  .
			const char * name,
			//! true,   .
			bool mandatory );
		virtual ~cmd_strvector_value_t();

		//!     
		//!  .
		/*!
			    
			   .

			   , 
			   .
		*/
		virtual int
		process_params(
			word_container_t & words,
			error_stream_t * error_stream );

		/*!   m_is_mandatory. */
		virtual bool
		is_mandatory() const;

		/*! \return true,    
			   . */
		virtual bool
		is_defined() const;

		//!   .
		size_t
		query_value_count() const;

		//!     .
		/*!
			   0.
		*/
		const char *	query_value(
			size_t index ) const;

	protected :
		/*!   . */
		char **	m_values;
		/*!     m_values. */
		size_t	m_vector_len;
		/*!  m_values. */
		size_t	m_vector_capacity;
		/*! true,   . */
		bool	m_is_mandatory;
};

/*
** cmd_response_file_t
*/
/*!
	\brief ,    (
	) response-.

	     ,
	  response-   :
	<tt><><-></tt>, ,
	<tt>@filelist.txt</tt>.
*/
class	cmd_response_file_t :
	public cmd_strvector_value_t
{
	public :
		//!  .
		cmd_response_file_t(
			//! ,  , 
			//!    response-.
			char first_char = '@' );
		~cmd_response_file_t();

		/*! \return true,    \c name
			  m_first_char. */
		virtual	bool
		compare_name( const char * name ) const;

		/*!   response- ( 
			 )   
			 .
			
			     ,
			   words.back(),  
			words.next(),    
			. */
		virtual int
		process_params(
			word_container_t & words,
			error_stream_t * error_stream );

	protected :
		/*! ,  ,  
			  response-. */
		char	m_first_char;
};

} /* namespace args_4 */

#endif
