/*!
	\file
	\brief    .
*/

#ifndef _AUTO_PTR_3__OBJ_PTR_HPP_
#define _AUTO_PTR_3__OBJ_PTR_HPP_

namespace auto_ptr_3 {

/*
** obj_ptr_t
*/

//!     
//!   .
template< class T >
class	obj_ptr_t {
	protected :
		//!    .
		//!   0,  obj_ptr_t   
		//!   .
		T *	m_p;

	public :
		typedef T type_t;

		//!   .
		obj_ptr_t() :
			m_p( 0 ) {
		}

		/*!      
			.
			\param ptr    .
		*/
		obj_ptr_t( T * ptr ) :
			m_p( ptr ) {
		}

		/*!
			    ,
			      0.
		*/
		~obj_ptr_t() {
			if( m_p )
				delete m_p;
		}

		/*!       .
			  0,    .
		*/
		T *
		query() {
			return m_p;
		}

		/*!       .
			  0,    .
		*/
		const T *
		query() const {
			return m_p;
		}

		/*!     .
			   ( )  .
		*/
		void
		set( T * ptr ) {
			m_p = ptr;
		}

		/*!    .
			   ( )  .
		*/
		void
		clear() {
			m_p = 0;
		}

		/*!   ,   .
		*/
		void
		destroy() {
			if( m_p ) {
				T * ptr = m_p;
				m_p = 0;

				delete ptr;
			}
		}

		/*!     obj_ptr_t.
		*/
		void
		swap( obj_ptr_t< T > & o ) {
			if( &o != this ) {
				T * ptr = m_p;

				m_p = o.m_p;

				o.m_p = ptr;
			}
		}

		//
		//   3
		//

		/*!     .
			\attention   ,  
				 .
		*/
		T &
		operator*() {
			return *m_p;
		}

		/*!     .
			\attention   ,  
				 .
		*/
		const T &
		operator*() const {
			return *m_p;
		}

		/*!     .
			\attention   ,  
				 .
		*/
		T *
		operator->() {
			return m_p;
		}

		/*!     .
			\attention   ,  
				 .
		*/
		const T *
		operator->() const {
			return m_p;
		}

		//
		//    std::auto_ptr
		//

		/*!       .
			  0,    .

			\note     std::auto_ptr
		*/
		T *
		get() const {
			return m_p;
		}

		/*!      
			      
			obj_ptr_t . ..   
			release  obj_ptr_t   
			.

			\par
			  0,     .

			\note     std::auto_ptr
		*/
		T *
		release() {
			T * p = m_p;
			m_p = 0;
			return p;
		}

		/*!        \p o,
			   .   
			obj_ptr_t    \p o.

			\note     std::auto_ptr
		*/
		void
		reset(
			//!     .
			T * o = 0 ) {
			if( o != m_p ) {
				destroy();
			}
			m_p = o;
		}

	private :
		/*! \attention   ,  
			 .
		*/
		obj_ptr_t( const obj_ptr_t< T > & o );
		/*! \attention   ,  
			 .
		*/
		obj_ptr_t< T > &
		operator=(
			const obj_ptr_t< T > & o );
};

//
// obj_destruction_t
//

//!  ,    .
/*!
	\since v.3.1
	
	. auto_ptr_3::soft_obj_ptr_t.
*/
template< class T >
class	obj_destruction_t
{
	private :
		//!     .
		T *	m_obj;
		//!     .
		bool	m_is_destroyable;

	public :
		//!  .
		obj_destruction_t(
			//!     .
			T * obj,
			//!     .
			bool is_destroyable )
		:
			m_obj( obj ),
			m_is_destroyable( is_destroyable )
		{
		}

		//!     .
		T *
		obj() const
		{
			return m_obj;
		}

		//!     .
		bool
		is_destroyable() const
		{
			return m_is_destroyable;
		}
};

//
// destroyable_obj_t
//

//!  ,    .
/*!
	\since v.3.1
	
	. auto_ptr_3::soft_obj_ptr_t.
*/
template< class T >
class	destroyable_obj_t :
	public obj_destruction_t< T >
{
	public :
		//!  .
		destroyable_obj_t(
			//! ,    .
			T * obj )
		:
			obj_destruction_t< T >( obj, true )
		{
		}
};

//
// nondestroyable_obj_t
//

//!  ,    
/*!
	\since v.3.1
	
	. auto_ptr_3::soft_obj_ptr_t.
*/
template< class T >
class	nondestroyable_obj_t :
	public obj_destruction_t< T >
{
	public :
		//!  .
		nondestroyable_obj_t(
			//! ,    .
			T & obj )
		:
			obj_destruction_t< T >( &obj, false )
		{
		}
};

//
// soft_obj_ptr_t
//

//!    .
/*!
	\since v.3.1

	     
	     ,  . ,
	   -:

\code
//  .
class	stream_t
{
	public :
		virtual stream_t &
		out( const char * what ) = 0;
};

//      
//   .
class	file_stream_t : public stream_t
{
	public :
		virtual stream_t &
		out( const char * what );
	...
};

// ,   ,   
//      
//  stream_t.
class	gzip_stream_t : public stream_t
{
	private :
		stream_t *	m_to;
		bool	m_is_must_be_destroyed;

	public :
		gzip_stream_t(
			// ,     
			//  .
			stream_t * to,
			//     
			//    ?
			bool is_must_be_destroyed )
		:
			m_to( to ),
			m_is_must_be_destroyed( is_must_be_destroyed )
		{
		}
		~gzip_stream_t()
		{
			if( m_is_must_be_destroyed )
				delete m_to;
		}

		virtual stream_t &
		out( const char * what );
	...
};
\endcode

	     auto_ptr_3::obj_ptr_t
	  , ..     
	 .     
	 soft_obj_ptr_t.     
	  :

\code
// ,   ,   
//      
//  stream_t.
class	gzip_stream_t : public stream_t
{
	private :
		auto_ptr_3::soft_obj_ptr_t< stream_t >	m_to;

	public :
		gzip_stream_t(
			// ,     
			//  .
			stream_t * to,
			//     
			//    ?
			bool is_must_be_destroyed )
		:
			m_to( to, is_must_be_destroyed )
		{
		}
		~gzip_stream_t()
		{
		}

		virtual stream_t &
		out( const char * what );
	...
};
\endcode

	   destroyable_obj_t 
	nondestroyable_obj_t   %soft_obj_ptr_t
	   :

\code
// ,   ,   
//      
//  stream_t.
class	gzip_stream_t : public stream_t
{
	private :
		auto_ptr_3::soft_obj_ptr_t< stream_t >	m_to;

	public :
		typedef auto_ptr_3::destroyable_obj_t< stream_t > destroyable_t;
		typedef auto_ptr_3::nondestroyable_obj_t< stream_t > nondestroyable_t;

		gzip_stream_t(
			// ,     
			//  .
			const auto_ptr_3::obj_destruction_t< stream_t > & to )
		:
			m_to( to )
		{
		}
		~gzip_stream_t()
		{
		}

		virtual stream_t &
		out( const char * what );
	...
};

void
use_gzip( stream_t * to )
{
	gzip_stream_t gzip_stream( gzip_stream_t::nondestroyable_t( to ) );
	gzip_stream.out( "Something" );
}
\endcode
*/
template< class T >
class	soft_obj_ptr_t : public obj_ptr_t< T >
{
	//!    .
	typedef obj_ptr_t< T >	base_type_t;
	public :
		//!  -.
		soft_obj_ptr_t()
		{
		}
		
		//!  .
		soft_obj_ptr_t(
			//!  .
			T * ptr,
			//!      .
			bool is_must_be_destroyed )
		:
			base_type_t( ptr ),
			m_is_must_be_destroyed( is_must_be_destroyed )
		{
		}

		//!  .
		soft_obj_ptr_t(
			//!      .
			const obj_destruction_t< T > & info )
		:
			base_type_t( info.obj() ),
			m_is_must_be_destroyed( info.is_destroyable() )
		{
		}

		~soft_obj_ptr_t()
		{
			if( !m_is_must_be_destroyed )
				//    
				// .
				base_type_t::clear();
		}

		//!     soft_obj_ptr_t.
		void
		swap( soft_obj_ptr_t< T > & o )
		{
			if( &o != this )
			{
				bool is_must_be_destroyed = m_is_must_be_destroyed;
				m_is_must_be_destroyed = o.m_is_must_be_destroyed;
				o.m_is_must_be_destroyed = is_must_be_destroyed;

				base_type_t::swap( o );
			}
		}

		//!     
		//! .
		/*!
			     
			 ,   
			.
		*/
		void
		reset(
			//!     .
			T * o = 0,
			//!      .
			//!     
			//!    .
			bool is_must_be_destroyed = true )
		{
			if( !m_is_must_be_destroyed )
				base_type_t::clear();
			base_type_t::reset( o );
			m_is_must_be_destroyed = is_must_be_destroyed;
		}

		//!     
		//! .
		/*!
			     
			 ,   
			.
		*/
		void
		reset(
			const obj_destruction_t< T > & info )
		{
			if( !m_is_must_be_destroyed )
				base_type_t::clear();
			base_type_t::reset( info.obj() );
			m_is_must_be_destroyed = info.is_destroyable();
		}

		//!      .
		bool
		is_destroyable() const
		{
			return m_is_must_be_destroyed;
		}

		//!    
		//!  .
		/*!
			\return    
			   .
		*/
		bool
		set_destroyable( bool is_destroyable )
		{
			bool r = m_is_must_be_destroyed;
			m_is_must_be_destroyed = is_destroyable;
			
			return r;
		}

	private :
		//!       .
		bool	m_is_must_be_destroyed;
};

} /* namespace auto_ptr_3 */

#endif
