/*

cls_2: Curl Like Syntax
Yauheni A. Akhotnikau (C) 2001-2003
eao197@yahoo.com
-------------------------------------------------

Permission is granted to anyone to use this software for any purpose on any
computer system, and to redistribute it freely, subject to the following
restrictions:

1. This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

2. The origin of this software must not be misrepresented, either by
   explicit claim or by omission.

3. Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

-------------------------------------------------

*/

/*!
	\file
*/

#if !defined( _CLS_2__TAG_BY_FACTORY_HPP_ )
#define _CLS_2__TAG_BY_FACTORY_HPP_

#include <string>
#include <vector>
#include <map>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <cls_2/h/cls.hpp>

namespace cls_2 {

/*!
	\brief   ,  -   .

	,       :

\verbatim
	{p
		This is normal text.
		{b This is bold text.
			{i This is italic bold text. }
		}
		{i This is italic text.
			{b This is bold italic text.
		}
	}
\endverbatim

	   {b}  {i}      
	.

	       
	    .

\code
	class	tag_bold_t : public tag_text_t {
		...
		private :
			tag_with_factory_t	m_nested;
	};
	class	tag_italic_t : public tag_text_t {
		...
		private :
			tag_with_factory_t	m_nested;
	};

	class	tag_bold_factory_t : public tfactory_t {
		public :
			virtual tag_t *
			operator()( tag_with_factory_t & tag_owner,
				const char * tag_name ) {
				return new tag_bold_t( ... );
			}
	};
	static tag_bold_factory_t
	g_tag_bold_factory;
	
	class	tag_italic_factory_t : public tfactory_t {
		public :
			virtual tag_t *
			operator()( tag_with_factory_t & tag_owner,
				const char * tag_name ) {
				return new tag_italic_t( ... );
			}
	};
	static tag_italic_factory_t
	g_tag_italic_factory;

	tag_bold_t::tag_bold_t(...) ...
	{
		m_nested.add_factory( g_tag_italic_factory );
	}

	tag_italic_t::tag_italic_t(...) ...
	{
		m_nested.add_factory( g_tag_bold_factory );
	}
\endcode
*/

class	tag_by_factory_t;

//
// tfactory_t
//

/*
	    .
*/
class	tfactory_t {
	public :
		virtual ~tfactory_t();

		//    .  
		//   - , 
		//     delete.
		virtual tag_t *
		operator()(
			// ,   .
			const tag_by_factory_t & tag_owner,
			//  ,   .
			const char * tag_name ) = 0;
};

//
// tfactory_constraint_t
//

/*
	      .
	       
	.      
	,     .

	      tfactory_constraint_t.
*/
class	tfactory_constraint_t {
	public :
		virtual ~tfactory_constraint_t();

		//    c_ok,    .
		virtual int
		check(
			const tag_by_factory_t & tag_owner,
			const char * tag_to_create ) = 0;
};

//
// tag_by_factory_t
//

/*
*/
class	tag_by_factory_t :
	public tag_t {
	public :
		tag_by_factory_t(
			//      
			// query_name.
			const char * pseudo_name,
			//   true,   
			//       .
			bool is_mandatory );
		virtual ~tag_by_factory_t();

		//      .
		//  factory    
		// tag_by_factory.
		void
		add_factory(
			//  ,    .
			const char * tag_name,
			// tag_by_factory_t   
			//   .
			tfactory_t & factory );

		//  .
		//     -.
		//  tag_by_factory_t   
		//  -.
		//   0,  ,   
		// .
		tfactory_constraint_t *
		set_constraint(
			tfactory_constraint_t * constraint );

		virtual int
		on_start(
			parser_context_t & context );

		virtual int
		on_finish(
			parser_context_t & context );

		virtual int
		on_tag(
			parser_context_t & context,
			tag_t & tag );

		virtual int
		on_tok_space(
			parser_context_t & context,
			const char * tok_value );

		virtual int
		on_tok_nonspace(
			parser_context_t & context,
			const char * tok_value );

		virtual int
		on_tok_string(
			parser_context_t & context,
			const char * tok_value );

		//     
		//    0,  
		//  .
		virtual tag_t **
		query_tags();

		//     , 
		//   .   
		//  -,   .
		virtual const char *
		query_name() const;

		//      .
		//      
		//  ,    
		//  .
		virtual bool
		compare_name( const char * name ) const;

		virtual bool
		is_mandatory() const;

		//  true,      
		//  .
		virtual bool
		is_defined() const;

		//  .
		//      .
		//    .
		virtual void
		reset();

		//   format    
		//  .
		virtual void
		format(
			tag_formatter_t & fmt );

		size_t
		size() const;

		const tag_t &
		at( size_t index ) const;

		//       .
		void
		make_defined(
			//       
			// ,    
			//  delete.
			tag_t * tag );

	private :
		typedef tag_t * ptr_t;
		typedef std::vector< ptr_t > vector_t;
		typedef std::map< std::string, tfactory_t * >
			tfactory_map_t;

		//     .
		tfactory_map_t	m_factories;

		//   .
		auto_ptr_3::obj_ptr_t< tag_t >	m_cur_tag;
		//   .
		vector_t	m_values;

		bool	m_is_mandatory;

		// .
		//   0,  , 
		//     .
		tfactory_constraint_t *	m_constraint;

		void
		destroy_contents();
};

} /* namespace cls_2 */

#endif
