/*

cls_2: Curl Like Syntax
Yauheni A. Akhotnikau (C) 2001-2003
eao197@yahoo.com
-------------------------------------------------

Permission is granted to anyone to use this software for any purpose on any
computer system, and to redistribute it freely, subject to the following
restrictions:

1. This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

2. The origin of this software must not be misrepresented, either by
   explicit claim or by omission.

3. Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

-------------------------------------------------

*/

/*!
	\file
	\brief      
	 C++.
*/

#include <fstream>
#include <sstream>

#include <string.h>
#include <errno.h>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <cls_2/h/es_file_line_num.hpp>
#include <cls_2/h/es_tag_stack.hpp>
#include <cls_2/h/def_msg_storage.hpp>

#include <cls_2/std_cpp/h/is_ios.hpp>
#include <cls_2/std_cpp/h/es_ios.hpp>
#include <cls_2/std_cpp/h/utils.hpp>

namespace cls_2
{

namespace std_cpp
{

//
// parse
//

int
parse_istream(
	std::istream & in,
	const std::string & stream_name,
	cls_2::tag_t ** tags,
	size_t tag_count,
	std::ostream * error_stream,
	unsigned int options )
{
	//  .
	parser_t parser;
	//   .
	for( size_t i = 0; i != tag_count; ++i )
	{
		parser.tag_add( *( tags[ i ] ) );
	}

	//  .
	is_ios_t input_stream( in );

	//  .
	auto_ptr_3::obj_ptr_t< error_stream_t >	es_ptr;
	if( error_stream )
	{
		es_ptr.set( new es_ios_t( error_stream ) );
		if( es_uses_file_line & options )
			//     
			//  ,      .
			es_ptr.set( new es_file_line_num_t(
				es_file_line_num_t::destroyable_t( es_ptr.release() ),
				stream_name.c_str(),
				input_stream ) );
		if( es_uses_tag_stack & options )
			//     
			//  ,   .
			es_ptr.set( new es_tag_stack_t(
				es_tag_stack_t::destroyable_t( es_ptr.release() ),
				parser ) );
	}

	//     .
	cls_2::def_msg_storage_t msgs;

	//  ...
	cls_2::parser_context_t context;
	// ...   ...
	context.set_input_stream( &input_stream );
	// ...   ...
	context.set_error_stream( es_ptr.get() );
	// ...     .
	context.set_msg_storage( &msgs );

	//  ,   .
	return parser.parse( context );
}

//
// parse_file
//

int
parse_file(
	const std::string & stream_name,
	cls_2::tag_t ** tags,
	size_t tag_count,
	std::ostream * error_stream,
	unsigned int options )
{
	std::ifstream in( stream_name.c_str(), std::ios::in );
	if( in )
	{
		//   .  
		//   .
		return parse_istream( in, stream_name, tags, tag_count,
			error_stream, options );
	}
	else
	{
		//   .
		if( error_stream )
		{
			*error_stream << stream_name << ": "
				<< strerror( errno ) << std::endl;
		}
	}
	
	return c_io_error;
}

//
// parse_file
//

int
parse_file(
	const std::string & stream_name,
	cls_2::tag_t & tag,
	std::string & error_stream,
	unsigned int options )
{
	//      .
	cls_2::tag_t * tags[] = { &tag };
	std::ostringstream errors;

	int ret_code = parse_file( stream_name, tags, 1, &errors, options );
	if( cls_2::c_ok != ret_code )
		error_stream = errors.str();

	return ret_code;
}

} /* namespace std_cpp */

} /* namespace cls_2 */
