/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief  ,      
		 .
*/

#if !defined( _OESS_1__DB__CLN__DB_HPP_ )
#define _OESS_1__DB__CLN__DB_HPP_

#include <set>

#include <oess_1/db/h/declspec.hpp>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/defs/h/ent_id.hpp>

#include <oess_1/io/h/binstream.hpp>

#include <oess_1/stdsn/h/serializable.hpp>

#include <oess_1/db/site/h/abstract_connector.hpp>

namespace oess_1 {

namespace db {

namespace cln {

//
// db_t
//

/*!
	\brief  ,      
	 .
*/
class	OESS_1__DB__TYPE	db_t
{
	friend class transaction_t;
	public :
		db_t(
			//!   - ,
			//!     
			//!   delete.
			oess_1::db::site::abstract_connector_t * connector );
		virtual ~db_t();

		//!       .
		virtual void
		attach(
			//!   .
			const std::string & logic_name ) = 0;

		//!   .
		virtual void
		detach() = 0;

		virtual oess_1::db::site::abstract_connector_t &
		query_connector();

		virtual const oess_1::db::site::abstract_connector_t &
		query_connector() const;

	//! \name     .
	/*! \{ */

		//!    .
		/*! \since v.1.1.0

			    
			  .    
			  . */
		virtual void
		slice_create(
			//!   .
			const std::string & name ) = 0;

		//!    .
		/*!
			\since v.1.4.0
		*/
		virtual void
		slice_destroy(
			//!   .
			const std::string & name ) = 0;

		//!     .
		/*!
			\since v.1.4.0
			\return true,   .
		*/
		virtual bool
		slice_is_exists(
			//!   .
			const std::string & name ) = 0;

		//!      .
		/*!
			\since v.1.4.0
		*/
		virtual std::set< std::string >
		slice_names() = 0;

	/*! \} */

	//! \name    .
	/*! \{ */

		//!  .
		/*! \return   . */
		virtual oess_1::ent_id_t
		ent_create(
			//! ,     .
			const std::string & slice_name,
			//!  .
			const oess_1::stdsn::serializable_t & ent ) = 0;

		//!  .
		virtual void
		ent_destroy(
			//!   .
			const oess_1::ent_id_t & ent_id ) = 0;

		//!      .
		/*!
			\since v.1.4.0
		*/
		virtual void
		ent_destroy_all(
			//!  ,      .
			const std::string & slice_name ) = 0;

		//!  .
		/*!
			\note  \a ent     , 
				   .
		*/
		virtual void
		ent_load(
			//!   .
			const oess_1::ent_id_t & ent_id,
			//!   .
			oess_1::stdsn::serializable_t & ent ) = 0;

		//!   .
		virtual void
		ent_update(
			//!   .
			const oess_1::ent_id_t & ent_id,
			//!   .
			const oess_1::stdsn::serializable_t & ent ) = 0;

		//!    
		//! .
		/*!
			      
			     ent_id,
			 NULL.

			 ent_id   NULL,  , 
			     .
			    , 
			  NULL .
		*/
		virtual oess_1::ent_id_t
		ent_find_next(
			//!  ,    .
			const std::string & slice_name,
			//!   .
			const oess_1::ent_id_t & ent_id ) = 0;
	/*! \} */

	/*! \name     raw- .
		  ,   , 
		  / 
		.
	*/
	/*! \{ */
		//!  .
		/*!
			\since v.1.4.0
			\return   .
		*/
		virtual oess_1::ent_id_t
		ent_raw_create(
			//! ,     .
			const std::string & slice_name,
			//!  .
			oess_1::io::ibinstream_t & image,
			//!     .
			oess_1::uint_t image_size ) = 0;

		//!    .
		/*!
			\since v.1.4.0
			\return    .
		*/
		virtual oess_1::uint_t
		ent_raw_load(
			//!   .
			const oess_1::ent_id_t & ent_id,
			//!   .
			oess_1::io::obinstream_t & to,
			//!    ,  
			//!   .
			//!    -,  
			//!     .
			//!   0,     , 
			//!      .
			oess_1::uint_t length_limit =
					static_cast< oess_1::uint_t >( -1 ) ) = 0;

		//!   .
		/*!
			\since v.1.4.0
		*/
		virtual void
		ent_raw_update(
			//!   .
			const oess_1::ent_id_t & ent_id,
			//!  .
			oess_1::io::ibinstream_t & image,
			//!     .
			oess_1::uint_t image_size ) = 0;
	/*! \} */

	protected :
		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_begin() = 0;

		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_commit() = 0;

		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_rollback() = 0;

	private :
		//! ,      .
		auto_ptr_3::obj_ptr_t<
			oess_1::db::site::abstract_connector_t >
			m_connector_ptr;

	
		db_t( const db_t & );
		db_t &
		operator=( const db_t & );
};

//
// create_std_db
//

/*!
	    db_t.

	\return     ,
	     
	delete.
*/
OESS_1__DB__FUNC(db_t *)
create_std_db(
	//!   - ,
	//!     
	//!   delete.
	oess_1::db::site::abstract_connector_t * connector );

//
// transaction_t
//

//! ,   ,    .
/*! \since v.1.1.0
	   . 
	  ,       
	 commit().

	 :
	\code
	void f( oess_1::db::cln::db_t & db )
	{
		//  .
		oess_1::db::cln::transaction_t trx( db );

		//  -   .
		...

		//  .
		trx.commit();
	}
	\endcode

	   .  ,  ,
	       
	      :
	\code
	void f( oess_1::db::cln::db_t & db )
	{
		//  .
		oess_1::db::cln::transaction_t trx( db, &std::cerr );

		//  - .
		oess_1::ent_id_t id = db.ent_create( "some_slice", some_object_t() );
		//  .
		db.ent_destroy( id );
		//     .
		db.ent_destroy( id );

		//      - 
		//   .    
		//   .
	}
	\endcode
*/
class	OESS_1__DB__TYPE	transaction_t
{
	//! ,    .
	db_t &	m_db;
	//!    ?
	bool	m_is_finished;
	//!     ,
	//!   .
	std::ostream *	m_errs;

public :
	transaction_t( db_t & db,
		std::ostream * errs = 0 );
	~transaction_t();

	//!  .
	/*! ,   ,   
		  . */
	void
	commit();
};

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif
