/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\since v.1.1.0
	\file
	\brief ,      
		   .
*/

#if !defined( _OESS_1__DB__CLN__LOCAL_DB_HPP_ )
#define _OESS_1__DB__CLN__LOCAL_DB_HPP_

#include <exception>

#include <oess_1/db/h/declspec.hpp>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/db/site/h/localhost.hpp>

#include <oess_1/db/cln/h/db.hpp>

namespace oess_1 {

namespace db {

namespace cln {

//
// local_db_t
//

/*!
	\since v.1.1.0
	\brief ,      
		   .

	 :
	-   localhost    localhost
	 ;
	-   localhost_connector  localhost;
	-   db     localhost
	 .

	       .
	   . ,
	    local_db_t  .
*/
class	OESS_1__DB__TYPE	local_db_t
{
	public :
		//!  ,    
		//!   .
		enum
		{
			//!    .
			auto_create = 0x1,
			//!     ,   
			//! ,     .
			/*!       auto_create. */
			no_overwrite = 0x2,
			//!    
			//!    .
			//! \deprecated     
			//!  .
			auto_flush_mode = 0x4,
			//!     read-only.   
			//!    auto_create.
			read_only_mode = 0x8
		};

		/*!
			\brief    .
			   . */
		local_db_t(
			/*!   . */
			const std::string & phys_db_name,
			/*!   ,  ,
				      . */
			unsigned int	initial_actions =
				auto_create | no_overwrite )
		throw( std::exception );

		/*!
			\since v.1.4.0
			\brief      .

			   .
		*/
		local_db_t(
			/*!   . */
			const std::string & physic_name,
			/*!   . */
			const oess_1::db::storage::config_t & config,
			/*!   ,  ,
				      . */
			unsigned int	initial_actions =
				auto_create | no_overwrite )
		throw( std::exception );

		/*!      
			  db. */
		~local_db_t() throw();

		/*!     db. */
		oess_1::db::cln::db_t *
		query_db() throw();

	private :
		/*!   .  */
		auto_ptr_3::obj_ptr_t< oess_1::db::site::localhost_t >
			m_localhost;
		/*!     . */
		auto_ptr_3::obj_ptr_t< oess_1::db::cln::db_t >
			m_db;

		local_db_t( const local_db_t & );
		local_db_t &
		operator=( const local_db_t & );
};


} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif
