/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\file
	\since v.1.0.9
	\brief      .
*/

#if !defined( _OESS_1__DB__CLN__SLICE_IMAGE_HPP_ )
#define _OESS_1__DB__CLN__SLICE_IMAGE_HPP_

#include <map>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/defs/h/ex.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/cln/h/errno.hpp>
#include <oess_1/db/cln/h/slice_index_base.hpp>
#include <oess_1/db/cln/h/slice_image_base.hpp>

namespace oess_1
{

namespace db
{

namespace cln
{

/*!
	\since v.1.0.9
	\brief      .

	   ,   slice_image_t
	        . 
	    .  , 
	 slice_image_t    ,   
	      .   ,  
	       
	 .    
	  .   , 
	      . 
	slice_image_t     
	,      .
*/
template< class Item >
class	slice_image_storage_t
{
public :
	slice_image_storage_t()
	{}
	virtual ~slice_image_storage_t()
	{}

	//!    .
	virtual void
	insert(
		const oess_1::ent_id_t & id,
		const Item & value ) = 0;

	//!     (   ).
	virtual void
	erase(
		const oess_1::ent_id_t & id ) = 0;

	//!      (   ).
	virtual void
	update(
		const oess_1::ent_id_t & id,
		const Item & value ) = 0;

	//!     .
	virtual void
	get(
		const oess_1::ent_id_t & id,
		auto_ptr_3::soft_obj_ptr_t< Item > & o ) = 0;

	//!  .
	virtual void
	clear() = 0;

	//!    .
	/*!  prev_id == null,    
		 .  null,  
		 . */
	virtual oess_1::ent_id_t
	next(
		const oess_1::ent_id_t & id ) = 0;

private :
	//!   .
	slice_image_storage_t(
		const slice_image_storage_t< Item > & );
	slice_image_storage_t< Item > &
	operator=(
		const slice_image_storage_t< Item > & );
};

/*!
	\since v.1.0.9
	\brief   ,    
		   .

	       
	    .

	      
	     .   
	       
	  .

	   ,   
	     . , - 
	  /     .
*/
template< class Item >
class	no_slice_ram_storage_t :
	public slice_image_storage_t< Item >
{
private :
	//!  ,    .
	oess_1::db::cln::db_t &	m_db;
	//!  ,    .
	std::string	m_slice_name;

public :
	//!  .
	no_slice_ram_storage_t(
		oess_1::db::cln::db_t & db,
		const std::string & slice_name )
	:
		m_db( db )
	,	m_slice_name( slice_name )
	{}
	virtual ~no_slice_ram_storage_t()
	{}

	/*!    . */
	virtual void
	insert(
		const oess_1::ent_id_t & id,
		const Item & value )
	{}

	/*!    . */
	virtual void
	erase(
		const oess_1::ent_id_t & id )
	{}

	/*!    . */
	virtual void
	update(
		const oess_1::ent_id_t & id,
		const Item & value )
	{}

	/*!     . */
	virtual void
	get(
		const oess_1::ent_id_t & id,
		auto_ptr_3::soft_obj_ptr_t< Item > & o )
	{
		o.reset( auto_ptr_3::destroyable_obj_t< Item >( new Item() ) );
		m_db.ent_load( id, *o );
	}

	/*!    . */
	virtual void
	clear()
	{}

	/*!     
		   . */
	virtual oess_1::ent_id_t
	next(
		const oess_1::ent_id_t & id )
	{
		return m_db.ent_find_next( m_slice_name, id );
	}
};

/*!
	\since v.1.3.1
	\author V.Kostin
	\brief   ,   
	  .

	     
	      .
*/
template< class Item >
class	slice_ram_storage_t :
	public slice_image_storage_t< Item >
{
private :
	//!  ,    .
	oess_1::db::cln::db_t &	m_db;
	//!  ,    .
	std::string	m_slice_name;

	//! \name   ,      .
	//! \{
		typedef
			oess_1::ent_id_t
			id_t;
	//! \}

	//! \name  ,   .
	//! \{
		typedef
			std::map< id_t, Item >
			slice_in_ram_t;

		slice_in_ram_t m_slice_in_ram;
	//! \}

public :
	//!  .
	slice_ram_storage_t(
		oess_1::db::cln::db_t & db,
		const std::string & slice_name )
	:
		m_db( db )
	,	m_slice_name( slice_name )
	{}
	virtual ~slice_ram_storage_t()
	{}

	/*!      . */
	virtual void
	insert(
		const oess_1::ent_id_t & id,
		const Item & value )
	{
		m_slice_in_ram.insert(
			slice_in_ram_t::value_type( id, value ) );
	}

	/*!    . */
	virtual void
	erase(
		const oess_1::ent_id_t & id )
	{
		m_slice_in_ram.erase( id );
	}

	/*!    . */
	virtual void
	update(
		const oess_1::ent_id_t & id,
		const Item & value )
	{
		m_slice_in_ram[ id ] = value;
	}

	/*!     . */
	virtual void
	get(
		const oess_1::ent_id_t & id,
		auto_ptr_3::soft_obj_ptr_t< Item > & o )
	{
		typename slice_in_ram_t::iterator it = m_slice_in_ram.find( id );
		if( m_slice_in_ram.end() != it )
			o.reset( auto_ptr_3::nondestroyable_obj_t< Item >(
				it->second ) );
		else
			OESS_THROW_LOGIC( oess_1::db::cln::err::c_unknown_ent_id,
				"can't find entity " << id << " in RAM cache" )
	}

	/*!   - ,    ,
	. */
	virtual void
	clear()
	{
		m_slice_in_ram.clear();
	}

	/*!     .
		\attention     ,
		  NULL .
	 */
	virtual oess_1::ent_id_t
	next(
		const oess_1::ent_id_t & id )
	{
		if( id )
			{
				typename slice_in_ram_t::iterator it = m_slice_in_ram.find( id );
				if( m_slice_in_ram.end() != it )
					{
						++it;
						if( m_slice_in_ram.end() != it )
							return it->first;
					}
			}
		else
			if( m_slice_in_ram.size() )
				return m_slice_in_ram.begin()->first;

		return oess_1::ent_id_t();
	}
};

/*!
	\since v.1.0.9
	\brief      .

	       .
*/
template<
	class Item,
	class Obj_storage = no_slice_ram_storage_t< Item > >
class slice_image_t :
    public slice_image_iface_t< Item >
{
	//!    .
	typedef slice_image_iface_t< Item >	base_t;

private :
	//!  .
	/*!    
		,     
		   
		   . */
	slice_image_storage_t< Item > *	m_obj_storage;

public:
	/*!  .  ,
		      flags
		   slice_image_base_t.

		    .

		   slice_image_base_t::auto_load,
		      .
	*/
	slice_image_t(
		//!     .
		oess_1::db::cln::db_t & db,
		//!  ,     .
		const std::string & slice_name,
		unsigned int flags = 0 )
	throw( std::exception )
	:
		base_t( db, slice_name, flags )
	,	m_obj_storage( new Obj_storage( db, slice_name ) )
	{
		if( base_t::auto_load & flags )
			reload();
	}
	virtual ~slice_image_t()
	{}

	/*!   . */
	void
	reload()
	{
		clear();

		//      , 
		//   ,    .
		db_t & d = base_t::query_db();
		const std::string & s = base_t::slice_name();

		oess_1::ent_id_t id;
		while( id = d.ent_find_next( s, id ) )
		{
			Item o;
			d.ent_load( id, o );
			m_obj_storage->insert( id, o );
			base_t::lo_refresh_indexes( id, o );
		}
	}

	/*!   . */
	void
	clear()
	{
		m_obj_storage->clear();
		base_t::lo_clear();
	}

	/*!     , ,
		   . */
	virtual oess_1::ent_id_t
	insert( const Item & o )
	{
		oess_1::ent_id_t id = base_t::lo_insert( o );
		m_obj_storage->insert( id, o );

		return id;
	}

	/*!     , ,
		   . */
	virtual void
	erase( const oess_1::ent_id_t & id )
	{
		base_t::lo_erase( id );
		m_obj_storage->erase( id );
	}

	/*!     ,  
		    . */
	virtual void
	update(
		const ent_id_t & id,
		const Item & o )
	{
		base_t::lo_update( id, o );
		m_obj_storage->update( id, o );
	}

	virtual void
	get(
		const ent_id_t & id,
		auto_ptr_3::soft_obj_ptr_t< Item > & o ) const
	{
		m_obj_storage->get( id, o );
	}

	/*!   . */
	virtual void
	attach(
		slice_index_base_t & index,
		/*!   
			slice_image_base_t::auto_destroy_index,  
			    
			  delete. */
		unsigned int flags = 0 )
	{
		base_t::attach( index, flags );

		//      
		//     .
		auto_ptr_3::soft_obj_ptr_t< Item > o;
		oess_1::ent_id_t id;
		while( id = m_obj_storage->next( id ) )
		{
			m_obj_storage->get( id, o );
			index.on_insert( id, *o );
		}
	}
};

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif
