/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\since v.1.0.9
	\file
	\brief       
		 .
*/

#if !defined( _OESS_1__DB__CLN__SLICE_IMAGE_BASE_HPP_ )
#define _OESS_1__DB__CLN__SLICE_IMAGE_BASE_HPP_

#include <exception>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/cln/h/db.hpp>

namespace oess_1
{

namespace db
{

namespace cln
{

namespace impl
{
	struct	slice_image_base_data_t;
} /* namespace impl */

class	slice_index_base_t;

/*!
	\since v.1.0.9
	\brief       
		 .

	     .
	,      
	      .
	  ,    
	     protected-
	.
*/
class OESS_1__DB__TYPE	slice_image_base_t
{
public:
    virtual ~slice_image_base_t();

	//!  ,  
	//!    .
	enum
	{
		//!     
		//!   .
		auto_load = 0x1,
		//!     
		//!   .
		auto_destroy_index = 0x2
	};

    /*!   .

		      .
		   .
    */
    virtual void
    attach(
		slice_index_base_t & index,
		//!  , ,   
		//!  .
		//!    auto_destroy_index, 
		//!     .
		unsigned int flags );

    /*!
    	  .    
    	       
    	auto_destroy_index.
    */
    virtual void
    detach(
        slice_index_base_t & index );

	//! ,    .
	oess_1::db::cln::db_t &
	query_db() const;

	//! ,    .
	const std::string &
	slice_name() const;

protected :
    /*!     ,
    	   auto_load (- , 
    	     
    	make_real_obj().

    	    . */
    slice_image_base_t(
			oess_1::db::cln::db_t & db,
			const std::string & slice_name,
			//!  , ,  
			//!    .
			unsigned int flags )
	    throw( std::exception );

    /*!   .

    	     ,  
    	   .
    */
    void
    lo_clear();

    /*!     , ,
		   . */
    ent_id_t
    lo_insert( const oess_1::stdsn::serializable_t & o )
	throw( std::exception );

    /*!     , ,
		   . */
    void
    lo_erase( const oess_1::ent_id_t & id )
	throw( std::exception );

    /*!     ,  
		    . */
    void
    lo_update( const oess_1::ent_id_t & id,
        const oess_1::stdsn::serializable_t & o )
	throw( std::exception );

	/*!       
		  slice_index_base_t::on_insert. */
	void
	lo_refresh_indexes(
		const oess_1::ent_id_t & id,
		const oess_1::stdsn::serializable_t & o );

private :
	//!    .
	auto_ptr_3::obj_ptr_t< impl::slice_image_base_data_t >	m_data;

private :
/*! \name   . */
/*! \{ */
	slice_image_base_t(
		const slice_image_base_t & o );
	slice_image_base_t &
	operator=(
		const slice_image_base_t & o );
/* \} */
};

/*!
	\since v.1.0.9
	\brief      .
*/
template< class Item >
class	slice_image_iface_t :
	public slice_image_base_t
{
public :
	virtual ~slice_image_iface_t()
	{}

	/*!     , ,
		   . */
	virtual oess_1::ent_id_t
	insert( const Item & o ) = 0;

	/*!         . */
	virtual void
	erase( const oess_1::ent_id_t & id ) = 0;

	/*!     ,  
		    . */
	virtual void
	update(
		const ent_id_t & id,
		const Item & o ) = 0;

	/*!   . */
	virtual void
	get(
		const ent_id_t & id,
		auto_ptr_3::soft_obj_ptr_t< Item > & o ) const = 0;

protected :
    slice_image_iface_t(
		oess_1::db::cln::db_t & db,
		const std::string & slice_name,
		//!  , ,  
		//!    .
		unsigned int flags )
	throw( std::exception )
	:
		slice_image_base_t( db, slice_name, flags )
	{}
};

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif
