/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\file
	\since v.1.4.0
	\brief         .
*/

#if !defined( _OESS_1__DB__CLN__SLICE_NONUNIQUE_INDEX_HPP_ )
#define _OESS_1__DB__CLN__SLICE_NONUNIQUE_INDEX_HPP_

#include <map>
#include <iterator>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/cln/h/slice_index_base.hpp>

namespace oess_1
{

namespace db
{

namespace cln
{

/*!
	\since v.1.4.0
	\brief         .
*/
template<
	//   .
	class Item,
	//   .
	class Key,
	//   .
	class Key_extractor >
class    slice_nonunique_index_t :
	public slice_index_base_t
{
private :
	//!       
	//!    .
	typedef std::multimap< Key, oess_1::ent_id_t >
		key_map_t;

	//!      
	//!   .
	typedef std::map< oess_1::ent_id_t, Key >
		id_map_t;

	//!  ,    .
	slice_image_iface_t< Item > *	m_slice;

	//!       .
	key_map_t	m_keys;
	//!     .
	id_map_t	m_ids;

	//!    .
	Key_extractor	m_key_extractor;

public :
	//!  .
	class	iterator
		:	public std::iterator<
				std::forward_iterator_tag,
				Item >
	{
		private :
			//!    .
			typedef typename key_map_t::iterator it_t;

			//!    .
			it_t	m_it;

			//!   ,  
			//!    .
			slice_image_iface_t< Item > *	m_slice;

			//!   .
			/*!    , 
				     
				operator*()  operator->()   
				  . */
			auto_ptr_3::soft_obj_ptr_t< Item >	m_value;

		public :
			iterator()
			:
				m_slice( 0 )
			{}
			iterator(
				slice_image_iface_t< Item > * slice,
				const it_t & it )
			:
				m_it( it )
			,	m_slice( slice )
			{}
			iterator(
				const iterator & o )
			:	m_it( o.m_it )
			,	m_slice( o.m_slice )
			{}
			~iterator()
			{}

			iterator &
			operator=(
				const iterator & o )
			{
				iterator t( o );
				swap( t );
				return *this;
			}

			void
			swap( iterator & o )
			{
				std::swap( m_it, o.m_it );
				std::swap( m_slice, o.m_slice );
				m_value.swap( o.m_value );
			}

			reference
			operator*()
			{
				if( !m_value.get() )
					//   .
					m_slice->get( m_it->second, m_value );
				return *m_value;
			}

			pointer
			operator->()
			{
				if( !m_value.get() )
					//   .
					m_slice->get( m_it->second, m_value );
				return m_value.get();
			}

			iterator &
			operator++()
			{
				m_value.reset();
				++m_it;
				return *this;
			}

			iterator
			operator++(int)
			{
				m_value.reset();
				return iterator( m_slice, m_it++ );
			}

			bool
			operator==( const iterator & o ) const
			{
				return ( m_it == o.m_it );
			}

			bool
			operator!=( const iterator & o ) const
			{
				return ( m_it != o.m_it );
			}

			const oess_1::ent_id_t &
			id() const
			{
				return m_it->second;
			}
	};

	//!   .
	class	const_iterator
		:	public std::iterator<
				std::forward_iterator_tag,
				Item,
				ptrdiff_t,
				const Item *,
				const Item & >
	{
		private :
			//!    .
			typedef typename key_map_t::const_iterator it_t;

			//!    .
			it_t	m_it;

			//!   ,  
			//!    .
			slice_image_iface_t< Item > *	m_slice;

			//!   .
			/*!    , 
				     
				operator*()  operator->()   
				  . */
			auto_ptr_3::soft_obj_ptr_t< Item >	m_value;

		public :
			typedef const Item *	pointer;
			typedef const Item &	reference;

			const_iterator()
			:
				m_slice( 0 )
			{}
			const_iterator(
				slice_image_iface_t< Item > * slice,
				const it_t & it )
			:
				m_it( it )
			,	m_slice( slice )
			{}
			const_iterator(
				const const_iterator & o )
			:	m_it( o.m_it )
			,	m_slice( o.m_slice )
			{}
			~const_iterator()
			{}

			const_iterator &
			operator=(
				const const_iterator & o )
			{
				const_iterator t( o );
				swap( t );
				return *this;
			}

			void
			swap( const_iterator & o )
			{
				std::swap( m_it, o.m_it );
				std::swap( m_slice, o.m_slice );
				m_value.swap( o.m_value );
			}

			reference
			operator*()
			{
				if( !m_value.get() )
					//   .
					m_slice->get( m_it->second, m_value );
				return *m_value;
			}

			pointer
			operator->()
			{
				if( !m_value.get() )
					//   .
					m_slice->get( m_it->second, m_value );
				return m_value.get();
			}

			const_iterator &
			operator++()
			{
				m_value.reset();
				++m_it;
				return *this;
			}

			const_iterator
			operator++(int)
			{
				m_value.reset();
				return const_iterator( m_slice, m_it++ );
			}

			bool
			operator==( const const_iterator & o ) const
			{
				return ( m_it == o.m_it );
			}

			bool
			operator!=( const const_iterator & o ) const
			{
				return ( m_it != o.m_it );
			}

			const oess_1::ent_id_t &
			id() const
			{
				return m_it->second;
			}
	};

public:
	//!  , 
	//!      .
	slice_nonunique_index_t(
		slice_image_iface_t< Item > & slice,
		//!    .
		const Key_extractor & key_extractor,
		//!   ,  
		//!    slice_image_base_t::attach().
		unsigned int attach_flags = 0 )
	:
		m_slice( &slice )
	,	m_key_extractor( key_extractor )
	{
		m_slice->attach( *this, attach_flags );
	}
	virtual ~slice_nonunique_index_t()
	{}

	//!   .
	iterator
	begin()
	{
		return iterator( m_slice, m_keys.begin() );
	}

	//!   .
	const_iterator
	begin() const
	{
		return const_iterator( m_slice, m_keys.begin() );
	}

	//!   .
	iterator
	end()
	{
		return iterator( m_slice, m_keys.end() );
	}

	//!   .
	const_iterator
	end() const
	{
		return const_iterator( m_slice, m_keys.end() );
	}

	//!    .
	/*!
		    , 
		   .
	*/
	iterator
	find( const Key & o )
	{
		return iterator( m_slice, m_keys.find( o ) );
	}

	//!    .
	/*!
		    , 
		   .
	*/
	const_iterator
	find( const Key & o ) const
	{
		return const_iterator( m_slice, m_keys.find( o ) );
	}

	//!   .
	/*!      ,
		    .

		   ,  end(). */
	iterator
	lower_bound( const Key & o )
	{
		return iterator( m_slice, m_keys.lower_bound( o ) );
	}

	//!   .
	/*!      ,
		    .

		   ,  end(). */
	const_iterator
	lower_bound( const Key & o ) const
	{
		return const_iterator( m_slice, m_keys.lower_bound( o ) );
	}

	//!   .
	/*!      ,
		    .

		   ,  end(). */
	iterator
	upper_bound( const Key & o )
	{
		return iterator( m_slice, m_keys.upper_bound( o ) );
	}

	//!   .
	/*!      ,
		    .

		   ,  end(). */
	const_iterator
	upper_bound( const Key & o ) const
	{
		return const_iterator( m_slice, m_keys.upper_bound( o ) );
	}

	//!       .
	/*!
		    
		    .
	*/
	void
	erase( const iterator & it )
	{
		if( it != end() )
		{
			//      id.
			//      
			// id,    , 
			//  it.id().    
			// ,   !
			oess_1::ent_id_t id_for_erasing = it.id();
			m_slice->erase( id_for_erasing );
		}
	}

	//!       .
	/*!
		  ,   .

		    
		    .
	*/
	void
	erase( const Key & o )
	{
		//    , ,  
		// :  erase(it)    , 
		//       .
		iterator it = find( o );
		while( it != end() )
			{
				erase( it );
				it = find( o );
			}
	}

	//!     .
	unsigned int
	size() const
	{
		return m_keys.size();
	}

/*! \name ,   slice_image_base_t. */
/*! \{ */

	/*!      . */
	virtual void
	on_clear()
	{
		m_keys.clear();
		m_ids.clear();
	}

	/*!     .
		       
		    .
	*/
	virtual void
	on_insert(
		const oess_1::ent_id_t & id,
		const oess_1::stdsn::serializable_t & o )
	{
		const Item & real_obj = dynamic_cast< const Item & >( o );
		Key k( m_key_extractor( real_obj ) );

		insert_pair( k, id );
		m_ids[ id ] = k;
	}

	/*!     . */
	virtual void
	on_erase(
		const oess_1::ent_id_t & id )
	{
		typename id_map_t::iterator it_id( m_ids.find( id ) );
		if( it_id != m_ids.end() )
		{
			remove_pair( it_id->second, id );
			m_ids.erase( it_id );
		}
	}

	/*!      . */
	virtual void
	on_update(
		const oess_1::ent_id_t & id,
		const oess_1::stdsn::serializable_t & o )
	{
		typename id_map_t::iterator it_id( m_ids.find( id ) );
		if( it_id != m_ids.end() )
		{
			const Item & real_obj = dynamic_cast< const Item & >( o );
			Key k( m_key_extractor( real_obj ) );

			if( k < it_id->second || k > it_id->second )
				{
					remove_pair( it_id->second, id );
					insert_pair( k, id );
					it_id->second = k;
				}
		}
	}

	/*!      . */
	virtual void
	on_attach()
	{
		//    .
	}

	/*!      . */
	virtual void
	on_detach()
	{
		//   .
		m_keys.clear();
		m_ids.clear();
		m_slice = 0;
	}

/*! \} */

private :
	//!     .
	void
	insert_pair(
		const Key & k,
		const oess_1::ent_id_t & id )
		{
			m_keys.insert( key_map_t::value_type( k, id ) );
		}

	//!         .
	void
	remove_pair(
		const Key & k,
		const oess_1::ent_id_t & id )
		{
			for( key_map_t::iterator it = m_keys.lower_bound( k ),
					it_end = m_keys.upper_bound( k );
					it != it_end; ++it )
				if( it->second == id )
					{
						m_keys.erase( it );
						return;
					}
		}
};

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif

