/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief    #db_t.
*/

#if !defined( _OESS_1__DB__CLN__IMPL__STD_DB_HPP_ )
#define _OESS_1__DB__CLN__IMPL__STD_DB_HPP_

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/db/cln/h/db.hpp>

namespace oess_1 {

namespace db {

namespace cln {

namespace impl {

class	ent_packer_base_t;
class ent_unpacker_base_t;

//
// std_db_t
//

/*!
	\brief    #db_t.
*/
class	std_db_t :
	public db_t
{
	public :
		//!  .
		std_db_t(
			//!      .
			oess_1::db::site::abstract_connector_t * connector );
		virtual ~std_db_t();

	/*! \name    */
	/*! \{ */
		virtual void
		attach(
			const std::string & logic_name );

		virtual void
		detach();

		virtual void
		slice_create(
			const std::string & name );

		virtual void
		slice_destroy(
			const std::string & name );

		virtual bool
		slice_is_exists(
			const std::string & name );

		virtual std::set< std::string >
		slice_names();

		virtual oess_1::ent_id_t
		ent_create(
			const std::string & slice_name,
			const oess_1::stdsn::serializable_t & ent );

		virtual void
		ent_destroy(
			const oess_1::ent_id_t & ent_id );

		virtual void
		ent_destroy_all(
			const std::string & slice_name );

		virtual void
		ent_load(
			const oess_1::ent_id_t & ent_id,
			oess_1::stdsn::serializable_t & ent );

		virtual void
		ent_update(
			const oess_1::ent_id_t & ent_id,
			const oess_1::stdsn::serializable_t & ent );

		virtual oess_1::ent_id_t
		ent_find_next(
			const std::string & slice_name,
			const oess_1::ent_id_t & ent_id );

		virtual oess_1::ent_id_t
		ent_raw_create(
			const std::string & slice_name,
			oess_1::io::ibinstream_t & image,
			oess_1::uint_t image_size );

		virtual oess_1::uint_t
		ent_raw_load(
			const oess_1::ent_id_t & ent_id,
			oess_1::io::obinstream_t & to,
			oess_1::uint_t length_limit );

		virtual void
		ent_raw_update(
			const oess_1::ent_id_t & ent_id,
			oess_1::io::ibinstream_t & image,
			oess_1::uint_t image_size );
	/*! \} */

	protected :
	/*! \name        */
	/*! \{ */
		virtual void
		transaction_begin();

		virtual void
		transaction_commit();

		virtual void
		transaction_rollback();
	/*! \} */

	private :
		/*!
			 true,     . */
		bool	m_is_attached;

		//!       .
		/*!  ,   . */
		std::string	m_logic_name;

		//!       .
		void
		cleanup();

		//!  ,     .
		void
		must_be_attached() const;

		//!   ent_create  ent_raw_create.
		/*!
			\since v.1.4.0

			        
			   .    
			   .  ,
			     
			ent_*_create     
			 ,     
			,   ent_packer_base_t.

			\return   .
		*/
		oess_1::ent_id_t
		do_ent_create_actions(
			//!  ,    .
			const std::string & slice_name,
			//! ,  ,    .
			const ent_packer_base_t & packer );

		//!   ent_load  ent_raw_load.
		/*!
			\since v.1.4.0

			        
			   .    
			   .  ,
			     
			ent_*_load     
			 ,     
			,   ent_unpacker_base_t.
		*/
		void
		do_ent_load_actions(
			//!   .
			const oess_1::ent_id_t & ent,
			//! ,  ,    .
			ent_unpacker_base_t & packer );

		//!   ent_update  ent_raw_update.
		/*!
			\since v.1.4.0

			        
			   .    
			   .  ,
			     
			ent_*_update     
			 ,     
			,   ent_packer_base_t.
		*/
		void
		do_ent_update_actions(
			//!   .
			const oess_1::ent_id_t & ent,
			//! ,  ,    .
			const ent_packer_base_t & packer );

		//!      -.
		/*!      --  
			  ,   . */
		void
		transmit(
			//!   - !
			oess_1::db::site::impl::request_t * req,
			oess_1::db::site::impl::resp_wrapper_t & resp_wrapper,
			oess_1::uint_t awaiting_resp_id );
};

} /* namespace impl */

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */

#endif
