/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\since v.1.1.0
	\file
	\brief ,      
		   .
*/

#include <oess_1/defs/h/ex.hpp>

#include <oess_1/db/cln/h/local_db.hpp>
#include <oess_1/db/cln/h/errno.hpp>

namespace oess_1 {

namespace db {

namespace cln {

//!   ,    
//! %local_db_t.
/*!
	\since v.1.4.0

	    auto_create  read_only_mode,
	  .

	\throw std::exception     .
*/
inline void
check_initial_actions( unsigned int initial_actions )
	{
		if( ( local_db_t::auto_create & initial_actions ) &&
				( local_db_t::read_only_mode & initial_actions ) )
			OESS_THROW_LOGIC(
					err::c_invalid_initial_actions,
					"local_db_t::auto_create cannot be used with "
					"local_db_t::read_only_mode" )
	}

//!      .
/*!
	\since v.1.4.0

	    local_db_t.   
	 ,      ,   
	  copy&paste.
*/
inline void
describe_and_attach(
	//!   .
	const std::string & phys_db_name,
	//! ,   .
	const oess_1::db::storage::config_t & config,
	//!  ,    .
	unsigned int initial_actions,
	//!    localhost,   
	//!  .
	oess_1::db::site::localhost_t & localhost,
	//!      db_t. 
	auto_ptr_3::obj_ptr_t< db_t > & db )
	{
		check_initial_actions( initial_actions );
   
		/*    :
			-    localhost;
			-  localhost_connector;
			-  db   localhost_connector;
			-  db  .
		*/
		const std::string logic_name( "db" );
   
		//   --  .
		if( local_db_t::auto_create & initial_actions )
		{
			oess_1::uint_t creation_mode = 0;
			if( local_db_t::no_overwrite & initial_actions )
				creation_mode |= oess_1::db::site::localhost_t::
					no_overwrite;
			localhost.db_create( phys_db_name, config, creation_mode );
		}
   
		localhost.db_describe( logic_name, phys_db_name, config,
				//   read_only_mode,    
				//   rean-only . 
				( 0 != ( local_db_t::read_only_mode & initial_actions ) ),
				true );
   
		db.set( oess_1::db::cln::create_std_db(
				oess_1::db::site::create_std_localhost_connector(
						localhost ) ) );
   
		db->attach( logic_name );
	}

//
// local_db_t
//

local_db_t::local_db_t(
	const std::string & phys_db_name,
	unsigned int initial_actions )
throw( std::exception )
:
	m_localhost( oess_1::db::site::create_std_localhost() )
	{
		describe_and_attach(
				phys_db_name,
				oess_1::db::storage::config_t(),
				initial_actions,
				*m_localhost,
				m_db );
	}

local_db_t::local_db_t(
	const std::string & phys_db_name,
	const oess_1::db::storage::config_t & config,
	unsigned int initial_actions )
throw( std::exception )
:
	m_localhost( oess_1::db::site::create_std_localhost() )
	{
		describe_and_attach(
				phys_db_name,
				config,
				initial_actions,
				*m_localhost,
				m_db );
	}

local_db_t::~local_db_t() throw()
{
}

oess_1::db::cln::db_t *
local_db_t::query_db() throw()
{
	return m_db.query();
}

} /* namespace cln */

} /* namespace db */

} /* namespace oess_1 */
