/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief  ,    
	  .
*/

#if !defined( _OESS_1__DB__IMPL__DB_CONTENT__DB_CONTENT_HPP_ )
#define _OESS_1__DB__IMPL__DB_CONTENT__DB_CONTENT_HPP_

#include <string>
#include <set>
#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/defs/h/types.hpp>
#include <oess_1/defs/h/ent_id.hpp>

#include <oess_1/io/h/stream.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/storage/h/config.hpp>

namespace oess_1 {

namespace db {

namespace impl {

namespace db_content {

//
// db_content_t
//

/*!
	\brief  ,    
	  .
*/
class	OESS_1__DB__TYPE	db_content_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~db_content_t();

		//!     .
		/*!      . */
		static void
		db_create(
			//!    .
			const std::string & name,
			//!    .
			const oess_1::db::storage::config_t & config );

		//!     .
		static void
		db_destroy(
			const std::string & name );

		//!     .
		static bool
		db_is_exists(
			const std::string & name );

		/*!
			\brief   .
		*/
		virtual void
		open(
			//!    .
			const std::string & name,
			//!    .
			const oess_1::db::storage::config_t & config,
			//! true,      read-only.
			bool is_read_only,
			//! true,      
			//! . ,    
			//!  read-only.
			bool is_auto_repair_enabled ) = 0;

		//!      .
		virtual void
		client_attach(
			const std::string & client_id ) = 0;

		//!     .
		/*!
			      ,  
			  .

			   ,   
			.
		*/
		virtual void
		client_detach(
			const std::string & client_id ) = 0;

		//!    .
		virtual oess_1::uint_t
		client_count() const = 0;

		//!    .
		virtual void
		slice_create(
			//! ,     .
			const std::string & client_id,
			//!   .
			const std::string & name ) = 0;

		//!    .
		/*!
			\since v.1.4.0
		*/
		virtual void
		slice_destroy(
			//! ,     .
			const std::string & client_id,
			//!   .
			const std::string & name ) = 0;

		//!     .
		/*!
			\since v.1.4.0
			\return true,   .
		*/
		virtual bool
		slice_is_exists(
			//! ,     .
			const std::string & client_id,
			//!   .
			const std::string & name ) = 0;

		//!       .
		/*!
			\since v.1.4.0
		*/
		virtual std::set< std::string >
		slice_names(
			//! ,     .
			const std::string & client_id ) = 0;

		//!  .
		virtual oess_1::ent_id_t
		ent_create(
			//! ,     .
			const std::string & client_id,
			//! ,     .
			const std::string & slice_name,
			//!  .
			oess_1::io::istream_t & ent_value,
			//!  .
			oess_1::uint_t ent_size ) = 0;

		//!  .
		virtual void
		ent_destroy(
			//! ,     .
			const std::string & client_id,
			//!  .
			const oess_1::ent_id_t & ent_id ) = 0;

		//!      .
		virtual void
		ent_destroy_all(
			//! ,     .
			const std::string & client_id,
			//!  .
			const std::string & slice_name ) = 0;

		//!  .
		virtual void
		ent_load(
			//! ,     .
			const std::string & client_id,
			//!  .
			const oess_1::ent_id_t & ent_id,
			//!    .
			oess_1::io::ostream_t & ent_value ) = 0;

		//!   .
		virtual void
		ent_update(
			//! ,     .
			const std::string & client_id,
			//!  .
			const oess_1::ent_id_t & ent_id,
			//!   .
			oess_1::io::istream_t & ent_value,
			//!    .
			oess_1::uint_t ent_size ) = 0;

		//!    .
		/*!       
			     ent_id,
			 NULL.
			 ent_id   NULL,  , 
			     .
			    , 
			  NULL . */
		virtual oess_1::ent_id_t
		ent_find_next(
			//! ,     .
			const std::string & client_id,
			//!  .
			const std::string & slice_name,
			//!  .
			const oess_1::ent_id_t & ent_id ) = 0;

		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_begin(
			const std::string & client_id ) = 0;

		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_commit(
			const std::string & client_id ) = 0;

		//!  .
		/*! \since v.1.1.0 */
		virtual void
		transaction_rollback(
			const std::string & client_id ) = 0;
};

//
// create_std_db_content
//

/*!
	    db_content_t.
	     ,
	     
	delete.
*/
std::auto_ptr< db_content_t >
create_std_db_content();

} /* namespace db_content */

} /* namespace impl */

} /* namespace db */

} /* namespace oess_1 */

#endif
