/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\file
	\since v.1.4.0
	\brief    .
*/

#if !defined( _OESS_1__DB__SITE__IMPL__SLICE_IS_EXISTS_CMD_HPP_ )
#define _OESS_1__DB__SITE__IMPL__SLICE_IS_EXISTS_CMD_HPP_

#include <oess_1/db/site/impl/h/request.hpp>
#include <oess_1/db/site/impl/h/response.hpp>

namespace oess_1 {

namespace db {

namespace site {

namespace impl {

//
// slice_is_exists_cmd_t
//

/*!
	\since v.1.4.0
	\brief       .
*/
class	slice_is_exists_cmd_t :
	public request_t
{
	OESS_SERIALIZER( slice_is_exists_cmd_t )
	public :
		slice_is_exists_cmd_t();
		slice_is_exists_cmd_t(
			//! ,   .
			const std::string & client_id,
			//! ,     .
			const std::string & db_logic_name,
			//!   .
			const std::string & slice_name );
		virtual ~slice_is_exists_cmd_t();

		enum {
			e_id = 1200
		};

		virtual oess_1::uint_t
		query_req_id() const;

		//!  ,     .
		const std::string &
		query_db_logic_name() const;

		//!   .
		const std::string &
		query_slice_name() const;

	private :
		//! ,     .
		std::string	m_db_logic_name;
		//!   .
		std::string	m_slice_name;
};

//
// slice_is_exists_cmd_result_t
//

/*!
	\since v.1.4.0
	\brief    slice_is_exists.
*/
class	slice_is_exists_cmd_result_t :
	public response_t
{
	OESS_SERIALIZER( slice_is_exists_cmd_result_t )
	public :
		/*!
			  ,    .
		*/
		slice_is_exists_cmd_result_t();
		//!      .
		slice_is_exists_cmd_result_t(
			//! true,   .
			bool is_exists );
		//!    .
		slice_is_exists_cmd_result_t(
			const result_t & result );
		virtual ~slice_is_exists_cmd_result_t();

		enum {
			e_id = 1201
		};

		virtual oess_1::uint_t
		query_resp_id() const;

		/*!
			\return true,   .
		*/
		bool
		is_exists() const;

	private :
		enum
			{
				//!  ,    .
				not_exists = 0,
				//!  ,   .
				exists = 1
			};

		//!   .
		oess_1::uchar_t	m_existence;
};

} /* namespace impl */

} /* namespace site */

} /* namespace db */

} /* namespace oess_1 */

#endif

