/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/

/*!
	\file
	\since v.1.4.0
	\brief    .
*/

#include <oess_1/stdsn/h/inout_templ.hpp>

#include <oess_1/db/site/impl/h/slice_is_exists_cmd.hpp>

namespace oess_1 {

namespace db {

namespace site {

namespace impl {

//
// slice_is_exists_cmd_t
//

slice_is_exists_cmd_t::slice_is_exists_cmd_t()
{
}

slice_is_exists_cmd_t::slice_is_exists_cmd_t(
	const std::string & client_id,
	const std::string & db_logic_name,
	const std::string & slice_name )
:	request_t( client_id )
,	m_db_logic_name( db_logic_name )
,	m_slice_name( slice_name )
{
}

slice_is_exists_cmd_t::~slice_is_exists_cmd_t()
{
}

oess_1::uint_t
slice_is_exists_cmd_t::query_req_id() const
{
	return e_id;
}

const std::string &
slice_is_exists_cmd_t::query_db_logic_name() const
{
	return m_db_logic_name;
}

const std::string &
slice_is_exists_cmd_t::query_slice_name() const
{
	return m_slice_name;
}

//
// slice_is_exists_cmd_result_t
//

slice_is_exists_cmd_result_t::slice_is_exists_cmd_result_t()
:	m_existence( not_exists )
{
}

slice_is_exists_cmd_result_t::slice_is_exists_cmd_result_t(
	bool is_exists )
:	m_existence( is_exists ? exists : not_exists )
{
}

slice_is_exists_cmd_result_t::slice_is_exists_cmd_result_t(
	const result_t & result )
:	response_t( result )
,	m_existence( not_exists )
{
}

slice_is_exists_cmd_result_t::~slice_is_exists_cmd_result_t()
{
}

oess_1::uint_t
slice_is_exists_cmd_result_t::query_resp_id() const
{
	return e_id;
}

bool
slice_is_exists_cmd_result_t::is_exists() const
{
	return ( not_exists != m_existence );
}

#include "slice_is_exists_cmd.ddl.cpp"

} /* namespace impl */

} /* namespace site */

} /* namespace db */

} /* namespace oess_1 */

