/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief   .
*/

#if !defined( _OESS_1__DB__STORAGE__CHAIN_STORAGE_HPP_ )
#define _OESS_1__DB__STORAGE__CHAIN_STORAGE_HPP_

#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/io/h/stream.hpp>
#include <oess_1/io/h/mem_buf.hpp>

#include <oess_1/stdsn/h/ent_std.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/storage/h/chain_id.hpp>
#include <oess_1/db/storage/h/config.hpp>

namespace oess_1 {

namespace db {

namespace storage {

//
// no_chain_length_limit
//

/*!
	\since v.1.4.0
	\brief ,      
		  .

	      \a max_length
	 chain_storage_t::chain_load().
*/
const oess_1::uint_t no_chain_length_limit =
		static_cast< oess_1::uint_t >( -1 );

//
// chain_storage_t
//

/*!
	\since v.1.4.0
	\brief        .

	,       
	 create_chain_storage()  open_chain_storage().

	,     . , 
	        chain_storage_t.
*/
class OESS_1__DB__TYPE	chain_storage_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~chain_storage_t();

	//! \name     .
	//! \{
		//!  .
		/*!
			\throw oess_1::logic_ex_t     
				    read-only.
		*/
		virtual void
		trx_start() = 0;

		//!   .
		/*!
			\throw oess_1::logic_ex_t    .
		*/
		virtual void
		trx_commit() = 0;

		//!   .
		/*!
			\throw oess_1::logic_ex_t    .
		*/
		virtual void
		trx_rollback() = 0;
	//! \}

	//! \name     .
	//! \{
		//!  .
		/*!
			\return   .
		*/
		virtual chain_id_t
		chain_create(
			//!      .
			oess_1::io::istream_t & from,
			//!     .
			oess_1::uint_t length ) = 0;

		//!  .
		virtual void
		chain_destroy(
			//!   .
			const chain_id_t & chain ) = 0;

		//!    .
		/*!
			   \a max_length,   \a to 
			  .     \a to 
			max_length .
		*/
		virtual void
		chain_load(
			//!   .
			const chain_id_t & chain,
			//!   .
			oess_1::io::ostream_t & to,
			//!    .
			oess_1::uint_t max_length ) = 0;

		//!   .
		/*!
			    /  ,
			      /.
		*/
		virtual void
		chain_update(
			//!   .
			const chain_id_t & chain,
			//!     .
			oess_1::io::istream_t & from,
			//!      .
			oess_1::uint_t length ) = 0;
	//! \}
	};

//! \name     .
//! \{

//
// create_chain_storage
//

/*!
	\since v.1.4.0
	\brief   .

	   .     ,
	  .

	      . ,
	       .  
	       .

	     
	chain_storage_t.        
	 .
*/
OESS_1__DB__FUNC(std::auto_ptr< chain_storage_t >)
create_chain_storage(
	//!    .
	const std::string & physic_name,
	//!    .
	const config_t & config,
	//!        .
	oess_1::io::istream_t & from,
	//!  ,     
	//!   .
	oess_1::uint_t length,
	//!     .
	chain_id_t & first_chain );

//
// open_chain_storage
//

/*!
	\since v.1.4.0
	\brief   .

	      ,
	      .

	     
	chain_storage_t.        
	 .

	\par   .
		    
		 .    
		   .  
		  :    
		\a config      -,
		    \a config.   
		    .
*/
OESS_1__DB__FUNC(std::auto_ptr< chain_storage_t >)
open_chain_storage(
	//!    .
	const std::string & physic_name,
	//!   .
	const config_t & config,
	//! true,      read-only.
	bool is_read_only,
	//! true,       .
	bool is_auto_repair_enabled,
	//!        .
	oess_1::io::ostream_t & to,
	//!    ,   
	//!    .
	oess_1::uint_t length,
	//!     .
	chain_id_t & first_chain );

//
// is_chain_storage_exists
//

/*!
	\since v.1.4.0
	\brief   .

	  , :
	-     trace-, 
	-      ""   .
*/
OESS_1__DB__FUNC(bool)
is_chain_storage_exists(
	//!   ,    .
	const std::string & physic_name );

//
// destroy_chain_storage
//

/*!
	\since v.1.4.0
	\brief  .

	  ,     
	.
*/
OESS_1__DB__FUNC(void)
destroy_chain_storage(
	//!    .
	const std::string & physic_name );

//! \}

/*! \name       
	 .
	\{ */

//!    .
template< class T >
chain_id_t
create_in_storage(
	chain_storage_t & storage, const T & what )
	{
		oess_1::io::mem_buf_t stream;
		oess_1::stdsn::oent_std_t oent( stream );
		oent << what;
		stream.set_pos( 0 );

		return storage.chain_create( stream, stream.size() );
	}

//!    .
template< class T >
void
load_from_storage(
	chain_storage_t & storage,
	const chain_id_t & chain,
	T & what,
	oess_1::uint_t length = no_chain_length_limit )
	{
		oess_1::io::mem_buf_t stream;
		storage.chain_load( chain, stream, length );

		stream.set_pos( 0 );
		oess_1::stdsn::ient_std_t ient( stream );
		ient >> what;
	}

//!     .
template< class T >
void
update_in_storage(
	chain_storage_t & storage,
	const chain_id_t & chain,
	const T & what )
	{
		oess_1::io::mem_buf_t stream;
		oess_1::stdsn::oent_std_t oent( stream );
		oent << what;
		stream.set_pos( 0 );

		return storage.chain_update( chain, stream, stream.size() );
	}

/*! \} */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

