/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief    .
*/

#if !defined( _OESS_1__DB__STORAGE__CONFIG_HPP_ )
#define _OESS_1__DB__STORAGE__CONFIG_HPP_

#include <string>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/stdsn/h/serializable.hpp>

#include <oess_1/db/h/declspec.hpp>

namespace oess_1 {

namespace db {

namespace storage {

//! \name   / .
//! \{

//!      .
/*!
	\since v.1.4.0
*/
typedef oess_1::uchar_t	trace_mode_t;

//! ,       .
/*!
	\since v.1.4.0

	       ,  
	  .
*/
const trace_mode_t	trace_no = 0;

//! ,        .
/*!
	\since v.1.4.0

	    trace-     -- .
	       
	  trace- .

	     .   
	   ,    , 
	         .

	 ,       
	    (,    
	)    , .. 
	    .
*/
const trace_mode_t	trace_last = 1;

//! ,       .
/*!
	\since v.1.4.0

	         
	trace-stream.       
	  trace-. ..   
	      ,  
	      trace-.

	         
	   .
*/
const trace_mode_t	trace_all = 2;

//! \}

//
// hard_config_t
//

/*!
	\since v.1.4.0
	\brief  ,    
		   .
*/
class	OESS_1__DB__TYPE	hard_config_t
	:	public oess_1::stdsn::serializable_t
	{
		typedef oess_1::stdsn::serializable_t	base_type_t;
		OESS_SERIALIZER( hard_config_t )
	public :
		//!    -.
		enum	defaults_t
			{
				//!    -.
				default_page_size = 4096,
				//!       -.
				default_chain_link_size = 256,
				//!        
				//!    -.
				default_trx_delta_block_size = 4096,
				//!    trace- -.
				default_trace_file_page_size = 512,
				//!     -.
				default_trace_mode = trace_last
			};

		//!  -.
		/*!
			    -.
		*/
		hard_config_t();
		//!   .
		/*!
			    , 
			 - , 
			oess_1::logic_ex_t.
		*/
		hard_config_t(
			//!     .
			oess_1::uint_t page_size,
			//!     (   ).
			oess_1::uint_t chain_link_size,
			//!        
			//!     trace-stream.
			oess_1::uint_t trx_delta_block_size,
			//!    trace-.
			oess_1::uint_t trace_file_page_size,
			//!    .
			trace_mode_t trace_mode );
		//!  .
		hard_config_t(
			const hard_config_t & o );
		virtual ~hard_config_t();

		//!  .
		hard_config_t &
		operator=( const hard_config_t & o );

		//!      .
		oess_1::uint_t
		page_size() const;

		//!      .
		oess_1::uint_t
		chain_link_size() const;

		//!        
		//!     trace-stream.
		oess_1::uint_t
		trx_delta_block_size() const;

		//!   trace-  .
		oess_1::uint_t
		trace_file_page_size() const;

		//!    .
		trace_mode_t
		trace_mode() const;

	private :
		//!      .
		oess_1::uint_t	m_page_size;

		//!      .
		oess_1::uint_t	m_chain_link_size;

		//!        
		//!     trace-stream.
		oess_1::uint_t	m_trx_delta_block_size;

		//!   trace-  .
		oess_1::uint_t	m_trace_file_page_size;

		//!    .
		trace_mode_t	m_trace_mode;
	};

//! \name     trace-stream.
//! \{

//!        trace-stream.
/*!
	\since v.1.4.0
*/
typedef oess_1::uchar_t	trace_compression_mode_t;

//!  .
/*!
	\since v.1.4.0

	     
	     (, - 
	       
	)      (,
	    
	).      
	      .
*/
const trace_compression_mode_t
trace_no_compression = 0;

//!       zlib.
/*!
	\since v.1.4.0
*/
const trace_compression_mode_t
trace_zlib_compression = 1;

//!  ,    -.
/*!
	\since v.1.4.0
*/
const trace_compression_mode_t
trace_default_compression = trace_no_compression;

//! \}

//
// config_t
//

/*!
	\since v.1.4.0
	\brief ,   .

	    (.. ,   
	   )   (.. ,  
	      ).  
	  hard_config_t.
*/
class OESS_1__DB__TYPE	config_t
	{
	public :
		//!    -.
		enum	defaults_t
			{
				//!   -.
				default_cache_size = 256,
				//!    trace- -.
				default_trace_file_size_limit = 10u * 1024 * 1024,
				//!  ,   trace-stream.
				default_trace_stream_buffered_pages = 128,
				//!   trace-stream.
				default_trace_stream_compression =
						trace_default_compression
			};

		//!  -.
		/*!
			    -.
		*/
		config_t();
		//!   .
		/*!
			   ,  
			   -.
		*/
		config_t(
			const hard_config_t & o );
		~config_t();

	//! \name      .
	//! \{
		//!    .
		void
		set_hard_config(
			const hard_config_t & config );

		//!      .
		oess_1::uint_t
		page_size() const;

		//!      .
		oess_1::uint_t
		chain_link_size() const;

		//!        
		//!     trace-stream.
		oess_1::uint_t
		trx_delta_block_size() const;

		//!   trace-  .
		oess_1::uint_t
		trace_file_page_size() const;

		//!    .
		trace_mode_t
		trace_mode() const;
	//! \}

	//! \name      .
	//! \{
		//!  .
		/*!
			\return   ,  
				   .
		*/
		oess_1::uint_t
		cache_size() const;

		//!   .
		void
		set_cache_size( oess_1::uint_t v );

		//!    trace-  .
		oess_1::uint_t
		trace_file_size_limit() const;

		//!    trace-  .
		void
		set_trace_file_size_limit( oess_1::uint_t v );

		//!   ,  trace-stream 
		//!    trace-    .
		/*!
			  .
		*/
		oess_1::uint_t
		trace_stream_buffered_pages() const;

		//!    ,  trace-stream
		//!     trace-  .
		void
		set_trace_stream_buffered_pages( oess_1::uint_t v );

		//!   trace-stream.
		trace_compression_mode_t
		trace_compression() const;

		//!    trace-stream.
		void
		set_trace_compression(
			const trace_compression_mode_t & v );
	//! \}

	//! \name      .
	//! \{
		//!      .
		std::string
		make_cfg_file_name(
			const std::string & physic_name ) const;

		//!      .
		std::string
		make_main_file_name(
			const std::string & physic_name ) const;

		//!   ""    .
		std::string
		make_unclear_file_name(
			const std::string & physic_name ) const;

		//!       .
		/*!      . */
		std::string
		make_tmp_main_file_name(
			const std::string & physic_name ) const;

		//!    ""  
		//!  .
		/*!      . */
		std::string
		make_tmp_unclear_file_name(
			const std::string & physic_name ) const;
	//! \}

	private :
		//!   .
		hard_config_t	m_hard_config;

	//! \name   .
	//! \{
		//!      .
		oess_1::uint_t	m_cache_size;

		//!   trace-  .
		oess_1::uint_t	m_trace_file_size_limit;

		//!    trace-stream   trace-.
		oess_1::uint_t	m_trace_stream_buffered_pages;

		//!   trace-stream.
		trace_compression_mode_t	m_trace_compression;
	//! \}
	};

//
// create_cfg_file
//

/*!
	\since v.1.4.0
	\brief      .
*/
OESS_1__DB__FUNC(void)
create_cfg_file(
	//!    .
	const std::string & file_name,
	//!    .
	const config_t & config );

//
// load_cfg_file
//

/*!
	\since v.1.4.0
	\brief      .
*/
OESS_1__DB__FUNC(config_t)
load_cfg_file(
	//!   .
	const std::string & file_name );

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

