/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/
#if !defined( _OESS_1__DB__STORAGE__DIRTY_PAGE_STORAGE_HPP_ )
#define _OESS_1__DB__STORAGE__DIRTY_PAGE_STORAGE_HPP_

#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/db/storage/h/loaded_page.hpp>

namespace oess_1 {

namespace db {

namespace storage {

//
// dirty_page_actor_t
//

/*!
	\since v.1.3.1
	\brief    ,   
		-     dirty page storage.
*/
class	OESS_1__DB__TYPE	dirty_page_actor_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~dirty_page_actor_t();

		//!        .
		virtual void
		operator()(
			//!     .
			oess_1::uint_t page_ordinal,
			//!  .
			const oess_1::char_t * page,
			//!   .
			//!  0 ,     ,
			//! ..     .
			//!      
			//!      .
			const oess_1::char_t * source_value,
			//!    .
			oess_1::uint_t page_size ) = 0;
	};

//
// dirty_page_storage_t
//

/*!
	\since v.1.3.1
	\brief    ,   
		      .
*/
class	OESS_1__DB__TYPE	dirty_page_storage_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~dirty_page_storage_t();

		//!    .
		/*!
			      ,    .
		*/
		virtual void
		put(
			const ref_loaded_page_t & page ) = 0;

		//!      .
		/*!
			  ,   \a page  
			  .
 
			\note     
				  clear().

			\return true,   .   
				 false.
		*/
		virtual bool
		find(
			//!     .
			oess_1::uint_t page_ordinal,
			//!    .
			ref_loaded_page_t & receiver ) const = 0;

		//!      .
		/*!
			\note      
				,   
				       
				  .
		*/
		virtual void
		for_each( dirty_page_actor_t & actor ) const = 0;

		//!  .
		/*!
			    .
		*/
		virtual void
		clear() = 0;

		//!    .
		/*!
			\return  ,   
				     clear.

			 0 ,      
			   .
		*/
		virtual oess_1::uint_t
		size() const = 0;
	};

//
// create_std_dirty_page_storage
//

/*!
	\since v.1.3.1
	\brief      .

	\return     .
*/
std::auto_ptr< dirty_page_storage_t >
create_std_dirty_page_storage();

} /* namespace storage */

} /* namespace db */

} /* namespace oess */

#endif

