/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*
	ObjESSty-1
*/
#if !defined( _OESS_1__DB__STORAGE__ERRNO_HPP_ )
#define _OESS_1__DB__STORAGE__ERRNO_HPP_

#include <oess_1/defs/h/errno_range.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace err {

const int	c_range_min = oess_1::impl::c_errno__db__storage__min;
const int	c_range_max = c_range_min + 1000;

//   .
const int	c_storage_open = c_range_min + 1;

//    .
const int	c_trans_not_started = c_range_min + 2;

//   .
const int	c_trans_started = c_range_min + 3;

//   .
const int	c_invalid_page_index = c_range_min + 4;

//    .
const int	c_storage_damage_possible = c_range_min + 5;

//     read-only.
const int	c_read_only_mode = c_range_min + 6;

//   .
const int	c_storage_not_open = c_range_min + 7;

//    .
const int	c_stream_index_open = c_range_min + 8;

//    .
const int	c_stream_index_not_open = c_range_min + 9;

//       .
const int	c_invalid_ind1_item_ordinal = c_range_min + 10;

//   .
const int	c_unexpected = c_range_min + 11;

//      ,
//  NULL.
const int	c_null_index_search_result = c_range_min + 12;

//   .
const int	c_stream_open = c_range_min + 13;

//   .
const int	c_stream_not_open = c_range_min + 14;

//       ,
//     .
const int	c_entity_too_large = c_range_min + 15;

//   .
const int	c_stream_full = c_range_min + 16;

//     .
const int	c_invalid_block_header = c_range_min + 17;

//!      .
const int	c_stream_is_not_in_directory = c_range_min + 18;

//      .
const int	c_unknown_stream = c_range_min + 19;

//     .
const int	c_page_not_in_cache = c_range_min + 20;

//!     .
const int	c_page_already_in_cache = c_range_min + 21;

//!   .
const int	c_cache_not_initialized = c_range_min + 22;

//!        .
const int	c_trx_invalid_lists = c_range_min + 23;

//!        
//!       .
const int	c_trx_invalid_mismatch_block_size = c_range_min + 24;

//! Trace-stream    .
const int	c_trace_stream_open = c_range_min + 25;

//!     trace-  .
const int	c_trace_file_not_found = c_range_min + 26;

//!  trace-,     .
const int	c_trace_file_found = c_range_min + 27;

//! Trace-stream    .
const int	c_trace_stream_not_open = c_range_min + 28;

//! Trace-stream      read-only  
//!   .
const int	c_read_only_trace_stream = c_range_min + 29;

//!      -  
//!  trace-stream.
/*!
	,  () trace-  
	.
*/
const int	c_insufficent_trace_info = c_range_min + 31;

//!      .
/*!
	    trace-   
	 ,     . 
	  .     
	 .

	    trace-   ,
	        
	.   1.4.0  1  8.   ,
	        
	.
*/
const int	c_invalid_page_block_count = c_range_min + 32;

//!    .
const int	c_trx_zlib_failure = c_range_min + 33;

//!       .
const int	c_trx_unknown_compression_mode = c_range_min + 34;

//!   trace-  .
const int	c_trx_file_search_failed = c_range_min + 35;

//!        .
const int	c_no_more_free_segments = c_range_min + 36;

//!   .
/*!
	      
	      .   
	       .

	  ,    
	     .
*/
const int	c_invalid_segment_ordinal = c_range_min + 37;

//!     ,     .
const int	c_segment_already_full = c_range_min + 38;

//!     ,     .
const int	c_segment_already_free = c_range_min + 39;

//!        .
const int	c_block_not_allocated = c_range_min + 40;

//!    .
/*!
	,      
	  .      
	-  (   ).
*/
const int	c_invalid_chain_link_ordinal = c_range_min + 41;

//!         .
const int	c_invalid_sizes = c_range_min + 42;

//!   .
/*!
	      
	      .
	  ,    
	     .
*/
const int	c_invalid_area_ordinal = c_range_min + 43;

//!   a     .
const int	c_area_already_full = c_range_min + 44;

//!   a     .
const int	c_area_already_free = c_range_min + 45;

//!        .
//!   .
const int	c_no_more_free_areas = c_range_min + 47;

//!        .
/*!
	       ,   ,
	        .
*/
const int	c_stream_name_not_unique = c_range_min + 48;

//!    trace-stream.
const int	c_invalid_trace_compression_mode = c_range_min + 49;

//!     .
/*!
	\since v.1.4.0

	     ,    trace-mode 
	  ,     . ,
	  committer,    
	 trace_last,   trace_all.
*/
const int	c_invalid_trace_mode = c_range_min + 50;

//! Trace-stream        
//! .
const int	c_trace_stream_full = c_range_min + 51;

/*!
	\since v.1.4.0
	\brief      snapshot-.
*/
const int	c_no_actual_snapshot = c_range_min + 52;

/*!
	\since v.1.4.0
	\brief   .

	       
	     
	.
*/
const int	c_unable_to_rename_file = c_range_min + 53;

/*!
	\since v.1.4.0
	\brief       .

	    snapshot-   
	 ,   snapshot-   ,  
	   . ,   ,  snapshot-  
	  .
*/
const int	c_db_page_count_mismatch = c_range_min + 54;

/*!
	\since v.1.4.0
	\brief  trace-,    
		    snapshot-.

	       snapshot-
	 ,     snapshot-  
	  trace-.    trace- 
	snapshot  ,    snapshot   
	 .      ,    
	  trace- (..       
	  trace-),  snapshot-  .
*/
const int	c_no_traces_for_snapshot = c_range_min + 55;

/*!
	\since v.1.4.0
	\brief       .

	, ,     -.
*/
const int	c_zero_total_page_count = c_range_min + 56;

} /* namespace err */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif
