/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief     / 
		      .
*/

#if !defined( _OESS_1__DB__STORAGE__PAGE_INDEX_CALCULATOR_HPP_ )
#define _OESS_1__DB__STORAGE__PAGE_INDEX_CALCULATOR_HPP_

#include <oess_1/defs/h/types.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/storage/h/chain_id.hpp>

namespace oess_1 {

namespace db {

namespace storage {

//
// area_ordinal_t
//

/*!
	\since v.1.4.0
	\brief      .
*/
typedef oess_1::ushort_t area_ordinal_t;

//
// segment_ordinal_t
//

/*!
	\since v.1.4.0
	\brief      .
*/
typedef oess_1::ushort_t segment_ordinal_t;

//
// page_index_calculator_t
//

/*!
	\since v.1.4.0
	\brief     / 
		      .

	       .
	 ,      
	(-    )  .
	..       .
*/
class	OESS_1__DB__TYPE	page_index_calculator_t
	{
	public :
		//!  .
		page_index_calculator_t(
			//!     .
			oess_1::uint_t page_size,
			//!     (  ).
			oess_1::uint_t chain_link_size );

		//!     .
		oess_1::uint_t
		common_header_page() const;

		//!     .
		area_ordinal_t
		area_count() const;

		//!       .
		segment_ordinal_t
		area_segment_count(
			const area_ordinal_t & area ) const;

		//!       .
		oess_1::uint_t
		area_header_page(
			const area_ordinal_t & area ) const;

		//!      .
		oess_1::uint_t
		segment_header_page(
			//! ,    .
			const area_ordinal_t & area,
			const segment_ordinal_t & segment ) const;

		//!         .
		oess_1::uint_t
		segment_page_count() const;

		//!     .
		oess_1::uint_t
		chain_link_on_page_count() const;

		//!      .
		chain_id_t
		first_chain_link_on_page(
			//!   .
			oess_1::uint_t page_ordinal ) const;

		//!   ,     .
		area_ordinal_t
		chain_link_area(
			const chain_id_t & block ) const;

		//!   ,     .
		segment_ordinal_t
		chain_link_segment(
			const chain_id_t & block ) const;

		//!   ,     .
		oess_1::uint_t
		chain_link_page(
			const chain_id_t & block ) const;

		//!       .
		oess_1::uint_t
		chain_link_page_offset(
			const chain_id_t & block ) const;

		//!   ,     
		//!  .
		/*!
			      .
		*/
		chain_id_t
		first_chain_id() const;

	private :
		//!    .
		oess_1::uint_t m_page_size;
		//!      .
		oess_1::uint_t m_chain_link_size;
		//!    .
		oess_1::uint_t m_page_chain_links;
		//!     .
		oess_1::uint_t m_areas;
		//!      .
		oess_1::uint_t m_area_segments;
		//!     .
		oess_1::uint_t m_last_area_segments;
		//!    .
		oess_1::uint_t m_area_pages;
		//!    .
		oess_1::uint_t m_segment_pages;
	};

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

