/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief    .
*/
#if !defined( _OESS_1__DB__STORAGE__STREAM_STORAGE_HPP_ )
#define _OESS_1__DB__STORAGE__STREAM_STORAGE_HPP_

#include <set>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/db/storage/h/chain_storage.hpp>
#include <oess_1/db/storage/h/entity_stream.hpp>

namespace oess_1 {

namespace db {

namespace storage {

/*
** stream_storage_t
*/

/*!
	\brief    .
*/
class	OESS_1__DB__TYPE	stream_storage_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~stream_storage_t();

	//! \name     .
	//! \{
		//!  .
		virtual void
		trans_start() = 0;

		/*!
			    .
		*/
		virtual void
		trans_commit() = 0;

		/*!
			\since v.1.3.1

			 .
		*/
		virtual void
		trans_rollback() = 0;
	//! \}

	//! \name      .
	//! \{

		//!  .
		/*!
			    .
		*/
		virtual void 
		stream_create(
			//!   .
			const std::string & name ) = 0;

		//!     .
		/*!
			\return true,   .
		*/
		virtual bool
		stream_is_exists(
			//!  ,   .
			const std::string & name ) const = 0;

		//!  .
		/*!
			       
			entity_stream_t,       
			   .
		*/
		virtual void
		stream_destroy(
			const std::string & name ) = 0;

		//!     .
		virtual std::set< std::string >
		stream_names() const = 0;

		//!      .
		/*!
			\note      
				  .    
				       
				stream_destroy.    
				       
				     .

			\throw oess_1::logic_ex_t,    .
		*/
		virtual entity_stream_t &
		stream_get(
			//!   .
			const std::string & name ) = 0;

		//!      .
		/*!
			\note      
				  .    
				       
				stream_destroy.    
				       
				     .

			\throw oess_1::logic_ex_t,    .
		*/
		virtual entity_stream_t &
		stream_get(
			//!   .
			const oess_1::uint_t & id ) = 0;
	//! \}
};

//
// create_stream_storage
//

/*!
	\since v.1.4.0
	\brief   .

	   .     ,
	  .

	     
	stream_storage_t.        
	 .

	\return     ,  
		    .
*/
OESS_1__DB__FUNC(stream_storage_t *)
create_stream_storage(
	//!    .
	const std::string & physic_name,
	//!    .
	const config_t & config );

//
// open_stream_storage
//

/*!
	\since v.1.4.0
	\brief   .

	      ,
	      .

	     
	stream_storage_t.        
	 .

	\return     ,  
		    .
*/
OESS_1__DB__FUNC(stream_storage_t *)
open_stream_storage(
	//!    .
	const std::string & physic_name,
	//!   .
	const config_t & config,
	//! true,      read-only.
	bool is_read_only,
	//! true,       .
	bool is_auto_repair_enabled );

//
// is_stream_storage_exists
//

/*!
	\since v.1.4.0
	\brief   .

	  , :
	-     trace-, 
	-      ""   .
*/
OESS_1__DB__FUNC(bool)
is_stream_storage_exists(
	//!   ,    .
	const std::string & physic_name );

//
// destroy_stream_storage
//

/*!
	\since v.1.4.0
	\brief  .

	  ,     
	.
*/
OESS_1__DB__FUNC(void)
destroy_stream_storage(
	//!    .
	const std::string & physic_name );

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif
