/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief  ,     
		    .
*/

#if !defined( _OESS_1__DB__STORAGE__IMPL__HEADER_PROCESSOR_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__HEADER_PROCESSOR_HPP_

#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/db/storage/h/chain_storage.hpp>

#include <oess_1/db/storage/impl/h/page_accessor.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// header_processor_t
//

/*!
	\since v.1.4.0
	\brief  ,     
		    .

	      --   
	  .  ,      
	   --   , 
	       
	 (  ).    
	     (..    
	  ).

	         
	std_chain_storage_t.     ,   
	    ,  
	 ,   .

	  -  (, ,
	,  ) \b 
	  operation_finished()    
	   (   )   
	   (..    
	      
	 ).

	    header_processor_t 
	  create_std_header_processor().
*/
class	header_processor_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~header_processor_t();

		//!   .
		/*!
			       
			 invalid_chain_id.
		*/
		virtual chain_id_t
		allocate_block() = 0;

		//!   .
		virtual void
		deallocate_block(
			//!   .
			const chain_id_t & block ) = 0;

		//!        .
		virtual chain_id_t
		get_next(
			//!    .
			const chain_id_t & block ) = 0;

		//!        .
		virtual void
		change_next(
			//!    .
			const chain_id_t & block,
			//! ,      .
			const chain_id_t & next ) = 0;

		//!        
		//!  .
		virtual void
		operation_finished(
			//! true,   read-only  (,
			//!    ).
			bool is_read_only = false ) = 0;
	};

//
// create_std_header_processor
//

/*!
	\since v.1.4.0
	\brief     %header_processor_t.

	\return     , 
		      .
*/
std::auto_ptr< header_processor_t >
create_std_header_processor(
	//!   .
	const config_t & config,
	//!      .
	page_accessor_t & accessor,
	//! true,       .
	bool is_read_only );

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif


