/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief     .
*/

#if !defined( _OESS_1__DB__STORAGE__IMPL__HEADERS_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__HEADERS_HPP_

#include <set>
#include <vector>

#include <cpp_util_2/h/nocopy.hpp>

#include <smart_ref_3/h/smart_ref.hpp>

#include <oess_1/db/storage/h/page_index_calculator.hpp>

#include <oess_1/db/storage/impl/h/segment_header_item.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// common_header_t
//

/*!
	\since v.1.4.0
	\brief     .

	    "".
	  ,     
	: init_new(), remove_free_area(), return_free_area().
	   ,     
	   ,    
	   .
*/
class	common_header_t
	{
	public :
		//!  .
		common_header_t(
			//!  .
			const page_index_calculator_t & calculator );

		//!   ,   
		//!  .
		/*!
			\throw oess_1::logic_ex_t,    
				  .
		*/
		area_ordinal_t
		get_free_area() const;

		//!      ,  
		//!   .
		/*!
			   .
		*/
		void
		remove_free_area(
			const area_ordinal_t & area );

		//!      , 
		//!   .
		/*!
			   .
		*/
		void
		return_free_area(
			const area_ordinal_t & area );

		//!      .
		/*!
			   .
		*/
		void
		init_new();

		//!      .
		/*!
			   .
		*/
		oess_1::io::ibinstream_t &
		read_from(
			oess_1::io::ibinstream_t & from );

		//!      .
		/*!
			    .
		*/
		oess_1::io::obinstream_t &
		write_to(
			oess_1::io::obinstream_t & to ) const;

		//!    ?
		bool
		is_changed() const;

		//!    .
		void
		force_not_changed();

		//!     ?
		/*!
			\return true,  .
		*/
		bool
		has_free_areas() const;

	private :
		//!  .
		page_index_calculator_t	m_calculator;

		//!       .
		typedef std::set< area_ordinal_t > area_ordinal_set_t;

		//!      .
		area_ordinal_set_t	m_free_areas;

		//! true,    .
		bool	m_is_changed;

		//!   ,     .
		/*!
			  ,    
			 m_free_area.

			\note    get_free_area.
		*/
		area_ordinal_t	m_first_free_area;
	};

//!    .
inline oess_1::io::obinstream_t &
operator<<(
	oess_1::io::obinstream_t & to,
	const common_header_t & what )
	{
		return what.write_to( to );
	}

//!    .
inline oess_1::io::ibinstream_t &
operator>>(
	oess_1::io::ibinstream_t & from,
	common_header_t & what )
	{
		return what.read_from( from );
	}

//
// ref_common_header_t
//

SMART_REF_3_DECL_REF( common_header_t, class )

//
// area_header_t
//

/*!
	\since v.1.4.0
	\brief     .

	    "".
	  ,     
	: init_new(), remove_free_segment(), return_free_segment().
	   ,     
	   ,    
	   .
*/
class	area_header_t
	{
	public :
		/*!
			\since v.1.4.0
			\brief      -.

			 ,   / 
			  ,      .
			        .
			   ,   / 
			        
			 .      
			-       
			/    .

			 io_buffer_t   ,  ,
			  ,   ,  
			    /.
		*/
		class	io_buffer_t
			:	private cpp_util_2::nocopy_t
			{
				public :
					//!  .
					io_buffer_t(
						const page_index_calculator_t & calculator );
					~io_buffer_t();

					//!   .
					inline segment_ordinal_t
					size() const
						{
							return m_buffer.size();
						}

					//!     .
					inline segment_ordinal_t *
					get()
						{
							return &m_buffer[ 0 ];
						}

				private :
					//!  .
					std::vector< segment_ordinal_t >	m_buffer;
			};
		//!  .
		area_header_t(
			//!  .
			const page_index_calculator_t & calculator,
			//!   .
			const area_ordinal_t & ordinal );

		//!    .
		const area_ordinal_t &
		area_ordinal() const;

		//!   ,   
		//!  .
		/*!
			\throw oess_1::logic_ex_t,    
				  .
		*/
		segment_ordinal_t
		get_free_segment() const;

		//!      ,  
		//!   .
		/*!
			   .
		*/
		void
		remove_free_segment(
			const segment_ordinal_t & segment );

		//!      , 
		//!   .
		/*!
			   .
		*/
		void
		return_free_segment(
			const segment_ordinal_t & segment );

		//!      .
		/*!
			   .
		*/
		void
		init_new();

		//!      .
		/*!
			   .
		*/
		oess_1::io::ibinstream_t &
		read_from(
			oess_1::io::ibinstream_t & from,
			io_buffer_t & io_buffer );

		//!      .
		/*!
			    .
		*/
		oess_1::io::obinstream_t &
		write_to(
			oess_1::io::obinstream_t & to,
			io_buffer_t & io_buffer ) const;

		//!    ?
		bool
		is_changed() const;

		//!    .
		void
		force_not_changed();

		//!     ?
		/*!
			\return true,  .
		*/
		bool
		has_free_segments() const;

	private :
		//!  .
		page_index_calculator_t	m_calculator;

		//!    .
		area_ordinal_t m_area_ordinal;

		//!       .
		typedef std::set< segment_ordinal_t > segment_ordinal_set_t;

		//!      .
		segment_ordinal_set_t	m_free_segments;

		//! true,    .
		bool	m_is_changed;

		//!   ,     .
		/*!
			  ,    
			 m_free_segments.

			\note    get_free_segment.
		*/
		segment_ordinal_t	m_first_free_segment;
	};

//
// ref_area_header_t
//

SMART_REF_3_DECL_REF( area_header_t, class )

//
// segment_header_t
//

/*!
	\since v.1.4.0
	\brief   .

	    "".
	  ,     
	: init_new(), remove_free_segment(), return_free_segment().
	   ,     
	   ,    
	   .
*/
class	segment_header_t
	{
	public :

		/*!
			\since v.1.4.0
			\brief      -.

			 ,   / 
			  ,      .
			        .
			   ,   / 
			        
			 .      
			-       
			/    .

			 io_buffer_t   ,  ,
			  ,   ,  
			    /.
		*/
		class	io_buffer_t
			:	private cpp_util_2::nocopy_t
			{
				public :
					//!  .
					io_buffer_t(
						const page_index_calculator_t & calculator );
					~io_buffer_t();

					//!   .
					inline oess_1::uint_t
					size() const
						{
							return m_buffer.size();
						}

					//!     .
					inline oess_1::uint_t *
					get()
						{
							return &m_buffer[ 0 ];
						}

				private :
					//!  .
					std::vector< oess_1::uint_t >	m_buffer;
			};

		//!  .
		segment_header_t(
			//!  .
			const page_index_calculator_t & calculator,
			//!     .
			const area_ordinal_t & area_ordinal,
			//!    .
			const segment_ordinal_t & ordinal );

		//!     .
		const area_ordinal_t &
		area_ordinal() const;

		//!    .
		const segment_ordinal_t &
		segment_ordinal() const;

		//!    .
		/*!
			   .
		*/
		chain_id_t
		allocate_block();

		//!   .
		/*!
			   .
		*/
		void
		deallocate_block(
			const chain_id_t & block_id );

		//!        .
		/*!
			\throw oess_1::logic_ex_t,  \a block_id 
				.
		*/
		chain_id_t
		next(
			const chain_id_t & block_id ) const;

		//!        .
		/*!
			   .

			\throw oess_1::logic_ex_t,  \a block_id 
				.
		*/
		void
		change_next(
			//! ,     .
			const chain_id_t & block_id,
			//!   .
			const chain_id_t & next );

		//!     .
		/*!
			   .
		*/
		void
		init_new();

		//!      .
		/*!
			   .
		*/
		oess_1::io::ibinstream_t &
		read_from(
			oess_1::io::ibinstream_t & from,
			io_buffer_t & io_buffer );

		//!      .
		/*!
			    .
		*/
		oess_1::io::obinstream_t &
		write_to(
			oess_1::io::obinstream_t & to,
			io_buffer_t & io_buffer ) const;

		//!    ?
		bool
		is_changed() const;

		//!    .
		void
		force_not_changed();

		//!    ?
		/*!
			\return true,  .
		*/
		bool
		has_free_blocks() const;

	private :
		//!  .
		page_index_calculator_t	m_calculator;

		//!     .
		area_ordinal_t m_area_ordinal;

		//!     .
		segment_ordinal_t	m_segment_ordinal;

		//! true,    .
		bool	m_is_changed;

		//!     ,  .
		unsigned int m_total_block_count;

		//!  ,    .
		/*!
			  ,     
			.
		*/
		unsigned int m_useful_block_count;

		//!     .
		/*!
			  ,     
			  m_items.
		*/
		chain_id_t	m_first_chain_id;

		//!     .
		chain_id_t	m_last_chain_id;

		//!      .
		/*!
			   ,    .
		*/
		chain_id_t	m_first_free_id;

		//!   .
		unsigned int	m_free_count;

		//!    .
		/*!
			 first     (true, 
				 ).

			 second     
			.
		*/
		typedef std::pair< bool, chain_id_t >	block_info_t;

		//!    .
		/*!
			    m_useful_block_count.
		*/
		std::vector< block_info_t >	m_items;

		//!       .
		unsigned int
		item_index( const chain_id_t & id ) const;
	};

//
// ref_segment_header_t
//

SMART_REF_3_DECL_REF( segment_header_t, class )

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

