/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief  ,      .
*/

#if !defined( _OESS_1__DB__STORAGE__IMPL__PAGE_ACCESSOR_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__PAGE_ACCESSOR_HPP_

#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/file/h/file.hpp>

#include <oess_1/db/storage/h/chain_storage.hpp>

#include <oess_1/db/storage/impl/h/cache.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// page_accessor_t
//

/*!
	\since v.1.4.0
	\brief  ,      .

	  std_chain_storage_t    
	  .      
	   . , 
	header_processor_t.      
	  .      
	    ,     ,  
	    .   
	    page_accessor_t,  
	    .

	    page_accessor_t  
	 create_std_page_accessor().

	\attention ,   read_access(),
		write_access()     
		  *_access       
		accessor-.
*/
class	page_accessor_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~page_accessor_t();

		//!      .
		/*!
			     ,    
			     
			write_access().

			\return true,      .
		*/
		virtual bool
		is_exists( oess_1::uint_t page ) = 0;

		//!      .
		/*!
			\throw oess_1::logic_ex_t,      
				.
		*/
		virtual const oess_1::char_t *
		read_access( oess_1::uint_t page ) = 0;

		//!      .
		/*!
			      ,   .
			     ,   
			     (   
			)   .

			\throw oess_1::logic_ex_t,     
				    (, 
				   read-only    ).
		*/
		virtual oess_1::char_t *
		write_access( oess_1::uint_t page ) = 0;
	};

//
// create_std_page_accessor
//

/*!
	\since v.1.4.0
	\brief     %page_accessor_t.

	\return     , 
		      .
*/
std::auto_ptr< page_accessor_t >
create_std_page_accessor(
	//!   .
	const config_t & config,
	//!   .
	oess_1::file::file_t & main_file,
	//!    .    
	//!   .
	oess_1::uint_t & main_file_logic_length,
	//!  .
	cache_t & cache,
	//!   .
	dirty_page_storage_t & dirty_pages,
	//! true,      .
	bool is_read_only );

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

