/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief      .
*/

#if !defined( _OESS_1__DB__STORAGE__IMPL__SEGMENT_HEADER_ITEM_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__SEGMENT_HEADER_ITEM_HPP_

#include <oess_1/defs/h/types.hpp>

#include <oess_1/io/h/bin_data_size.hpp>
#include <oess_1/io/h/binstream.hpp>

#include <oess_1/db/storage/h/chain_id.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// segment_header_item_t
//

/*!
	\since v.1.4.0
	\brief      .
*/
class	segment_header_item_t
	{
	public :
		enum {
			//!     .
			image_size =
					oess_1::io::bin_data_size_t< chain_id_t >::image_size +
					oess_1::io::bin_data_size_t< chain_id_t >::image_size,

			//!    oess_1::uint_t, 	
			//!     .
			uints_count = 2
		};

		//!  -.
		segment_header_item_t()
			:	m_self( invalid_chain_id )
			,	m_next( invalid_chain_id )
			{}

		//!  .
		explicit segment_header_item_t(
			//!  ,     .
			const chain_id_t & self,
			//!    .
			const chain_id_t & next = invalid_chain_id )
			:	m_self( self )
			,	m_next( next )
			{}

		//!   ,     .
		const chain_id_t &
		self() const
			{
				return m_self;
			}

		//!      .
		const chain_id_t &
		next() const
			{
				return m_next;
			}

		//!      .
		void
		change_next( const chain_id_t & next )
			{
				m_next = next;
			}

		//!      .
		/*!
			      
			       
			   ,    
			 .

			\return ,     
				.
		*/
		inline oess_1::uint_t *
		write_to(
			//!     .
			oess_1::uint_t * to ) const
			{
				*(to++) = m_self;
				*(to++) = m_next;
            
				return to;
			}

		//!    .
		/*!
			     , 
			      
			     .    
			  .

			\return ,      
				.
		*/
		inline const oess_1::uint_t *
		read_from(
			const oess_1::uint_t * from )
			{
				m_self = *(from++);
				m_next = *(from++);
      
				return from;
			}

		//!  " ".
		/*!    . */
		inline bool
		operator<( const segment_header_item_t & o ) const
			{
				return ( m_self < o.m_self );
			}

	private :
		//!     .
		chain_id_t	m_self;
		//!    .
		chain_id_t	m_next;
	};

//!    .
inline oess_1::uint_t *
operator<<(
	oess_1::uint_t * to,
	const segment_header_item_t & what )
	{
		return what.write_to( to );
	}

//!    .
inline const oess_1::uint_t *
operator>>(
	const oess_1::uint_t * from,
	segment_header_item_t & what )
	{
		return what.read_from( from );
	}

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

