/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\since v.1.4.0
	\brief      .
*/
#if !defined( _OESS_1__DB__STORAGE__IMPL__STD_CACHE_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__STD_CACHE_HPP_

#include <map>

#include <oess_1/db/storage/impl/h/cache.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

/*!
	\since v.1.4.0
	\brief      .
*/
class	std_cache_t
	:	public	cache_t
	{
	public :
		std_cache_t();
		virtual ~std_cache_t();

	//! \name   .
	//! \{
		virtual void
		reinit(
			oess_1::uint_t cache_size,
			oess_1::uint_t page_size );

		virtual void
		cleanup();

		virtual const oess_1::char_t *
		read_access(
			oess_1::uint_t ordinal );

		virtual oess_1::char_t *
		write_access(
			oess_1::uint_t ordinal );

		virtual oess_1::char_t *
		push(
			const ref_loaded_page_t & page,
			dirty_page_storage_t & storage,
			bool is_read_only_access );

		virtual void
		copy_all_dirties(
			dirty_page_storage_t & storage );

		virtual void
		clear_all_dirties();

		virtual void
		throw_away_all_dirties();
	//! \}

		/*!
			  .
		*/
		struct	page_t
			{
				//!  .
				ref_loaded_page_t m_page;

				//!     .
				unsigned long	m_last_access_time;

				//!  ,   ''.
				bool	m_is_dirty;

				//! ,    std::map.
				page_t()
					:	m_last_access_time( 0 )
					,	m_is_dirty( false )
					{}

				//!  .
				page_t(
					//!  .     
					//! ,    .
					const ref_loaded_page_t & page,
					//!  .
					unsigned long last_access_time )
					:	m_page( page )
					,	m_last_access_time( last_access_time )
					,	m_is_dirty( true )
					{
						if( m_page->has_previous() )
							m_is_dirty = false;
					}

				//!    ref_loaded_page_t.
				const ref_loaded_page_t &
				page() const
					{
						return m_page;
					}

				//!       
				//!  .
				oess_1::char_t *
				current(
					unsigned long last_access_time )
					{
						m_last_access_time = last_access_time;
						return m_page->current();
					}

				//!       
				//!  .
				const oess_1::char_t *
				current() const
					{
						return m_page->current();
					}

				//!    .
				unsigned long
				last_access_time() const
					{
						return m_last_access_time;
					}

				//!    ?
				bool
				is_dirty() const
					{
						return m_is_dirty;
					}

				//!   .
				void
				make_dirty()
					{
						m_is_dirty = true;
					}

				//!   .
				/*!
					    
					 .
				*/
				void
				make_clear()
					{
						m_is_dirty = false;
						m_page->change_previous_by_current();
					}
			};

		//!     .
		typedef std::map< oess_1::uint_t, page_t >	page_map_t;

	private :
		//!   .
		oess_1::uint_t	m_max_size;
		//!   .
		oess_1::uint_t	m_page_size;

		//!     .
		unsigned int	m_timer;

		//!  .
		page_map_t	m_pages;

	//! \name     .
	//! \{
		//!  ,       .	
		oess_1::uint_t	m_last_accessed_page_index;
		//!  ,       .
		page_t *	m_last_accessed_page;
	//! \}

		//!  -    	
		//!     .
		void
		push_out_some_page(
			//!   .
			dirty_page_storage_t & storage );
	};

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

