/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.4.0
	\file
	\brief     .
*/

#if !defined( _OESS_1__DB__STORAGE__IMPL__STD_CHAIN_STORAGE_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__STD_CHAIN_STORAGE_HPP_

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/file/h/file.hpp>

#include <oess_1/db/storage/h/dirty_page_storage.hpp>
#include <oess_1/db/storage/h/chain_storage.hpp>

#include <oess_1/db/storage/trx/h/types.hpp>

#include <oess_1/db/storage/impl/h/cache.hpp>
#include <oess_1/db/storage/impl/h/page_accessor.hpp>
#include <oess_1/db/storage/impl/h/header_processor.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// std_chain_storage_t
//

/*!
	\since v.1.4.0
	\brief    %chain_storage_t.
*/
class	std_chain_storage_t
	:	public chain_storage_t
	{
	public :
		/*!
			 .

			   .
		*/
		std_chain_storage_t(
			//!   ,    .
			const std::string & physic_name,
			//!    .
			const std::string & main_file_name,
			//!        .
			//! ,      read-only.
			const std::string & work_file_name,
			//!   .
			const config_t & config,
			//! true,      read-only.
			bool is_read_only,
			//!     ,  ,
			//!   .
			bool is_auto_repair_enabled );

		virtual ~std_chain_storage_t();

	//! \name   .
	//! \{
		virtual void
		trx_start();

		virtual void
		trx_commit();

		virtual void
		trx_rollback();

		virtual chain_id_t
		chain_create(
			oess_1::io::istream_t & from,
			oess_1::uint_t length );

		virtual void
		chain_destroy(
			const chain_id_t & chain );

		virtual void
		chain_load(
			const chain_id_t & chain,
			oess_1::io::ostream_t & to,
			oess_1::uint_t max_length );

		virtual void
		chain_update(
			const chain_id_t & chain,
			oess_1::io::istream_t & from,
			oess_1::uint_t length );
	//! \}

	private :
		//!   ,    .
		const std::string	m_physic_name;
		//!    .
		const std::string	m_main_file_name;
		//!        .
		//! ,      read-only.
		const std::string	m_work_file_name;
		//!   .
		const config_t	m_config;
		//!  .
		page_index_calculator_t	m_calculator;

		//! true,      read-only.
		bool m_is_read_only;

		//! true,     .
		/*!
			    true     
			read-write,     
			    .
		*/
		bool m_is_ok;

		//! true,   .
		bool	m_is_trx;

		/*!
			\brief trace-stream   .
		*/
		std::auto_ptr<
				oess_1::db::storage::trx::trace_stream_t >
			m_trace_stream;

		/*!
			   .
		*/
		oess_1::file::file_t	m_main_file;
		/*!
			  .

			   ,   
			    ,   
			    .

			      
			     .
		*/
		oess_1::uint_t	m_main_file_logic_length;

		//!   .
		std::auto_ptr< cache_t >	m_cache;

		//!    .
		std::auto_ptr< dirty_page_storage_t > m_dirty_pages;

		//!      ,  
		//!     
		//! .
		std::auto_ptr< page_accessor_t > m_page_accessor;

		//!     ,  
		//!  .
		/*!
			     .  
			  .
		*/
		std::auto_ptr< header_processor_t > m_header_processor;

	//! \name   public-.
	//! \{
		//!   .
		void
		lo_trx_commit();

		//!   .
		void
		lo_trx_rollback();

		//!         .
		chain_id_t
		lo_chain_create(
			oess_1::io::istream_t & from,
			oess_1::uint_t length );

		//!   .
		void
		lo_chain_destroy(
			const chain_id_t & head );

		//!   .
		void
		lo_chain_load(
			const chain_id_t & chain,
			oess_1::io::ostream_t & to,	
			oess_1::uint_t max_length );

		//!   .
		void
		lo_chain_update(
			const chain_id_t & chain,
			oess_1::io::istream_t & from,
			oess_1::uint_t length );
	//! \}

		/*!
			      
			 (   
			std::exception ).

			  m_is_ok  .
		*/
		void
		cleanup();

		/*!
			   .

			   ,   
			  read-write   , 
			   .
		*/
		void
		try_open_main_file(
			bool is_repair_enabled );

		/*!
			     trace-stream.

			    unclear-.
		*/
		void
		try_recover_main_file();
		
		/*!
			    .
			     read-write, 
			     unclear-,
			  unclear-.

			  read-only    
			   .
		*/
      void
		try_open_existing_main_file();
		
		/*!
			     ,
			 trace-stream    .
		*/
		void
		initialize_open_storage();

		/*!
			  ,    .
		*/
		void
		ensure_trx_started();

		/*!
			    m_dirty_pages   .
		*/
		void
		make_transaction_log();

		/*!
			    m_dirty_pages    .

			       
			   .
		*/
		void
		commit_transaction_to_main_file();

//FIX ME:    ensure_valid_chain_id,   
//,  chain_id == invalid_chain_id.

		/*!
			// .

			 \a head == invalid_chain_id,   
			     .

			 \a head != invalid_chain_id,   
			  .    
			  (     ),
			 .    
			  .
		*/
		chain_id_t
		write_chain(
			page_accessor_t & page_accessor,
			header_processor_t & header_processor,
			const chain_id_t & head,
			//!   .
			oess_1::io::istream_t & from,
			//!   .
			oess_1::uint_t length );

		/*!
			 .
		*/
		void
		deallocate_chain(
			page_accessor_t & page_accessor,
			header_processor_t & header_processor,
			const chain_id_t & head );

		/*!
			  ,    length .
		*/
		void
		load_chain(
			page_accessor_t & page_accessor,
			header_processor_t & header_processor,
			const chain_id_t & head,
			//!   .
			oess_1::io::ostream_t & to,
			//!   .
			oess_1::uint_t length );

		/*!
			       .

			\return    \a from   
				.
		*/
		oess_1::uint_t
		write_block(
			page_accessor_t & page_accessor,
			//!  ,     .
			const chain_id_t & block,
			//!       .
			oess_1::io::istream_t & from,
			//!     from.
			oess_1::uint_t length );

		/*!
			      .

			\return      .
		*/
		oess_1::uint_t
		load_block(
			page_accessor_t & page_accessor,
			//!  ,     .
			const chain_id_t & block,
			//!   .
			oess_1::io::ostream_t & to,
			//!  .
			oess_1::uint_t length );

		/*!
			  header_processor,   
			  .
		*/
		header_processor_t &
		get_header_processor();
	};

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif


