/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief    %entity_stream_t.
*/
#if !defined( _OESS_1__DB__STORAGE__IMPL__STD_ENTITY_STREAM_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__STD_ENTITY_STREAM_HPP_

#include <oess_1/stdsn/h/serializable.hpp>

#include <oess_1/db/storage/h/entity_stream.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

//
// stream_desc_t
//

/*!
	\since v.1.4.0
	\brief   .

	      
	stream_item_t.       
	   , .. .
*/
class	stream_desc_t
	:	public oess_1::stdsn::serializable_t
	{
		typedef oess_1::stdsn::serializable_t base_type_t;
		OESS_SERIALIZER( stream_desc_t )

	public :
		//!  -.
		stream_desc_t();
		//!  .
		stream_desc_t(
			const stream_desc_t & o );
		virtual ~stream_desc_t();

		//!  .
		stream_desc_t &
		operator=(
			const stream_desc_t & o );

		//!     .
		chain_id_t
		first() const;

		//!     .
		void
		set_first(
			const chain_id_t & chain );

		//!     .
		chain_id_t
		last() const;

		//!      .
		void
		set_last(
			const chain_id_t & chain );

	private :
		//!     .
		chain_id_t	m_first;
		//!     .
		chain_id_t	m_last;
	};

//
// stream_item_t
//

/*!
	\since v.1.4.0
	\brief    .

	       
	 stream_item_t.  stream_item_t   
	  ,     
	  .
*/
class	stream_item_t
	:	public oess_1::stdsn::serializable_t
	{
		typedef oess_1::stdsn::serializable_t base_type_t;
		OESS_SERIALIZER( stream_item_t )

	public :
		//!  -.
		stream_item_t();
		//!        .
		stream_item_t(
			//!   .
			const chain_id_t & description_chain,
			//!  .
			const chain_id_t & prev,
			//!    .
			oess_1::uint_t & length,
			//!     .
			const chain_id_t & entity_chain );
		//!  .
		stream_item_t(
			const stream_item_t & o );
		virtual ~stream_item_t();

		enum
			{
				//!       .
				/*!
					\note   , 
						   .
				*/
				length_limit = 32
			};
	
		//!  .
		stream_item_t &
		operator=(
			const stream_item_t & o );

		//!      .
		chain_id_t
		description_chain() const;

		//!      .
		void
		set_description_chain(
			const chain_id_t & chain );

		//!      .
		chain_id_t
		prev() const;

		//!      .
		void
		set_prev(
			const chain_id_t & chain );

		//!      .
		chain_id_t
		next() const;

		//!      .
		void
		set_next(
			const chain_id_t & chain );

		//!    .
		void
		set_entity_params(
			//!    .
			oess_1::uint_t length,
			//!  ,    .
			const chain_id_t & chain );

		//!     .
		oess_1::uint_t
		length() const;

		//!   ,    .
		const chain_id_t &
		entity_chain() const;

	private :
		//!   .
		chain_id_t	m_description_chain;
		//!  .
		chain_id_t	m_prev;
		//!  .
		chain_id_t	m_next;
		//!    .
		oess_1::uint_t	m_length;
		//!     .
		chain_id_t	m_entity_chain;
	};

/*
** std_entity_stream_t
*/

/*!
	\brief    %entity_stream_t.

	\attention       
		     .  
		        
		   .     
		std_entity_stream_t  ,    
		,      .
*/
class	std_entity_stream_t
	:	public entity_stream_t
	{
	public :
		//!  .
		std_entity_stream_t(
			//! ,     .
			chain_storage_t & storage,
			//!    .
			oess_1::uint_t id,
			//!  ,    
			//!   .
			const chain_id_t & description_chain );
		virtual ~std_entity_stream_t();

	//! \name   .
	//! \{
		virtual oess_1::uint_t
		id() const;

		virtual chain_id_t
		entity_create(
			oess_1::io::istream_t & ent_value,
			oess_1::uint_t ent_size );

		virtual void
		entity_load(
			const chain_id_t & ent_id,
			oess_1::io::ostream_t & ent_value ) const;

		virtual void
		entity_update(
			const chain_id_t & ent_id,
			oess_1::io::istream_t & ent_value,
			oess_1::uint_t ent_size );

		virtual void
		entity_destroy(
			const chain_id_t & ent_id );

		virtual void
		entity_destroy_all();

		virtual chain_id_t 
		entity_find_next(
			const chain_id_t & ent_id ) const;
	//! \}

	private :
		//! ,     .
		chain_storage_t &	m_storage;
		//!    .
		oess_1::uint_t m_id;
		//!  ,    
		//!   .
		chain_id_t	m_description_chain;

		//!     .
		stream_desc_t	m_desc;
	};

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif

