/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief    %stream_storage_t.
*/
#if !defined( _OESS_1__DB__STORAGE__IMPL__STD_STREAM_STORAGE_HPP_ )
#define _OESS_1__DB__STORAGE__IMPL__STD_STREAM_STORAGE_HPP_

#include <map>

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/db/storage/h/chain_storage.hpp>
#include <oess_1/db/storage/h/stream_storage.hpp>

#include <oess_1/db/storage/impl/h/stream_storage_dir.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace impl {

/*
** std_stream_storage_t
*/

/*!
	\brief    %stream_storage_t.
*/
class	std_stream_storage_t
	:	public stream_storage_t
	{
	public :
		//!  .
		std_stream_storage_t(
			//!   .   
			//!  .
			const std::string & physic_name,
			//! ,   .
			//!      , 
			//!       
			//! std_stream_storage_t.
			std::auto_ptr< chain_storage_t > storage,
			//!  -  .
			const chain_id_t & meta_info_id,
			//! ,     -.
			oess_1::io::istream_t & meta_info_stream );
		virtual ~std_stream_storage_t();

	//! \name   .
	//! \{
		virtual void
		trans_start();

		virtual void
		trans_commit();

		virtual void
		trans_rollback();

		virtual void 
		stream_create(
			const std::string & name );

		virtual bool
		stream_is_exists(
			const std::string & name ) const;

		virtual void
		stream_destroy(
			const std::string & name );

		virtual std::set< std::string >
		stream_names() const;

		virtual entity_stream_t &
		stream_get(
			const std::string & name );

		virtual entity_stream_t &
		stream_get(
			const oess_1::uint_t & id );
	//! \}

	private :
		//!   .   
		//!  .
		const std::string	m_physic_name;

		//!  .
		std::auto_ptr< chain_storage_t >	m_storage;

		//!  -  .
		chain_id_t	m_meta_info_id;

		//!  .
		stream_storage_dir_t	m_dir;

		//!  ,      .
		bool	m_is_ok;

		//!    entity_stream.
		typedef std::map< oess_1::uint_t, ref_entity_stream_t >
				entity_stream_map_t;

		//!   entity_stream.
		entity_stream_map_t	m_streams;

	//! \name  public-.
	//! \{
		void 
		lo_stream_create(
			const std::string & name );

		void
		lo_stream_destroy(
			const std::string & name );

		entity_stream_t &
		lo_stream_get(
			const oess_1::uint_t & id );
	//! \}

		//!   ,      .
		/*!
			\throw oess_1::physic_ex_t,  true != m_is_ok.
		*/
		void
		ensure_all_right() const;

		//!   %stream_storage_dir_t   
		//!    .
		/*!
			   entity_stream_t 
			    .
		*/
		void
		parse_meta_info(
			//!     
			//! stream_storage_dir_t.
			oess_1::io::istream_t & meta_info_stream );

		//!   %entity_stream_t   ,
		//!    .
		void
		create_entity_stream_objects();

		//!    - 
		//!  .
		void
		reload_meta_info_on_rollback();

		//!    %m_dir  .
		void
		save_meta_info();
};

} /* namespace impl */

} /* namespace storage */

} /* namespace db */

} /* namespace oess_1 */

#endif
