/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\since v.1.4.0
	\brief      .
*/

#include <algorithm>

#include <oess_1/db/storage/h/loaded_page.hpp>

namespace oess_1 {

namespace db {

namespace storage {

//
// loaded_page_t
//

loaded_page_t::loaded_page_t(
	oess_1::uint_t ordinal,
	oess_1::uint_t page_size )
	:	m_ordinal( ordinal )
	,	m_page_size( page_size )
	,	m_buffer( new oess_1::char_t[ page_size << 1 ] )
	,	m_previous( m_buffer + page_size )
	,	m_has_previous( false )
	{}

loaded_page_t::~loaded_page_t()
	{
		delete[] m_buffer;
	}

void
loaded_page_t::change_previous_by_current()
	{
		std::copy( m_buffer, m_previous, m_previous );
		m_has_previous = true;
	}

//
// ref_loaded_page_t
//

SMART_REF_3_IMPL_REF( loaded_page_t )


} /* namespace storage */

} /* namespace db */

} /* namespace oess */

