/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\since v.1.4.0
	\brief       
		 .
*/
#if !defined( _OESS_1__DB__STORAGE__TRX__TYPES_HPP_ )
#define _OESS_1__DB__STORAGE__TRX__TYPES_HPP_

#include <list>
#include <set>
#include <memory>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/db/h/declspec.hpp>

#include <oess_1/defs/h/types.hpp>

#include <oess_1/stdsn/h/serializable.hpp>
#include <oess_1/stdsn/h/shptr.hpp>

#include <oess_1/db/storage/h/config.hpp>
#include <oess_1/db/storage/h/dirty_page_storage.hpp>

namespace oess_1 {

namespace db {

namespace storage {

namespace trx {

//
// delta_info_t
//

/*!
	\since v.1.4.0
	\brief     .

	     . 
	       .
	    
	   delta_info_t.

	.. delta_info_t    . 
	       ( ).
	       .
*/
class	OESS_1__DB__TYPE	delta_info_t
	:	public oess_1::stdsn::serializable_t
	{
	OESS_SERIALIZER_EX( delta_info_t, OESS_1__DB__TYPE )
	typedef oess_1::stdsn::serializable_t	base_type_t;
	public :
		//!  -.
		delta_info_t();
		//!  .
		delta_info_t(
			//!      .
			oess_1::uint_t index,
			//!   .
			const std::string & value );
		virtual ~delta_info_t();

		//!      .
		oess_1::uint_t
		index() const;

		//!   .
		const std::string &
		value() const;

	private :
		//!      .
		oess_1::uint_t	m_index;

		//!   .
		std::string	m_value;
	};

//
// delta_info_shptr_t
//

/*!
	\since v.1.4.0
	\brief    delta_info_t.
*/
class	OESS_1__DB__TYPE	delta_info_shptr_t
	:	public oess_1::stdsn::shptr_t
	{
	OESS_SERIALIZER_EX( delta_info_shptr_t, OESS_1__DB__TYPE )
	OESS_1_SHPTR_IFACE( delta_info_shptr_t,
			oess_1::db::storage::trx::delta_info_t,
			oess_1::stdsn::shptr_t )
	};

//
// delta_info_list_t
//

/*!
	\since v.1.4.0
	\brief      .
*/
typedef std::list< delta_info_shptr_t > delta_info_list_t;

//
// page_info_t
//

/*!
	\since v.1.4.0
	\brief     .

	     trace-.
*/
class	OESS_1__DB__TYPE	page_info_t
	:	public oess_1::stdsn::serializable_t
	{
	OESS_SERIALIZER_EX( page_info_t, OESS_1__DB__TYPE )

	//!    .
	typedef oess_1::stdsn::serializable_t	base_type_t;

	public :
		//!  -.
		page_info_t();
		//!   .
		page_info_t(
			//!     .
			oess_1::uint_t ordinal );
		virtual ~page_info_t();

		//!      .
		oess_1::uint_t
		ordinal() const;

		//!     .
		void
		add( const delta_info_shptr_t & delta );

		//!   .
		const delta_info_list_t &
		deltas() const;

		//!    delta  .
		oess_1::uint_t
		summary_delta_length() const;

	private :
		//!     .
		oess_1::uint_t	m_ordinal;

		//!     .
		delta_info_list_t	m_deltas;
	};

//
// page_info_list_t
//

/*!
	\since v.1.4.0
	\brief     .
*/
typedef std::list< page_info_t >	page_info_list_t;

//
// trace_stream_item_t
//

/*!
	\since v.1.4.0
	\brief    trace-.
*/
class	OESS_1__DB__TYPE	trace_stream_item_t
	:	public oess_1::stdsn::serializable_t
	{
	OESS_SERIALIZER_EX( trace_stream_item_t, OESS_1__DB__TYPE )
	typedef oess_1::stdsn::serializable_t base_type_t;

	public :
		//!  -.
		trace_stream_item_t();
		//!   .
		/*!
			      
			     make_item().
		*/
		trace_stream_item_t(
			//!      .
			oess_1::uint_t main_file_length );
		virtual ~trace_stream_item_t();

		//!     .
		oess_1::uint_t
		main_file_length() const;

		//!     .
		void
		add(
			const page_info_t & page );

		//!     .
		const page_info_list_t &
		pages() const;

		//!     delta   .
		oess_1::uint_t
		summary_length() const;

	private :
		//!      .
		oess_1::uint_t	m_main_file_length;

		//!    .
		page_info_list_t	m_pages;
	};

//
// make_item.
//

/*!
	\since v.1.4.0
	\brief     
		  trace-stream.

	    \a storage.  
	    trace-stream.
*/
OESS_1__DB__FUNC(void)
make_item(
	//!    .
	dirty_page_storage_t & storage,
	//!  .
	const config_t & config,
	//!    .
	//!     .  
	//!     
	//! .
	trace_stream_item_t & receiver );

//
// image_header_t
//

/*!
	\since v.1.4.0
	\brief     trace-.
*/
class	OESS_1__DB__TYPE	image_header_t
	:	public oess_1::stdsn::serializable_t
	{
		OESS_SERIALIZER_EX( image_header_t, OESS_1__DB__TYPE )
		typedef oess_1::stdsn::serializable_t base_type_t;

	public :
		//!      .
		typedef trace_compression_mode_t	compression_mode_t;

		//!   .
		image_header_t();
		//!   .
		image_header_t(
			//!   .
			const compression_mode_t & compression,
			//!    .
			oess_1::uint_t uncompressed_size,
			//!    .
			oess_1::uint_t compressed_size );
		virtual ~image_header_t();

		//!   .
		const compression_mode_t &
		compression_mode() const;

		//!    .
		oess_1::uint_t
		uncompressed_size() const;

		//!    .
		oess_1::uint_t
		compressed_size() const;

	private :
		//!   .
		compression_mode_t m_compression_mode;

		//!    .
		oess_1::uint_t	m_uncompressed_size;

		//!    .
		oess_1::uint_t	m_compressed_size;
	};

//
// trace_file_number_t
//

/*!
	\since v.1.4.0
	\brief    trace-.
*/
typedef oess_1::uint_t trace_file_number_t;

//
// trace_file_number_set_t
//

/*!
	\since v.1.4.0
	\brief    trace-.
*/
typedef std::set< trace_file_number_t >	trace_file_number_set_t;

//
// snapshot_collection_t
//

/*!
	\since v.1.4.0
	\brief       snapshot- .
*/
class	OESS_1__DB__TYPE	snapshot_collection_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~snapshot_collection_t();

		//!     snapshot-.
		virtual unsigned int
		size() const = 0;

		//!    snapshot-,    .
		virtual trace_file_number_set_t
		non_actual() const = 0;

		//!    snapshot-.
		/*!
			\throw oess_1::ex_t,    snapshot-.
		*/
		virtual trace_file_number_t
		actual() const = 0;

		//!     snapshot-.
		/*!
			\return true,    snapshot.
		*/
		virtual bool
		has_actual() const = 0;

		//!       snapshot-.
		/*!
			\attention    snapshot_collection_t
			     snapshot  
			.  ,     
			   ,    snapshot- (.., 
			 1.4.0,    ,   ).
		*/
		virtual void
		include(
			//!   ,    snapshot.
			const std::string & file_name,
			//!   trace-,   
			//!   snapshot-.
			trace_file_number_t last_used_trace ) = 0;

		//!    snapshot-  
		//!  .
		/*!
			\note     include()  
			  snapshot-   ,    
			    ,    snapshot-.

			\throw oess_1::ex_t,    snapshot-.
		*/
		virtual void
		get_actual(
			//!   ,   .
			const std::string & file_name ) const = 0;

		//!   snapshot-  .
		virtual void
		destroy() = 0;
	};

//
// committer_t
//

/*!
	\since v.1.4.0
	\brief  ,    
		trace-stream     
		     .
*/
class	OESS_1__DB__TYPE	committer_t
	:	private cpp_util_2::nocopy_t
	{
	public :
		virtual ~committer_t();

		/*!
			  trace_stream_t   initialize() 
			,     trace-  
			.     snapshot-,  
			trace-    ..     
			snapshot-.  trace_stream_t     snapshot- 
			 ,     committer-.   
			 trace-    
			committer.   trace_stream_t   
			detect_nonactual_traces()     
			trace-. Committer      
			trace-,   .    
			    trace-,   
			 .     trace-  ,
			      .

			trace_stream_t   \a traces    trace-,
			   trace-,   trace_stream_t
			   . , trace_stream_t  
			 trace-,      
			(    ).   
			detect_nonactual_traces()     .
		*/
		virtual void
		detect_nonactual_traces(
			//!    trace-,  
			//! trace_stream_t    .
			//!      trace-, 
			//!   .
			trace_file_number_set_t & traces ) = 0;

		/*!
			  trace_stream_t  
			   ,    
			    .
		*/
		virtual void
		initialize(
			//!     .
			oess_1::uint_t total_page_count,
			//!     .
			oess_1::uint_t page_size,
			//!     .
			//! ,  page_size   block_size.
			oess_1::uint_t block_size ) = 0;

		/*!
			  trace_stream_t  
			  .
		*/
		virtual void
		finalize() = 0;

		/*!
			  trace_stream_t  
			     
			 .

			     initialize().

			\note  ,    trace-
			   committer-   
			 . ,  0-  committer- 
			   12034. Committer   ,
			         
			12033   .
			   ,    
			      .
		*/
		virtual void
		store_block(
			//!   .
			oess_1::uint_t ordinal,
			//!   ()   .
			oess_1::uint_t block_index,
			//!  .
			const std::string & block_value ) = 0;

		/*!
			  true,  ,   
			 trace- (  reduce_traces_set())
			      .
			,      trace- 
			  snapshot-,   trace- (  
			 snapshot-)    .    
			  ,  
			  (..     
			snapshot-). trace_stream_t    ,  ,
			 false,  trace_stream_t   
			  .    
			 true,     
			  .
		*/
		virtual bool
		is_insufficient_trace_info_allowed() const = 0;
	};

//
// trace_stream_t
//

/*!
	\since v.1.4.0
	\brief     
		.

	 append  reverse_replay    
	    open  create.

	 is_exists  destroy     
	,   trace-stream  .
*/
class	OESS_1__DB__TYPE	trace_stream_t
	:	private	cpp_util_2::nocopy_t
	{
	public :
		virtual ~trace_stream_t();

		/*!
			 trace-stream.

			     ,  
			.
		*/
		virtual void
		open(
			//!   .
			const std::string & physic_name,
			//!  .
			const config_t & config,
			//!    read-only?
			bool is_read_only ) = 0;

		/*!
			 trace-stream.

			 ,       
			.        destroy.
		*/
		virtual void
		create(
			//!    .
			const std::string & physic_name,
			//!   .
			const config_t & config ) = 0;

		/*!
			 trace-stream.

			    .
		*/
		virtual void
		destroy(
			//!   ,   
			//!  .
			const std::string & physic_name ) = 0;

		/*!
			  trace-stream.

			\return true,      trace-.
		*/
		virtual bool
		is_exists(
			//!       
			//! trace-stream.
			const std::string & physic_name ) = 0;

		/*!
			  trace-stream   .

			      
			trace-     trace-,
			    .
		*/
		virtual void
		append(
			const trace_stream_item_t & item ) = 0;

		/*!
			   trace-.

			    
			    \a committer-.

			    ,
			       
			  trace-   
			.

			 \a used_files   trace-,
			     
			 .  \a unused_files  
			trace-,    .
		*/
		virtual void
		reverse_replay(
			committer_t & committer,
			trace_file_number_set_t * used_files = 0,
			trace_file_number_set_t * unused_files = 0,
			//!      
			//!    trace-.
			std::ostream * verbose_stream = 0 ) = 0;

		/*!
			 trace-stream.
		*/
		virtual void
		close() = 0;
	};

//
// create_std_trace_stream
//

/*!
	\since v.1.4.0
	\brief     %trace_stream_t.

	 ,     
	 .

	\return     , 
		      . 
*/
OESS_1__DB__FUNC(std::auto_ptr< trace_stream_t >)
create_std_trace_stream(
	//!     .
	trace_mode_t trace_mode );

//
// create_std_file_committer
//

/*!
	\since v.1.4.0
	\breif     %committer_t.

	\return     , 
		      .

	 ,     
	 .

	\par   oess_1::db::storage::trace_all
	  %committer_t    
	<physic_name>.replay     
	.     (  
	 finalize()),     
	<physic_name>.oess.dat.

	\par   oess_1::db::storage::trace_last
	  %committer_t  ""  .
	  initialize "" ,  , 
	  .   finalize (..  
	trace-  ),  ""  
	  .

	\par   oess_1::db::storage::trace_no
	     ,  
	 -committer-  .
*/
OESS_1__DB__FUNC(std::auto_ptr< committer_t >)
create_std_file_committer(
	//!   ,    .
	const std::string & physic_name,
	//!      .
	trace_mode_t trace_mode );

//
// create_std_snapshot_collection
//

/*!
	\since v.1.4.0
	\brief    %std_snapshot_collection_t.
*/
OESS_1__DB__FUNC(std::auto_ptr< snapshot_collection_t >)
create_std_snapshot_collection(
	//!        snapshot-.
	const std::string & physic_name );

//
// trace_file_extension
//

/*!
	\since v.1.4.0
	\brief   ,  
		 trace-.

	\return  ".trace".
*/
OESS_1__DB__FUNC(std::string)
trace_file_extension();

//
// create_trace_file_name
//

/*!
	\since v.1.4.0
	\brief    trace-   .
*/
OESS_1__DB__FUNC(std::string)
create_trace_file_name(
	const std::string & physic_name,
	trace_file_number_t trace_file_number );

//
// snapshot_file_extension
//

/*!
	\since v.1.4.0
	\brief   ,  
		 snapshot-.

	\return  ".snapshot".
*/
OESS_1__DB__FUNC(std::string)
snapshot_file_extension();

//
// create_snapshot_file_name
//

/*!
	\since v.1.4.0
	\brief    snapshot-   .
*/
OESS_1__DB__FUNC(std::string)
create_snapshot_file_name(
	const std::string & physic_name,
	trace_file_number_t snapshot_file_number );

} /* namespace trx */

} /* namespace storage */

} /* namespace db */

} /* namespace oess */

#endif

