/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

#include <oess_1/defs/log/h/err.hpp>

#include <oess_1/defs/h/err_code.hpp>

namespace oess_1 {

err_code_t::err_code_t(
	const std::string & file,
	int line )
	:
		m_code( 0 ),
		m_file( file ),
		m_line( line )
{
}

err_code_t::err_code_t(
	int_t code,
	const std::string & file,
	int line,
	const std::string & desc )
	:
		m_code( code ),
		m_file( file ),
		m_line( line ),
		m_desc( desc )
{
}

err_code_t::err_code_t(
	const err_code_t & o )
	:
		m_code( o.m_code ),
		m_file( o.m_file ),
		m_line( o.m_line ),
		m_desc( o.m_desc )
{
}

err_code_t::~err_code_t()
{
}

err_code_t &
err_code_t::operator=(
	const err_code_t & o )
{
	if( &o != this )
	{
		m_code = o.m_code;
		m_file = o.m_file;
		m_line = o.m_line;
		m_desc = o.m_desc;
	}
	return *this;
}

OESS_1__DEFS__FUNC( std::ostream & )
err_code_to_std_ostream( std::ostream & o, const err_code_t & c )
{
	if( c ) {
		o << c.m_file << ":" << c.m_line << ": " << c.m_code << " ["
			<< c.m_desc.c_str() << "]";
		o << std::flush;
	}
	else
		o << c.m_code;

	return o;
}

} /* namespace oess_1 */
