/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/defs/h/ent_id.hpp
	\brief  ent_id_t.
*/

#if !defined( _OESS_1__DEFS__ENT_ID_HPP_ )
#define _OESS_1__DEFS__ENT_ID_HPP_

#include <oess_1/defs/h/declspec.hpp>

#include <iostream>

#include <oess_1/defs/h/types.hpp>

namespace oess_1 {

//
// ent_id_t
//

//!	   .
/*!
	    
	NULL-. .. ,  ,
	      
	(  ).  NULL-
	      
	 \c NULL.  , 
	  :
	\code
	oess_1::ent_id_t e = NULL;
	if( e == NULL ) ...
	\endcode

	\attention
	   ,  
	     ObjESSty.
	   ent_id_t    ,
	  ,  .
*/
struct OESS_1__DEFS__TYPE	ent_id_t {
	oess_1::uint_t	m_slice_id;
	oess_1::uint_t	m_ent_id;

	//!    ,
	//!  NULL.
	ent_id_t();
	//!   .
	ent_id_t(
		oess_1::uint_t slice_id,
		oess_1::uint_t ent_id );
	//!  .
	ent_id_t(
		const ent_id_t & o );
	~ent_id_t();

	//!  .
	ent_id_t &
	operator=(
		const ent_id_t & o );

	//!  true,   NULL.
	operator bool() const;

	//!  true,  NULL.
	bool
	operator!() const;

	//!   .
	bool
	operator==( const ent_id_t & o ) const;

	//!   .
	bool
	operator!=( const ent_id_t & o ) const;

	//!   " ".
	/*!
		  ,   ent_id_t
		    
		 STL.
	*/
	bool
	operator<( const ent_id_t & o ) const;

	//!  ,  NULL.
	void
	make_null();
};

//  /  ent_id_t  std::stream.
//       ,
//    ,  NULL.

OESS_1__DEFS__FUNC( std::ostream & )
ent_id_to_std_ostream( std::ostream & s, const ent_id_t & o );

OESS_1__DEFS__FUNC( std::istream & )
ent_id_from_std_ostream( std::istream & s, ent_id_t & o );

//!   ent_id_t   .
inline std::ostream &
operator<<( std::ostream & s, const ent_id_t & o )
{
	return ent_id_to_std_ostream( s, o );
}

//!   ent_id_t   .
/*!
	      ,
	   ,  NULL.
*/
inline std::istream &
operator>>( std::istream & s, ent_id_t & o )
{
	return ent_id_from_std_ostream( s, o );
}

} /* namespace oess_1 */

#endif

