/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/defs/h/err_code.hpp
	\brief  err_code_t.
*/

#if !defined( _OESS_1__DEFS__ERR_CODE_HPP_ )
#define _OESS_1__DEFS__ERR_CODE_HPP_

#include <oess_1/defs/h/declspec.hpp>

#include <iostream>

#include <string>

#include <oess_1/defs/h/types.hpp>

namespace oess_1 {

//!    .
/*!
	      
	  ,   
	 , ,  .
*/
struct OESS_1__DEFS__TYPE	err_code_t {
	//!   .
	int_t	m_code;

	//!  ,      err_code_t.
	std::string	m_file;

	//!   ,     
	//! err_code_t.
	/*!
		,     
		    
		\c __LINE__.
	*/
	int	m_line;

	//!   .  .
	/*!
		,     
		( 0),    .

		\note  
		,    .  
		  Windows    
		(,   CP866,  
		 -- CP1251).
	*/
	std::string	m_desc;

	/*!
		    .
		 m_code  0, 
		  m_file  m_line.
	*/
	err_code_t(
		//!  ,      err_code_t.
		const std::string & file,
		//!   ,     
		//! err_code_t.
		int line );

	/*!
		    .
	*/
	err_code_t(
		//!   .
		int_t code,
		//!  ,      err_code_t.
		const std::string & file,
		//!   ,     
		//! err_code_t.
		int line,
		//!   .
		const std::string & desc );

	//!  .
	err_code_t(
		const err_code_t & o );

	~err_code_t();

	//!  .
	err_code_t &
	operator=(
		const err_code_t & o );

	//!    ?
	/*!
		\return true,  m_code    ( 
		  ). false,  m_code  0 ( ).
	*/
	inline
	operator bool() const {
		return ( 0 != m_code );
	}

	//!   .
	/*!
		  :
		\code
		if( err == 3 ) ...
		\endcode
	*/
	inline bool
	operator==( int c ) const {
		return ( c == m_code );
	}

	//! .
	/*!
		  m_code.
	*/
	inline bool
	operator==( const err_code_t & o ) const {
		return ( m_code == o.m_code );
	}

	//!   .
	/*!
		  :
		\code
		if( err != 3 ) ...
		\endcode
	*/
	inline bool
	operator!=( int c ) const {
		return ( c != m_code );
	}

	//! .
	/*!
		  m_code.
	*/
	inline bool
	operator!=( const err_code_t & o ) const {
		return ( m_code != o.m_code );
	}

	//!   ?
	/*!
		\return true,  m_code  0 ( ).
		false,  m_code    ( 
		  ).
	*/
	inline bool
	operator!() const {
		return ( !m_code );
	}
};

//!    ,   err_code_t.
/*!
	\param code	  .
	\param desc	,    const std::string &
		 const char *.

	:
	\code
	oess_1::err_code_t
	some_func()
	{
		if( ... )
			return oess_1_make_err( -1, "invalid argument" );

		if( ... )
			return oess_1_make_err( -2,
				std::string( "log" ) + ": file not found" );
	}
	\endcode
*/
#define oess_1_make_err(code, desc) oess_1::err_code_t(code, __FILE__, __LINE__, desc)

//!   ObjESSty.
/*!
	\if oess_full_api
	  err_code_t    .

	    '<<' 
	 err_code_t.
	\endif
*/
OESS_1__DEFS__FUNC( std::ostream & )
err_code_to_std_ostream( std::ostream & o, const err_code_t & c );

//!   err_code_t    .
/*!
	   :

	< >:< >:< > [<>]
*/
inline std::ostream &
operator<<( std::ostream & o, const err_code_t & c )
{
	return err_code_to_std_ostream( o, c );
}

} /* namespace oess_1 */

#endif
