/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

#if !defined( _OESS_1__FILE__FILE_HPP_ )
#define _OESS_1__FILE__FILE_HPP_

#include <oess_1/file/h/declspec.hpp>

#include <auto_ptr_3/h/vect_ptr.hpp>

#include <oess_1/io/h/binstream.hpp>

namespace oess_1 {

namespace file {

//
// file_t
//

/*!
	    .
*/
class	OESS_1__FILE__TYPE	file_t :
	public oess_1::io::ibinstream_t,
	public oess_1::io::obinstream_t
{
	public :
		file_t();
		virtual ~file_t();

		/*!   . */
		void
		open(
			/*!   . */
			const std::string & file_name,
			/*! true,     
				  . */
			bool is_read_only );

		/*!     
			. */
		void
		create(
			/*!   . */
			const std::string & file_name );

		/*!  .
			  . */
		void
		close();

		/*!     . */
		oess_1::uint_t
		tell();

		/*!    . */
		void
		seek( oess_1::uint_t pos );

		/*!   . */
		oess_1::uint_t
		length();

		/*!    ? */
		virtual bool
		eof() const;

		/*!     . */
		void
		chsize(
			oess_1::uint_t size );

		/*!  . */
		static void
		remove( const std::string & file_name );

		/*!
			\since v.1.1.0
			\brief      .

			\return true,   . false,   
				,      
				   (, ). */
		static bool
		is_file_exists(
			const std::string & file_name );

		/*!
			\since v.1.4.0
			\brief  .

			  \a auto_destroy_existing_target == true,
			   \a to   \a to ,
			    \a to.

			  \a auto_destroy_existing_target == false,
			  \a to    
			  ,   .
		*/
		static void
		rename_file(
			//!  .
			const std::string & what,
			//!   .
			const std::string & to,
			//!    \a to,  \a to .
			bool auto_destroy_existing_target = false );

	protected :
		/*
			!
			   file_t 
			 ,    /
			   in_reserve/in_shift
			(out_reserve/out_shift).
			  in_reserve  
			   .  out_shift
			    .
			..  /
			     
			 file_t    *_reserve/ *_shift.
			     .
		*/

		/*!    ,  
			  .
			      
			   ,   
			.

			   ( )
			    . */
		virtual const oess_1::char_t *
		in_reserve(
			/*!  ,  
				. */
			size_t item_count,
			/*!     . */
			size_t item_size,
			/*!  ,  
				 . */
			size_t & item_available );

		/*!  .   
			  ,    
			  out_reserve.

			  .
		*/
		virtual void
		in_shift(
			/*!    . */
			size_t item_count,
			/*!     . */
			size_t item_size );

		/*!    ,  
			  .
			      
			   ,   
			.

			    
			. */
		virtual oess_1::char_t *
		out_reserve(
			/*!  ,  
				. */
			size_t item_count,
			/*!     . */
			size_t item_size,
			/*!  ,    
				 . */
			size_t & item_available );

		/*!  .   
			  ,    
			  out_reserve.

			   ( )
			    . */
		virtual void
		out_shift(
			/*!    . */
			size_t item_count,
			/*!     . */
			size_t item_size );

	private :
		/*!    -. */
		auto_ptr_3::vect_ptr_t< oess_1::char_t >	m_buf;

		/*!   . */
		int	m_file;
		/*! true,      read-only. */
		bool	m_is_read_only;

		/*!    .
			   , 
			 . */
		void
		is_workable(
			/*! true,    read-only
				     
				. */
			bool is_read_only_op ) const;
};

} /*! namespace file */

} /*! namespace oess_1 */

#endif
