/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/io/h/bin_data_size.hpp
	\brief  oess_1::io::bin_data_size_t.
*/

#if !defined( _OESS_1__IO__BIN_DATA_SIZE_HPP_ )
#define _OESS_1__IO__BIN_DATA_SIZE_HPP_

#include <oess_1/defs/h/types.hpp>

namespace oess_1 {

namespace io {

/*!
	\brief     
	   .

	 :
	\code
	int char_size = oess_1::io::bin_data_size_t< oess_1::char_t >::image_size;
	int int_size = oess_1::io::bin_data_size_t< oess_1::int_t >::image_size;
	int double_size = oess_1::io::bin_data_size_t< oess_1::double_t >::image_size;
	\endcode

	\attention
	     oess_1::char_t,
	oess_1::schar_t, oess_1::uchar_t, oess_1::short_t, oess_1::ushort_t,
	oess_1::int_t, oess_1::uint_t, oess_1::single_t, oess_1::double_t.
*/
template< class Type >
struct	bin_data_size_t {
};

//!   char.
template<>
struct	bin_data_size_t< oess_1::char_t >
	{
		enum
			{
				image_size = 1
			};
	};

//!   signed char.
template<>
struct	bin_data_size_t< oess_1::schar_t >
	{
		enum
			{
				image_size = 1
			};
	};

//!   unsigned char.
template<>
struct	bin_data_size_t< oess_1::uchar_t >
	{
		enum
			{
				image_size = 1
			};
	};

//!   short.
template<>
struct	bin_data_size_t< oess_1::short_t >
	{
		enum
			{
				image_size = 2
			};
	};

//!   unsigned short.
template<>
struct	bin_data_size_t< oess_1::ushort_t >
	{
		enum
			{
				image_size = 2
			};
	};

//!   int.
template<>
struct	bin_data_size_t< oess_1::int_t >
	{
		enum
			{
				image_size = 4
			};
	};

//!   unsigned int.
template<>
struct	bin_data_size_t< oess_1::uint_t >
	{
		enum
			{
				image_size = 4
			};
	};

//!   flost.
template<>
struct	bin_data_size_t< oess_1::single_t >
	{
		enum
			{
				image_size = 4
			};
	};

//!   double.
template<>
struct	bin_data_size_t< oess_1::double_t >
	{
		enum
			{
				image_size = 8
			};
	};

} /* namespace io */

} /* namespace oess_1 */

#endif
