/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\since v.1.1.1
	\file
	\brief    -.
*/

#if !defined( _OESS_1__IO__BINBUFFER_HPP_ )
#define _OESS_1__IO__BINBUFFER_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <oess_1/io/h/binstream.hpp>

namespace oess_1 {

namespace io {

//
// ibinbuffer_t
//

/*!
	\since v.1.1.1
	\brief    -.

	      
	  .   
	 :
		-      (size);
		-    .

	 %ibinbuffer_t   , 
	     -.
*/
class	OESS_1__IO__TYPE	ibinbuffer_t
	: public ibinstream_t
{
	public :
		virtual ~ibinbuffer_t();

		//!     .
		virtual size_t
		size() const = 0;

		//!    .
		virtual size_t
		pos() const = 0;

		virtual bool
		eof() const = 0;

	protected :
		virtual const oess_1::char_t *
		in_reserve(
			size_t item_count,
			size_t item_size,
			size_t & item_available ) = 0;

		virtual void
		in_shift(
			size_t item_count,
			size_t item_size ) = 0;
};

//
// obinbuffer_t
//

/*!
	\since v.1.1.1
	\brief    -.

	      
	   ( 
	 ).   
	 :
		-    (capacity).  
	    
	,     .  
	  ,   
	      
	;
		-       (size);
		-    .

	 %obinbuffer_t   , 
	     -.
*/
class	OESS_1__IO__TYPE	obinbuffer_t
	:	public obinstream_t
{
	public :
		virtual ~obinbuffer_t();

		//!   .
		virtual size_t
		capacity() const = 0;

		//!     .
		virtual size_t
		size() const = 0;

		//!    .
		virtual size_t
		pos() const = 0;

	protected :
		virtual oess_1::char_t *
		out_reserve(
			size_t item_count,
			size_t item_size,
			size_t & item_available ) = 0;

		virtual void
		out_shift(
			size_t item_count,
			size_t item_size ) = 0;
};

} /* namespace io */

} /* namespace oess_1 */

#endif
