/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief  %oess_1::io::ibinstream_t,
	%oess_1::io::obinstream_t.
*/

#if !defined( _OESS_1__IO__BIN_STREAM_HPP_ )
#define _OESS_1__IO__BIN_STREAM_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <oess_1/io/h/stream.hpp>

namespace oess_1 {

namespace io {

//
// ibinstream_t
//

/*!
	\brief    .

	  
	  BigEndian   .
*/
class	OESS_1__IO__TYPE	ibinstream_t :
	public istream_t {
	public :
		virtual ~ibinstream_t();

		virtual void
		read( oess_1::char_t * p, size_t count );

		virtual void
		read( oess_1::schar_t * p, size_t count );

		virtual void
		read( oess_1::uchar_t * p, size_t count );

		virtual void
		read( oess_1::short_t * p, size_t count );

		virtual void
		read( oess_1::ushort_t * p, size_t count );

		virtual void
		read( oess_1::int_t * p, size_t count );

		virtual void
		read( oess_1::uint_t * p, size_t count );

		virtual void
		read( oess_1::single_t * p, size_t count );

		virtual void
		read( oess_1::double_t * p, size_t count );

		virtual void
		read( std::string & p );

		virtual bool
		eof() const = 0;

	protected :
		friend	class ibuffer_accessor_t;
		friend	class	isubbinstream_t;

		//!    ,  
		//!   .
		/*!
			      
			   ,   
			.
		*/
		virtual const oess_1::char_t *
		in_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,  
			//!  .
			size_t & item_available ) = 0;

		//!  .
		/*!
			  
			  ,    
			  out_reserve.
		*/
		virtual void
		in_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size ) = 0;
};

//
// obinstream_t
//

/*!
	\brief    .

	  
	     BigEndian.
*/
class	OESS_1__IO__TYPE	obinstream_t :
	public ostream_t {
	public :
		virtual ~obinstream_t();

		virtual void
		write( const oess_1::char_t * p, size_t count );

		virtual void
		write( const oess_1::schar_t * p, size_t count );

		virtual void
		write( const oess_1::uchar_t * p, size_t count );

		virtual void
		write( const oess_1::short_t * p, size_t count );

		virtual void
		write( const oess_1::ushort_t * p, size_t count );

		virtual void
		write( const oess_1::int_t * p, size_t count );

		virtual void
		write( const oess_1::uint_t * p, size_t count );

		virtual void
		write( const oess_1::single_t * p, size_t count );

		virtual void
		write( const oess_1::double_t * p, size_t count );

		virtual void
		write( const std::string & p );

	protected :
		friend	class obuffer_accessor_t;

		//!    ,  
		//!   .
		/*!
			      
			   ,   
			.
		*/
		virtual oess_1::char_t *
		out_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,    
			//!  .
			size_t & item_available ) = 0;

		//!  .
		/*!
			  
			  ,    
			  out_reserve.
		*/
		virtual void
		out_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size ) = 0;
};

} /* namespace io */

} /* namespace oess_1 */

#endif

