/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/io/h/bstring_buf.hpp
	\brief  %oess_1::io::ibstring_t, %oess_1::io::obstring_t.
*/

#if !defined( _OESS_1__IO__BSTRING_BUF_HPP_ )
#define _OESS_1__IO__BSTRING_BUF_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <auto_ptr_3/h/vect_ptr.hpp>

#include <oess_1/io/h/binbuffer.hpp>

namespace oess_1 {

namespace io {

//
// ibstring_t
//

//!      std::string.
/*!
	,   
	  std::string,  
	.

	    
	 std::string,     
	.

	 ibstring_t   std::string
	  .

	      .

	\attention    1.4.0  , 
	  ibstring_t  
	:        
	ibstring_t.

	\note v.1.1.1
	   oess_1::io::ibinbuffer_t.
*/
class	OESS_1__IO__TYPE	ibstring_t :
	public ibinbuffer_t
{
	public :
		ibstring_t(
			//!   ,   
			//!   .
			/*!
				    
				,    
				 ibstring_t.
			*/
			const std::string & from );
		virtual ~ibstring_t();

		//!   .
		/*!
			\deprecated   
				oess_1::io::ibstring_t::size(). */
		size_t
		capacity() const;

		//!   .
		/*!
			\since v.1.1.1 */
		virtual size_t
		size() const;

		//!  ,    
		//!   .
		virtual size_t
		pos() const;

		//!  ,    
		//!   .
		/*!
			  ,  pos ,
			 size().
		*/
		void
		set_pos( size_t pos );

		//!    ?
		/*!
			\return true,    
			 .
		*/
		virtual bool
		eof() const;

	protected :
		//!     .
		/*!
			      
			   ,  
			.
		*/
		virtual const oess_1::char_t *
		in_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,  
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		in_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//!     .
		const std::string &	m_from;
		//!  .
		size_t	m_pos;

	private :
		//!   .
		ibstring_t(
			const ibstring_t & );
		//!   .
		ibstring_t &
		operator=(
			const ibstring_t & );
};

//
// obstring_t
//

//!      std::string.
/*!
	,   
	  std::string.

	    
	 std::string,     
	.

	 obstring_t   std::string
	  .

	     . 
	   - ,  
	   .

	     .
	    , .. 
	    .

	\note v.1.1.1
	   oess_1::io::obinbuffer_t.
*/
class	OESS_1__IO__TYPE	obstring_t :
	public obinbuffer_t
{
	public :
		obstring_t(
			//!   ,   
			//!   .
			/*!
				    
				,    
				 bstring_t.
			*/
			std::string & to );
		virtual ~obstring_t();

		//!   .
		/*!
			\deprecated   
				oess_1::io::obstring_t::size(). */
		size_t
		len() const;

		//!   .
		/*!
			\size v.1.1.1
			\return   capacity  . */
		virtual size_t
		capacity() const;

		//!     .
		/*!
			\size v.1.1.1
			\return   size  . */
		virtual size_t
		size() const;

		//!    .
		/*!
			\size v.1.1.1
			\return   size  ,
				..     
				 . */
		virtual size_t
		pos() const;

	protected :
		//!     .
		virtual oess_1::char_t *
		out_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,    
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		out_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//! ,    .
		std::string &	m_to;

		//!      
		//! out_reserve  out_shift.
		auto_ptr_3::vect_ptr_t< oess_1::char_t >	m_buf;

	private :
		//!   .
		obstring_t(
			const obstring_t & );
		//!   .
		obstring_t &
		operator=(
			const obstring_t & );
};

} /* namespace io */

} /* namespace oess_1 */

#endif
