/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief  %oess_1::io::ifixed_mem_buf_t,
	%oess_1::io::ofixed_mem_buf_t.
*/

#if !defined( _OESS_1__IO__FIXED_MEM_BUF_HPP_ )
#define _OESS_1__IO__FIXED_MEM_BUF_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <auto_ptr_3/h/vect_ptr.hpp>

#include <oess_1/io/h/binbuffer.hpp>

namespace oess_1 {

namespace io {

//
// ifixed_mem_buf_t
//

//!       .
/*!
	  obinstream_t 
	   .

	    
	,     
	.

	 ifixed_mem_buf_t   
	  .

	,     
	.    
	  .

	\note v.1.1.1
	   oess_1::io::ibinbuffer_t.
*/
class	OESS_1__IO__TYPE	ifixed_mem_buf_t :
	public ibinbuffer_t
{
	public :
		ifixed_mem_buf_t(
			//! ,   
			//!  .
			/*!
				    ,
				   ifixed_mem_buf_t.
			*/
			const oess_1::char_t * buf,
			//!    .
			size_t capacity );
		virtual ~ifixed_mem_buf_t();

		//!  .
		/*!
			\return  ,   .
			\deprecated   
				oess_1::io::ifixed_mem_buf_t::size().
		*/
		size_t
		capacity() const;

		//!    .
		/*!
			\since v.1.1.1
			\return  ,   .
		*/
		virtual size_t
		size() const;

		//!  ,    
		//!   .
		virtual size_t
		pos() const;

		//!  ,    
		//!   .
		/*!
			  ,  pos ,
			 capacity().
		*/
		void
		set_pos( size_t pos );

		//!  true,  pos() == capacity().
		virtual bool
		eof() const;

		//!     .
		const oess_1::char_t *
		ptr() const;

		//!      .
		const oess_1::char_t *
		current_ptr() const;

	protected :
		//!     .
		/*!
			      
			   ,  
			.
		*/
		virtual const oess_1::char_t *
		in_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,  
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		in_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//! ,     .
		const oess_1::char_t *	m_buf;
		//!  .
		/*!
			  .
		*/
		size_t	m_capacity;
		//!    .
		size_t	m_pos;

	private :
		//!   .
		ifixed_mem_buf_t(
			const ifixed_mem_buf_t & );
		//!   .
		ifixed_mem_buf_t &
		operator=(
			const ifixed_mem_buf_t & );
};

//
// ofixed_mem_buf_t
//

//!       .
/*!
	  obinstream_t 
	   .

	    
	,     
	.

	 ofixed_mem_buf_t   
	  .

	     ,
	  size().

	\par v.1.1.1
	   oess_1::io::obinbuffer_t.
	
	\par v.1.1.1
	 ofixed_mem_buf_t ,   
	  (.. size() == capacity()). 
	       
	 . ,     , 
	    .  
	   
	 \a initial_size.     0,
	 ,      initial_size
	.
*/
class	OESS_1__IO__TYPE	ofixed_mem_buf_t :
	public obinbuffer_t
{
	public :
		ofixed_mem_buf_t(
			//! ,   
			//!  .
			/*!
				    ,
				   fixed_mem_buf_t.
			*/
			oess_1::char_t * buf,
			//!    .
			size_t capacity,
			//!       .
			size_t initial_size = 0 );
		virtual ~ofixed_mem_buf_t();

		//!  .
		/*!
			\return  ,   .
		*/
		virtual size_t
		capacity() const;

		//!      .
		/*!
			\since v.1.1.1 */
		virtual size_t
		size() const;

		//!  ,    
		//!   .
		virtual size_t
		pos() const;

		//!  ,    
		//!   .
		/*!
			  ,  pos ,
			 size().
		*/
		void
		set_pos( size_t pos );

		//!     .
		const oess_1::char_t *
		ptr() const;

		//!      .
		const oess_1::char_t *
		current_ptr() const;

	protected :
		//!     .
		virtual oess_1::char_t *
		out_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,    
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		out_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//! ,     .
		oess_1::char_t *	m_buf;
		//!  .
		/*!
			  .
		*/
		size_t	m_capacity;
		//!   ,   .
		size_t	m_size;
		//!    .
		size_t	m_pos;

	private :
		//!   .
		ofixed_mem_buf_t(
			const ofixed_mem_buf_t & );
		//!   .
		ofixed_mem_buf_t &
		operator=(
			const ofixed_mem_buf_t & );
};

} /* namespace io */

} /* namespace oess_1 */

#endif
