/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/io/h/mem_buf.hpp
	\brief  %oess_1::io::mem_buf_t.
*/

#if !defined( _OESS_1__IO__MEM_BUF_HPP_ )
#define _OESS_1__IO__MEM_BUF_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <auto_ptr_3/h/vect_ptr.hpp>

#include <oess_1/io/h/binbuffer.hpp>

namespace oess_1 {

namespace io {

//
// mem_buf_t
//

//!  /   
//!  .
/*!
	  ibinstream_t, obinstream_t
	  ,   
	  .

	\note
	     
	. :
\code
oess_1::io::mem_buf_t buf1( 1024 );
oess_1::io::mem_buf_t buf2( 2048 );

buf1.set_size( 256 );
buf1.set_pos( 26 );

buf2.set_size( 512 );
buf2.set_pos( 384 );

buf2 = buf1;
//  26 == buf2.pos()
\endcode

	\par v.1.1.1
	   oess_1::io::ibinbuffer_t,
	oess_1::io::obinbuffer_t.
*/
class	OESS_1__IO__TYPE	mem_buf_t :
	public ibinbuffer_t,
	public obinbuffer_t
{
	public :
		/*!
			   
			 e_initial_capacity.
		*/
		mem_buf_t();
		mem_buf_t(
			//!   .
			size_t capacity );
		//!  .
		mem_buf_t(
			const mem_buf_t & o );
		virtual ~mem_buf_t();

		mem_buf_t &
		operator=(
			const mem_buf_t & o );

		enum {
			e_initial_capacity = 64
		};

		//!   .
		virtual size_t
		capacity() const;

		//!  .
		/*!
			  .

			 save_contents  'true', 
			   ,
			  'size' = min( new_capacity, size )
			  .
		*/
		void
		change_capacity(
			//!   .
			size_t new_capacity,
			//!    
			//!  .
			bool save_contents = true );

		//!   .
		/*!
			 ,    .
		*/
		virtual size_t
		size() const;

		//!     .
		/*!
			  .

			    
			new_size <= capacity().
		*/
		void
		change_size(
			//!   .
			size_t new_size );

		//!  ,    
		//!     .
		virtual size_t
		pos() const;

		//!  ,    
		//!     .
		/*!
			  ,  pos <= size().
		*/
		void
		set_pos( size_t pos );

		//!  true,  pos() == size().
		virtual bool
		eof() const;

		//!     .
		const oess_1::char_t *
		ptr() const;

		//!      .
		const oess_1::char_t *
		current_ptr() const;

		//!  .
		/*!
			  to   to_pos  count 
			  from_pos.

			 -   to_pos 
			 ( count  
			-).

			  -  .

			   .
		*/
		void
		subbuf(
			/*!    . */
			size_t from_pos,
			/*! -. */
			mem_buf_t & to,
			/*!        . */
			size_t to_pos,
			/*!   . */
			size_t count ) const;

		//!  .
		/*!
			  to  count 
			  from_pos.

			 to      
			,   count .

			  -  .

			   .
		*/
		void
		subbuf(
			/*!    . */
			size_t from_pos,
			/*!       . */
			void * to,
			/*!   . */
			size_t count ) const;

		//!  .
		/*!
			 count   from c  from_pos  
			to_pos.    "".

			  .

			   -  .

			     from 
			.

			  to_pos > size()  .
		*/
		void
		insert(
			/*! ,    . */
			size_t to_pos,
			/*! - . */
			const mem_buf_t & from,
			/*!   -,  
				 . */
			size_t from_pos,
			/*!   . */
			size_t count );

		//!  .
		/*!
			 count   from  
			to_pos.    "".

			  .

			,  from     
			  count .

			  to_pos > size()  .
		*/
		void
		insert(
			size_t to_pos,
			const void * from,
			size_t count );

		//!    .
		/*!
			 count     from_offs.
			   "".

			  from_offs > size()  
			(from_offs + count > size())  .
		*/
		void
		erase(
			size_t from_offs,
			size_t count );

		//!    .
		/*!
			    .
			     .
		*/
		void
		swap(
			mem_buf_t & o );

	protected :
		//!     .
		/*!
			      
			   ,  
			.
		*/
		virtual const oess_1::char_t *
		in_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,  
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		in_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

		//!     .
		virtual oess_1::char_t *
		out_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,    
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		out_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//! .
		auto_ptr_3::vect_ptr_t< char >	m_buf;
		//!   .
		size_t	m_size;
		//!    .
		size_t	m_pos;
};

} /* namespace io */

} /* namespace oess_1 */

#endif
