/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/io/h/stream.hpp
	\brief  %oess_1::io::istream_t, %oess_1::io::ostream_t.
*/

#if !defined( _OESS_1__IO__STREAM_HPP_ )
#define _OESS_1__IO__STREAM_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <string>

#include <oess_1/defs/h/types.hpp>
#include <oess_1/defs/h/ent_id.hpp>

namespace oess_1 {

namespace io {

//
// istream_t
//

/*!
	\brief    .

	     
	  .
*/
class	OESS_1__IO__TYPE	istream_t {
	public :
		virtual ~istream_t();

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::char_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::schar_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::uchar_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::short_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::ushort_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::int_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::uint_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::single_t * p, size_t count ) = 0;

		/*!
			 \a count   
			      \a p.
		*/
		virtual void
		read( oess_1::double_t * p, size_t count ) = 0;

		/*!
			     
			   \a p.    
			  ,    
			    .

			     
			  .
		*/
		virtual void
		read( std::string & p ) = 0;

		//!    ?
		/*!
			\return true,    .
		*/
		virtual bool
		eof() const = 0;
};

//
// ostream_t
//

/*!
	\brief    .

	     
	  .
*/
class	OESS_1__IO__TYPE	ostream_t {
	public :
		virtual ~ostream_t();

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::char_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::schar_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::uchar_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::short_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::ushort_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::int_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::uint_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::single_t * p, size_t count ) = 0;

		/*!
			 \a count ,  
			 \a p   .
		*/
		virtual void
		write( const oess_1::double_t * p, size_t count ) = 0;

		/*!
			    \a p.    
			 ,  
			      .

			     
			  .
		*/
		virtual void
		write( const std::string & p ) = 0;
};


//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::char_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::char_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::schar_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::schar_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::uchar_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::uchar_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::short_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::short_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::ushort_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::ushort_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::int_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::int_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::uint_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::uint_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::single_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::single_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::double_t & o ) {
	s.read( &o, 1 );
	return s;
}

//!    \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::double_t & o ) {
	s.write( &o, 1 );
	return s;
}

//!   \a o    \a s.
inline istream_t &
operator>>( istream_t & s, std::string & o ) {
	s.read( o );
	return s;
}

//!   \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const std::string & o ) {
	s.write( o );
	return s;
}

//!   \a o    \a s.
inline istream_t &
operator>>( istream_t & s, oess_1::ent_id_t & o ) {
	s.read( &o.m_slice_id, 1 );
	s.read( &o.m_ent_id, 1 );
	return s;
}

//!   \a o    \a s.
inline ostream_t &
operator<<( ostream_t & s, const oess_1::ent_id_t & o ) {
	s.write( &o.m_slice_id, 1 );
	s.write( &o.m_ent_id, 1 );
	return s;
}

} /* namespace io */

} /* namespace oess_1 */

#endif
