/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/io/h/subbinstream.hpp
	\brief  %oess_1::io::isubbinstream_t.
*/

#if !defined( _OESS_1__IO__SUBBINSTREAM_HPP_ )
#define _OESS_1__IO__SUBBINSTREAM_HPP_

#include <oess_1/io/h/declspec.hpp>

#include <oess_1/io/h/binbuffer.hpp>

namespace oess_1 {

namespace io {

//
// isubbinstream_t
//

//!   .
/*!
	  obinstream_t 
	      
	 .

	      TLV
	(oess_1::tlv::compound_tlv_t)   
	      Length .
	      
	 ( TLV)    
	       .
	       
	  Length    ,   
	    .    
	 .

	 isubbinstream_t    
	.       
	.    ,   
	. ,    
	,       Length .

	    isubbinstream_t   
	   Length ,   isubbinstream_t
	       .

	\note v.1.1.1
	   oess_1::io::ibinbuffer_t.
*/
class	OESS_1__IO__TYPE	isubbinstream_t :
	public ibinbuffer_t
{
	public :
		isubbinstream_t(
			//!   .
			ibinstream_t & source,
			//!   .
			size_t capacity );
		virtual ~isubbinstream_t();

		//!  .
		/*!
			\return  ,   .
		*/
		size_t
		capacity() const;

		//!     .
		/*!
			\return  ,   .
		*/
		virtual size_t
		size() const;

		//!     .
		virtual size_t
		pos() const;

		//!  true,  pos() == capacity().
		virtual bool
		eof() const;

	protected :
		//!     .
		/*!
			      
			   ,  
			.
		*/
		virtual const oess_1::char_t *
		in_reserve(
			//!  ,  
			//! .
			size_t item_count,
			//!     .
			size_t item_size,
			//!  ,  
			//!  .
			size_t & item_available );

		//!     .
		virtual void
		in_shift(
			//!    .
			size_t item_count,
			//!     .
			size_t item_size );

	private :
		//!  .
		ibinstream_t &	m_source;
		//!   .
		/*!
			  .
		*/
		size_t	m_capacity;
		//!     .
		size_t	m_pos;

	private :
		//!   .
		isubbinstream_t(
			const isubbinstream_t & );
		//!   .
		isubbinstream_t &
		operator=(
			const isubbinstream_t & );
};

} /* namespace io */

} /* namespace oess_1 */

#endif

