/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

#include <oess_1/defs/h/ex.hpp>

#include <oess_1/io/h/errno.hpp>
#include <oess_1/io/h/subbinstream.hpp>

namespace oess_1 {

namespace io {

//
// isubbinstream_t
//

isubbinstream_t::isubbinstream_t(
	ibinstream_t & source,
	size_t capacity )
	:
		m_source( source ),
		m_capacity( capacity ),
		m_pos( 0 )
{
}

isubbinstream_t::~isubbinstream_t()
{
}

size_t
isubbinstream_t::capacity() const
{
	return m_capacity;
}

size_t
isubbinstream_t::size() const
{
	return m_capacity;
}

size_t
isubbinstream_t::pos() const
{
	return m_pos;
}

bool
isubbinstream_t::eof() const
{
	return ( pos() == capacity() );
}

const oess_1::char_t *
isubbinstream_t::in_reserve(
	size_t item_count,
	size_t item_size,
	size_t & item_available )
{
	// ,   
	//  .
	size_t items = ( m_capacity - m_pos ) / item_size;
	if( !items )
		OESS_THROW_LOGIC( oess_1::io::err::c_eof,
			"substream capacity: " << m_capacity
			<< ", pos: " << m_pos
			<< ", item size: " << item_size )

	if( items > item_count )
		items = item_count;

	//     
	//  .
	return m_source.in_reserve( items, item_size, item_available );
}

void
isubbinstream_t::in_shift(
	size_t item_count,
	size_t item_size )
{
	m_pos += item_count * item_size;
	// m_pos     m_capacity!
	if( m_pos > m_capacity )
		m_pos = m_capacity;

	//      
	//  .
	m_source.in_shift( item_count, item_size );
}

} /* namespace io */

} /* namespace oess_1 */

