/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief     , 
		      .
*/

#include <set>

#include <cpp_util_2/h/nocopy.hpp>

#include <oess_1/defs/log/h/out.hpp>

#include <oess_1/stdsn/h/serializable.hpp>
#include <oess_1/stdsn/h/ent_std.hpp>

namespace oess_1 {

namespace stdsn {

//
// operation_history_t
//

/*!
	\brief    .
	\since v.1.2.0

	   
	           
	   .
*/
class	operation_history_t
	:	private cpp_util_2::nocopy_t
	{
	private :
		/*!
			\brief     .
			\since v.1.2.0

			  , ..   
			       
			   . ,   ,  
			   ,  
			 .
		*/
		struct	obj_info_t
			{
				//!    .
				const void *	m_object;
				//!     .
				const std::string *	m_type_name;

				//!  -.
				obj_info_t()
					:	m_object( 0 )
					,	m_type_name( 0 )
					{}
				//!  .
				obj_info_t(
					const void * object,
					const std::string * type_name )
					:	m_object( object )
					,	m_type_name( type_name )
					{}

				//!  .
				/*!
					    ,  
					  .
				*/
				bool
				operator<( const obj_info_t & o ) const
					{
						return ( m_object < o.m_object ||
							( m_object == o.m_object &&
							m_type_name < o.m_type_name ) );
					}
			};

		//!     .
		typedef std::set< obj_info_t >	set_t;

		//!   .
		set_t	m_objects;

	public :
		//!    .
		bool
		insert(
			const void * object,
			const std::string * type_name )
			{
				//  set::insert  pair<iterator, bool>,   second
				//   true,      .
				// ..    .
				return m_objects.insert(
						obj_info_t( object, type_name )
					).second;
			}

		//!    .
		void
		erase(
			const void * object,
			const std::string * type_name )
			{
				m_objects.erase( obj_info_t( object, type_name ) );
			}
	};

//
// ient_std_t
//

ient_std_t::ient_std_t(
	oess_1::io::istream_t & stream )
	:	m_stream( stream )
	,	m_history( new operation_history_t() )
	{}

ient_std_t::~ient_std_t()
	{}

bool
ient_std_t::obj_start(
	const void * object,
	const std::string * type_name,
	const std::string & scheme )
	{
		return m_history->insert( object, type_name );
	}

void
ient_std_t::obj_finish(
	const void * object,
	const std::string * type_name )
	{}

void
ient_std_t::obj_complete(
	const void * object,
	const std::string * type_name )
	{
		m_history->erase( object, type_name );
	}

void
ient_std_t::unpack_type_name(
	std::string & type_name )
	{
		query_stream() >> type_name;
	}

oess_1::io::istream_t &
ient_std_t::query_stream()
	{
		return m_stream;
	}

bool
ient_std_t::check_optional_attr_existence(
	const std::string & attr_name )
	{
		oess_1::uchar_t flag;
		query_stream() >> flag;

		return ( opt_attr_present_flag == flag );
	}

iextension_t *
ient_std_t::extension_start()
	{
		std::string data;
		m_stream >> data;

		return new iextension_std_t( data );
	}

void
ient_std_t::extension_finish(
	iextension_t & extension )
	{}

//
// iextension_std_t
//

iextension_std_t::iextension_std_t(
	const std::string & data )
	:	m_data( data )
	,	m_stream( m_data )
	,	m_ient( m_stream )
	{}

iextension_std_t::~iextension_std_t()
	{}

bool
iextension_std_t::obj_start(
	const void * object,
	const std::string * type_name,
	const std::string & scheme )
	{
		return m_ient.obj_start( object, type_name, scheme );
	}

void
iextension_std_t::obj_finish(
	const void * object,
	const std::string * type_name )
	{
		m_ient.obj_finish( object, type_name );
	}

void
iextension_std_t::obj_complete(
	const void * object,
	const std::string * type_name )
	{
		m_ient.obj_finish( object, type_name );
	}

void
iextension_std_t::unpack_type_name(
	std::string & type_name )
	{
		m_ient.unpack_type_name( type_name );
	}

oess_1::io::istream_t &
iextension_std_t::query_stream()
	{
		return m_ient.query_stream();
	}

bool
iextension_std_t::check_optional_attr_existence(
	const std::string & attr_name )
	{
		return m_ient.check_optional_attr_existence( attr_name );
	}

iextension_t *
iextension_std_t::extension_start()
	{
		return m_ient.extension_start();
	}

void
iextension_std_t::extension_finish(
	iextension_t & extension )
	{
		m_ient.extension_finish( extension );
	}

bool
iextension_std_t::is_present() const
	{
		return ( !m_data.empty() );
	}

void
iextension_std_t::unpack_opaque( unknown_extension_t & ext )
	{
		ext = unknown_extension_t( m_data );
	}

//
// oent_std_t
//

oent_std_t::oent_std_t(
	oess_1::io::ostream_t & stream )
	:	m_stream( stream )
	,	m_history( new operation_history_t() )
	{}

oent_std_t::~oent_std_t()
	{}

bool
oent_std_t::obj_start(
	const void * object,
	const std::string * type_name,
	const std::string & scheme )
	{
		return m_history->insert( object, type_name );
	}

void
oent_std_t::obj_finish(
	const void * object,
	const std::string * type_name )
	{}

void
oent_std_t::obj_complete(
	const void * object,
	const std::string * type_name )
	{
		m_history->erase( object, type_name );
	}

void
oent_std_t::pack_type_name(
	const std::string & type_name,
	const std::string & /*type_scheme*/ )
	{
		query_stream() << type_name;
	}

oess_1::io::ostream_t &
oent_std_t::query_stream()
	{
		return m_stream;
	}

bool
oent_std_t::set_optional_attr_existence(
	const std::string & attr_name,
	bool existence )
	{
		oess_1::uchar_t flag( existence ? opt_attr_present_flag :
			opt_attr_missing_flag );
		query_stream() << flag;

		return existence;
	}

oextension_t *
oent_std_t::extension_start()
	{
		return new oextension_std_t();
	}

void
oent_std_t::extension_finish(
	oextension_t & extension )
	{
		extension.pack( m_stream );
	}

//
// oextension_std_t
//

oextension_std_t::oextension_std_t()
	:	m_stream( m_data )
	,	m_oent( m_stream )
	{}

oextension_std_t::~oextension_std_t()
	{}

bool
oextension_std_t::obj_start(
	const void * object,
	const std::string * type_name,
	const std::string & scheme )
	{
		return m_oent.obj_start( object, type_name, scheme );
	}

void
oextension_std_t::obj_finish(
	const void * object,
	const std::string * type_name )
	{
		m_oent.obj_finish( object, type_name );
	}

void
oextension_std_t::obj_complete(
	const void * object,
	const std::string * type_name )
	{
		m_oent.obj_complete( object, type_name );
	}

void
oextension_std_t::pack_type_name(
	const std::string & type_name,
	const std::string & type_scheme )
	{
		m_oent.pack_type_name( type_name, type_scheme );
	}

oess_1::io::ostream_t &
oextension_std_t::query_stream()
	{
		return m_oent.query_stream();
	}

bool
oextension_std_t::set_optional_attr_existence(
	const std::string & attr_name,
	bool existence )
	{
		return m_oent.set_optional_attr_existence( attr_name, existence );
	}

oextension_t *
oextension_std_t::extension_start()
	{
		return m_oent.extension_start();
	}

void
oextension_std_t::extension_finish(
	oextension_t & extension )
	{
		m_oent.extension_finish( extension );
	}

void
oextension_std_t::pack( oess_1::io::ostream_t & to ) const
	{
		to << m_data;
	}

void
oextension_std_t::pack_opaque(
	const unknown_extension_t & ext )
	{
		m_data = ext.get();
	}

} /* namespace stdsn */

} /* namespace oess_1 */
