/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file
	\brief     
		.
*/

#if !defined( _OESS_1__STDSN__ENT_STD_HPP_ )
#define _OESS_1__STDSN__ENT_STD_HPP_

#include <auto_ptr_3/h/obj_ptr.hpp>

#include <oess_1/io/h/bstring_buf.hpp>

#include <oess_1/stdsn/h/ent.hpp>

namespace oess_1 {

namespace stdsn {

class	operation_history_t;

/*!
	\since v.1.2.0
	\brief    
		 .
*/
const oess_1::uchar_t	opt_attr_missing_flag = 0;

/*!
	\since v.1.2.0
	\brief    
		 .
*/
const oess_1::uchar_t	opt_attr_present_flag = 1;

//
// ient_std_t
//

/*!
	\brief   ient_t  ,  
	    .
*/
class	OESS_1__STDSN__TYPE	ient_std_t :
	public ient_t
{
	public :
		ient_std_t(
			oess_1::io::istream_t & stream );
		virtual ~ient_std_t();

		virtual bool
		obj_start(
			const void * object,
			const std::string * type_name,
			const std::string & scheme );

		virtual void
		obj_finish(
			const void * object,
			const std::string * type_name );

		virtual void
		obj_complete(
			const void * object,
			const std::string * type_name );

		//!        .
		virtual void
		unpack_type_name(
			std::string & type_name );

		virtual oess_1::io::istream_t &
		query_stream();

		/*!
			   
			  
			oess_1::uchar_t  
			oess_1::stdsn::opt_attr_missing_flag 
			oess_1::stdsn::opt_attr_existence_flag.
		*/
		virtual bool
		check_optional_attr_existence(
			const std::string & attr_name );

		/*!
			      
			    (std::string).   
			       
			    iextension_std_t.
		*/
		virtual iextension_t *
		extension_start();

		/*!
			  .
		*/
		virtual void
		extension_finish(
			iextension_t & extension );

	private :
		oess_1::io::istream_t &	m_stream;

		//!   ,     .
		/*!
			\since v.1.2.0
		*/
		auto_ptr_3::obj_ptr_t< operation_history_t >	m_history;

	private :
	//! \name   .
	//! \{
		ient_std_t(
			const ient_std_t & );
		ient_std_t &
		operator=(
			const ient_std_t & );
	//! \}
};

//
// iextension_std_t
//

/*!
	\since v.1.2.0
	\brief   iextension_t  ,  
	    .

	 ,        
	 ient_std_t  iextension_std_t, 
	iextension_no_marker_t    ient_no_marker_t
	     .
*/
class	OESS_1__STDSN__TYPE	iextension_std_t
:	public iextension_t
{
	public :
		iextension_std_t(
			//!   .
			const std::string & data );
		virtual ~iextension_std_t();

	//! \name   .
	//! \{
		virtual bool
		obj_start(
			const void * object,
			const std::string * type_name,
			const std::string & scheme );

		virtual void
		obj_finish(
			const void * object,
			const std::string * type_name );

		virtual void
		obj_complete(
			const void * object,
			const std::string * type_name );

		virtual void
		unpack_type_name(
			std::string & type_name );

		virtual oess_1::io::istream_t &
		query_stream();

		virtual bool
		check_optional_attr_existence(
			const std::string & attr_name );

		virtual iextension_t *
		extension_start();

		virtual void
		extension_finish(
			iextension_t & extension );
	//! \}

		/*!
			\return \a true,      
				.
		*/
		virtual bool
		is_present() const;

		virtual void
		unpack_opaque( unknown_extension_t & ext );

	private :
		//!   .
		const std::string	m_data;
		//!    .
		oess_1::io::ibstring_t	m_stream;
		//! ,    
		//!    .
		ient_std_t	m_ient;

	private :
	//! \name   .
	//! \{
		iextension_std_t(
			const iextension_std_t & );
		iextension_std_t &
		operator=(
			const iextension_std_t & );
	//! \}
};

//
// oent_std_t
//

/*!
	\brief   oent_t  ,  
	    .
*/
class	OESS_1__STDSN__TYPE	oent_std_t :
	public oent_t
{
	public :
		oent_std_t( oess_1::io::ostream_t & stream );
		virtual ~oent_std_t();

		virtual bool
		obj_start(
			const void * object,
			const std::string * type_name,
			const std::string & scheme );

		virtual void
		obj_finish(
			const void * object,
			const std::string * type_name );

		virtual void
		obj_complete(
			const void * object,
			const std::string * type_name );

		//!        .
		virtual void
		pack_type_name(
			const std::string & type_name,
			const std::string & /*type_scheme*/ );

		virtual oess_1::io::ostream_t &
		query_stream();

		/*!
			   
			  
			oess_1::uchar_t  
			oess_1::stdsn::opt_attr_missing_flag 
			oess_1::stdsn::opt_attr_existence_flag.
		*/
		virtual bool
		set_optional_attr_existence(
			const std::string & attr_name,
			bool existence );

		/*!
			\return   
				oextension_std_t.
		*/
		virtual oextension_t *
		extension_start();

		/*!
			     
			  \a extension   pack().
		*/
		virtual void
		extension_finish(
			oextension_t & extension );

	private :
		oess_1::io::ostream_t &	m_stream;

		//!   ,     .
		/*!
			\since v.1.2.0
		*/
		auto_ptr_3::obj_ptr_t< operation_history_t >	m_history;

	private :
	//! \name   .
	//! \{
		oent_std_t(
			const oent_std_t & );
		oent_std_t &
		operator=(
			const oent_std_t & );
	//! \}
};

//
// oextension_std_t
//

/*!
	\brief   %oextension_t  ,  
	    .

	 ,        
	 oent_std_t  oextension_std_t, 
	oextension_no_marker_t    oent_no_marker_t
	     .
*/
class	OESS_1__STDSN__TYPE	oextension_std_t
:	public oextension_t
{
	public :
		oextension_std_t();
		virtual ~oextension_std_t();

	//! \name  .
	//! \{
		virtual bool
		obj_start(
			const void * object,
			const std::string * type_name,
			const std::string & scheme );

		virtual void
		obj_finish(
			const void * object,
			const std::string * type_name );

		virtual void
		obj_complete(
			const void * object,
			const std::string * type_name );

		virtual void
		pack_type_name(
			const std::string & type_name,
			const std::string & /*type_scheme*/ );

		virtual oess_1::io::ostream_t &
		query_stream();

		virtual bool
		set_optional_attr_existence(
			const std::string & attr_name,
			bool existence );

		virtual oextension_t *
		extension_start();

		virtual void
		extension_finish(
			oextension_t & extension );

		/*!
			       (std::string).
		*/
		virtual void
		pack( oess_1::io::ostream_t & to ) const;

		virtual void
		pack_opaque( const unknown_extension_t & ext );

	private :
		//!   .
		std::string	m_data;
		//!     
		//!     %m_data.
		oess_1::io::obstring_t	m_stream;
		//! ,    .
		oent_std_t	m_oent;

	private :
	//! \name   .
	//! \{
		oextension_std_t(
			const oextension_std_t & );
		oextension_std_t &
		operator=(
			const oextension_std_t & );
	//! \}
};

} /* namespace stdsn */

} /* namespace oess_1 */

#endif

