/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/tlv/h/default_processors.hpp
	\brief  
	%oess_1::tlv::tag_processor_t,
	%oess_1::tlv::length_processor_t  
	.
*/

#if !defined( _OESS_1__TLV__DEFAULT_PROCESSORS_HPP_ )
#define _OESS_1__TLV__DEFAULT_PROCESSORS_HPP_

#include <oess_1/tlv/h/declspec.hpp>

#include <oess_1/io/h/bin_data_size.hpp>

#include <oess_1/tlv/h/base.hpp>

namespace oess_1
{

namespace tlv
{

//
// default_tag_processor_t
//

//!   tag_processor_t.
/*!
	  ,   Tag 
	    .
*/
template< class Tag_type = default_tag_type_t >
class	default_tag_processor_t :
	public tag_processor_t
{
	public :
		//!   default_tag_processor_t< Tag_type >.
		typedef default_tag_processor_t< Tag_type > self_t;

	private :
		//!   Tag.
		Tag_type	m_id;

	public :
		//!  -.
		default_tag_processor_t()
			:
				m_id()
		{
		}

		//!      Tag.
		default_tag_processor_t( Tag_type id )
			:
				m_id( id )
		{
		}
		//!  .
		default_tag_processor_t( const self_t & o )
			:
				m_id( o.m_id )
		{
		}
		virtual ~default_tag_processor_t()
		{
		}

		//!  .
		self_t &
		operator=( const self_t & o )
		{
			m_id = o.m_id;
			return this;
		}

		//!     Tag_type.
		virtual size_t
		tlv_tag_size() const
		{
			return oess_1::io::bin_data_size_t< Tag_type >::image_size;
		}

		//!    Tag_type  \a s.
		/*!
			   
			 default_tag_processor_t.
		*/
		virtual void
		tlv_unpack_tag(
			//! ,     
			//!    Tag.
			oess_1::io::ibinstream_t & s )
		{
			s >> m_id;
		}

		//!    Tag_type  \a s.
		/*!
			   
			default_tag_type_t.
		*/
		virtual void
		tlv_pack_tag(
			//! ,     
			//!    Tag.
			oess_1::io::obinstream_t & s ) const
		{
			s << m_id;
		}

		//!   .
		virtual default_tag_type_t
		query_tag() const
		{
			return (default_tag_type_t) m_id;
		}

		//!   .
		virtual void
		set_tag( const default_tag_type_t & value )
		{
			m_id = value;
		}
};

//!   tlv_unpack_tag  \a o.
inline oess_1::io::ibinstream_t &
operator>>(
	//! ,     
	//!    Tag.
	oess_1::io::ibinstream_t & s,
	//!   .
	tag_processor_t & o )
{
	o.tlv_unpack_tag( s );
	return s;
}

//!   tlv_pack_tag  \a o.
inline oess_1::io::obinstream_t &
operator<<(
	//! ,     
	//!    Tag.
	oess_1::io::obinstream_t & s,
	//!   .
	const tag_processor_t & o )
{
	o.tlv_pack_tag( s );
	return s;
}

//
// default_length_processor_t
//

//!   length_processor_t.
/*!
	  ,   Length 
	    .
*/
template< class Length_type >
class	default_length_processor_t :
	public length_processor_t
{
	public :
		default_length_processor_t()
		{
		}
		
		virtual ~default_length_processor_t()
		{
		}

		//!    
		//! Length_type
		virtual size_t
		tlv_length_size(
			//!   TLV.
			size_t value_length ) const
		{
			return oess_1::io::bin_data_size_t< Length_type >::image_size;
		}

		//!    Length_type  \a s.
		virtual void
		tlv_unpack_length(
			//! ,     
			//!    Length.
			oess_1::io::ibinstream_t & s,
			//!    Length
			size_t & value_length )
		{
			Length_type l;
			s >> l;
			value_length = l;
		}

		//!    Length  \a s.
		virtual void
		tlv_pack_length(
			//! ,     
			//!    Length.
			oess_1::io::obinstream_t & s,
			//!   Length.
			size_t value_length ) const
		{
			Length_type l = value_length;
			s << l;
		}
};

} /* namespace tlv */

} /* namespace oess_1 */

#endif

