/*

Copyright (c) 2002-2005, Yauheni Akhotnikau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

- The name of the author may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

/*!
	\file oess_1/tlv/h/tagged_scalar_wrapper.hpp
	\brief  %oess_1::tlv::tagged_scalar_wrapper_t.
*/

#if !defined( _OESS_1__TLV__TAGGED_SCALAR_WRAPPER_HPP_ )
#define _OESS_1__TLV__TAGGED_SCALAR_WRAPPER_HPP_

#include <oess_1/tlv/h/declspec.hpp>

#include <string>

#include <oess_1/tlv/h/base.hpp>
#include <oess_1/tlv/h/default_processors.hpp>
#include <oess_1/tlv/h/errno.hpp>

namespace oess_1
{

namespace tlv
{

//
// tagged_scalar_wrapper_t
//

//! TLV      
//!    Tag.
/*!
//FIX ME: !
	\par    Serializer:
	-     Serializer::value_type_t;
	-   ,    Tag:
\code
Serializer::Serializer( tag_value );
\endcode.
   tag_value   Serializer
   oess_1::tlv::default_tag_type_t;
	-   ,   
Tag  Value:
\code
Serializer::Serializer( tag_value,
	const value_type_t & value );
\endcode.
   tag_value   Serializer
   oess_1::tlv::default_tag_type_t;
	-      :
\code
Serializer::Serializer( const Serializer & o );
Serializer &
Serializer::operator=( const Serializer & o );
\endcode
	-   :
\code
const value_type_t &
Serializer::query_value() const;
\endcode
	-   :
\code
void
Serializer::set_value( const value_type_t & value );
\endcode
*/
template< default_tag_type_t Tag_value,
	class Serializer >
class	tagged_scalar_wrapper_t :
	public tlv_base_t
{
	public :
		//!     Value.
		typedef typename Serializer::value_type_t
			value_type_t;

		//!   tagged_scalar_wrapper_t< Tag_value,
		//! Serializer >.
		typedef tagged_scalar_wrapper_t< Tag_value,
				Serializer >
			self_t;

	private :
		//!   TLV.
		Serializer	m_tlv;

	public :
		//!   .
		tagged_scalar_wrapper_t()
			:
				m_tlv( Tag_value )
		{
		}

		//!    Value.
		tagged_scalar_wrapper_t(
			const value_type_t & value )
			:
				m_tlv( Tag_value, value )
		{
		}

		//!  .
		tagged_scalar_wrapper_t(
			const self_t & o )
			:
				m_tlv( o.m_tlv )
		{
		}

		virtual ~tagged_scalar_wrapper_t()
		{
		}

		//!  .
		self_t &
		operator=( const self_t & o )
		{
			if( &o != this )
			{
				m_tlv = o.m_tlv;
			}
			return *this;
		}

		//!     Value.
		const value_type_t &
		query_value() const
		{
			return m_tlv.query_value();
		}

		//!    Value.
		void
		set_value(
			//!     Value.
			const value_type_t & value )
		{
			m_tlv.set_value( value );
		}

		//!    Tag.
		virtual default_tag_type_t
		tlv_tag() const
		{
			return Tag_value;
		}

		//!  ,  
		//!    TLV (  Tag, Length,
		//! Value).
		virtual size_t
		tlv_size() const
		{
			return m_tlv.tlv_size();
		}

		//!   Length  Value   .
		/*!
			\throw std::exception   .
		*/
		virtual void
		tlv_unpack(
			//! ,     
			//!    Length  Value.
			oess_1::io::ibinstream_t & s,
			//!    
			default_tag_type_t t)
		{
			m_tlv.tlv_unpack( s, t );
		}

		//!  TLV   .
		virtual void
		tlv_pack(
			//! ,     
			//!   TLV.
			oess_1::io::obinstream_t & s ) const
		{
			m_tlv.tlv_pack( s );
		}
};

} /* namespace tlv */

} /* namespace oess_1 */

#endif

